#!/usr/bin/wish

#################################################################################
# CADO4MI - CADO4MI: Computer Assisted Design of Oligonucleotide For MIcroarray #
#                                                                               #
# CADO4MI is a program to design oligonucleotide for microarrays or PCR.        #
#                                                                               #
# Copyright (C) 2006 Jean Muller (jean.muller@igbmc.u-strasbg.fr)               # 
#                                                                               #
# Please cite the following article:                                            #
#    Muller J et al                                                             #
#    Design and evaluation of Actichip,                                         #
#    a thematic microarray for the study of the actin cytoskeleton.             #
#    BMC Genomics 2007 Aug 29;8(1):294.                                         #
#                                                                               #
# This is part of CADO4MI source code.                                          #
#                                                                               #
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public License                   # 
# as published by the Free Software Foundation; either version 2                # 
# of the License, or (at your option) any later version.                        #
#                                                                               #
# This program is distributed in the hope that it will be useful,               # 
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
# GNU General Public License for more details.                                  #
#                                                                               #
# You should have received a copy of the GNU General Public License             #
# along with this program; if not, write to the Free Software                   #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                     #
#################################################################################

set ProcsAuDepart [info procs]

############################################
# Recherche des fichiers sources           #
# Positionnement des variables importantes #
############################################
proc ProgSourceDir {} {

    #Permet de determiner la racine des fichiers sources .tcl

    global ProgSourceDir
    if {[info exists ProgSourceDir] && [file exists $ProgSourceDir]} {return $ProgSourceDir}

    set Script [info script]

    if {$Script==""} {
	tk_messageBox -message "I cannot find the script directory"
	set ProgSourceDir "."
    } else {
	if {[catch {
	    while {[file exists $Script] && [file type $Script]=="link"} {
		set RepScript [file dirname $Script]
		if {[catch {set Script [file readlink $Script]} Message]} { break }
		if {[file pathtype $Script]=="relative"} {
		    while {[regexp {^\.\./} $Script]} {
			regsub {^\.\./} $Script "" Script
			set RepScript [file dirname $RepScript]
		    }
		    set Script "[file join $RepScript $Script]"
		}
	    }
	} Message]} {
	    #tk_messageBox -message "Crashes with message \n$Message"
	}
	set ProgSourceDir [file dirname $Script]
    }

    global InfoScript ; set InfoScript $Script
    return $ProgSourceDir
}

proc CADO4MI_PATH {What} {

    #Permet de chercher les chemins des programmes ou des banques importants pour CADO4MI
    #Inclut un test si l executable existe vraiment meme si il specifie dans le fichier

    global CADO4MI_PATH

    set PathFile "[file join "[ProgSourceDir]" "CADO4MI.path"]"

    if {[info exists CADO4MI_PATH($What)]}  {return [set CADO4MI_PATH($What)]}
    #if {[info exists CADO4MI_PATH(Loaded)]} {return "$What"}

    if {! [info exists CADO4MI_PATH(Loaded)] && [file exists $PathFile]} {
	set CADO4MI_PATH(Loaded) 1
	set F [open "$PathFile"]
	while {[gets $F Ligne]>=0} {
	    if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	    if {$Ligne == ""}                          {continue}
	    if {[regexp {^\#} $Ligne]}                 {continue}
	    
	    set  Ligne [split $Ligne "\t"]
	    if { [info exists W]} {unset W; unset Where}
	    set W     ""
	    set Where ""
	    set W     [lindex $Ligne 0]
	    set Where [lindex $Ligne 1]
	    if {![info exists W]} {continue}
	    
	    #1- Serie de tests pour verifier que les repertoires existent ou que les programmes s executent
	    set W_Exists 0
	    set W_Exec   0
	    set Commande ""
	    set Commande $Where

	    #1-1 Test si c un executable
	    if {[catch {eval exec $Commande} Message]} {
		if {[regexp -nocase "invalid command name" $Message] || [regexp -nocase "no such file or directory" $Message]} {
		    set W_Exec 0
		} else { 
		    set W_Exec 1
		}
	    } else {
		set     W_Exec 1
	    }

	    #1-2 Test si c un repertoire ou un fichier existant
	    if {[file exists $Commande]} {
		set W_Exists 1
	    } else {
		set W_Exists 0
	    } 
	    
	    #1-3 Decision Finale
	    if {$W_Exec || $W_Exists} {
		set CADO4MI_PATH($W) $Where  
	    } else {
		#set CADO4MI_PATH($W) ""
	    }
	}
	close $F
    }
    
    #2- On test egalement notre executable ou dir (What) qui peut ne pas etre dans la liste chargee
    set W_Exists 0
    set W_Exec   0
    if {      [info exists CADO4MI_PATH($What)]} {
	set Commande  [set CADO4MI_PATH($What)]
    } elseif {[info exists CADO4MI_PATH(Loaded)]} {
	set Commande  $What
    } else {
	set Commande  $What
    }

    #2-1 Test si c un executable
    if {[catch {eval exec $Commande} Message]} {
	if {[regexp -nocase "invalid command name" $Message] || [regexp -nocase "no such file or directory" $Message]} {
	    set W_Exec 0
	} else { 
	    set W_Exec 1
	}
    } else {
	set     W_Exec 1
    }

    #2-2 Test si c un repertoire ou un fichier existant
    if {[file exists $Commande]} {
	set W_Exists 1
    } else {
	set W_Exists 0
    }

    #2-3 Decision Finale
    if {$W_Exec || $W_Exists} {
	set CADO4MI_PATH($What) $Commande
    } else {
	set CADO4MI_PATH($What) ""
    }
    
    return [CADO4MI_PATH $What]
}

########################################
# Definition des variables importantes #
########################################
global GscopeDir ; set GscopeDir "[ProgSourceDir]"
global env
global RepertoireDuGenome
if {! [info exists env(REPERTOIREDUGENOME)]} {
    set     RepertoireDuGenome [CADO4MI_PATH Data_DIR]
    if {   $RepertoireDuGenome == "" || ! [file exists $RepertoireDuGenome]} {
	set RepertoireDuGenome $ProgSourceDir
    }
}

proc HOMEDIR {} {

    #Home directory du user en cours

    global HOMEDIR
    if {[info exists HOMEDIR] && [file exists $HOMEDIR]} {return $HOMEDIR}	
    set     HOMEDIR ""
    if {[info exists env(HOME)]} {
	set HOMEDIR [file join [set env(HOME)]]
    }
    if {   $HOMEDIR == ""} {
	set HOMEDIR [file join [RepertoireDuGenome]]
    }
    return $HOMEDIR
}

#Source des procedures de CADO4MI
#################################
set GscopeJean "[ProgSourceDir]"
source "[file join $GscopeJean CADO4MI_Source.tcl]"
source "[file join $GscopeJean CADO4MI_Tools1.tcl]"
source "[file join $GscopeJean CADO4MI_ActiChip.tcl]"

if {[RepertoireDuGenome] == ""} {
    RepertoireDuGenome $ProgSourceDir
}

#Definition des fontes de CADO4MI
#################################
CADO4MI_Font

proc EtVoilaToutesLesProcs {} {
    
    #Permet d extraire toutes les procedures sourcees par CADO4MI et de les ecrire dans un fichier .tcl
    #But: faire une version compilable (ex: pour windows)

    global ProcsAuDepart
    
    #set ListeProc [lsort [info procs]]
    set ListeProc [info procs]
    set f [open "AllProcs_CADO4MI.tcl" w]
    foreach e $ListeProc {
	if {[lsearch $ProcsAuDepart $e] != -1} {continue}
	set a [info args $e]
	set b [info body $e]
	set sar ""
	foreach g $a {
	    if {[info default $e $g Def]} {
		if {$Def == ""} {set Def "\"\""}
		if {[regexp " " $Def]} {set Def "\"$Def\""}
		append sar "{$g $Def} "
	    } else {
		append sar "$g "
	    }
	}
	set sar [string trim $sar]
	puts $f "proc $e {$sar} {"
	puts $f $b
	puts $f "}"
	puts $f ""
    }
    close $f
    
    return ""
}

if {[lindex $argv 0] == "ToutesLesProcs"} {
    EtVoilaToutesLesProcs
    exit
}
###########################################
################# CADO4MI #################
###########################################
if {$argv!={}} {
    if {[lindex $argv 0] == "puts"} {
	puts [eval [lrange $argv 1 end]]
	exit
    }
    CADO4MI_Main $argv    
} else {
    CADO4MI_Main
}
################### FIN ###################
