#################################################################################
# CADO4MI - CADO4MI: Computer Assisted Design of Oligonucleotide For MIcroarray #
#                                                                               #
# CADO4MI is a program to design oligonucleotide for microarrays or PCR.        #
#                                                                               #
# Copyright (C) 2006 Jean Muller (jean.muller@igbmc.u-strasbg.fr)               # 
#                                                                               #
# Please cite the following article:                                            #
#    Muller J et al                                                             #
#    Design and evaluation of Actichip,                                         #
#    a thematic microarray for the study of the actin cytoskeleton.             #
#    BMC Genomics 2007 Aug 29;8(1):294.                                         #
#                                                                               #
# This is part of CADO4MI source code.                                          #
#                                                                               #
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public License                   # 
# as published by the Free Software Foundation; either version 2                # 
# of the License, or (at your option) any later version.                        #
#                                                                               #
# This program is distributed in the hope that it will be useful,               # 
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
# GNU General Public License for more details.                                  #
#                                                                               #
# You should have received a copy of the GNU General Public License             #
# along with this program; if not, write to the Free Software                   #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                     #
#################################################################################

proc BlastTools {} {
    if {[winfo children .] != ""} {
	set       w [NomDe blasttools]
	toplevel $w
    } else {
	set w "."
	$w config -bd 3 -relief flat
    }
    wm title    $w "BlastTools"
    wm iconname $w "BlastTools"
    
    if {$w == "."} {set w ""}
    set    BoutonDrawBlast                 "${w}.boutondrawblast"
    set    BoutonClose                     "${w}.boutonclose"
    button $BoutonDrawBlast                -borderwidth 1 -text "Draw Blast"
    button $BoutonClose                    -borderwidth 1 -text "CLOSE"
    pack   $BoutonDrawBlast                -side top -fill both -expand 1
    pack   $BoutonClose                    -side top -fill both -expand 1
    if {$w == ""} {set w "."}

    $BoutonDrawBlast  configure -command {DrawBlast}
    $BoutonClose      configure -command "destroy $w;return"
}
proc DrawBlast {{FichierBlast ""} {ExpectMax 1.e200} {NbSubjectMax 1.e200} {IdMin 0.0} {TailleMaxDUneRegionNonAlignee 1.e200}} {
    global TabOptionsDeDrawBlast

    if {[winfo children .] != ""} {
	set       w [NomDe drawblast]
	toplevel $w
	wm title    $w "DrawBlast"
	wm iconname $w "DrawBlast"
    } else {
	set w "."
	$w config -bd 3 -relief flat
	wm title    $w "DrawBlast"
	wm iconname $w "DrawBlast"
    }

    AskOptionsDeDrawBlast set $w ExpectMax                     $ExpectMax
    AskOptionsDeDrawBlast set $w NbSubjectMax                  $NbSubjectMax
    AskOptionsDeDrawBlast set $w IdMin                         $IdMin
    AskOptionsDeDrawBlast set $w FichierBlast                  $FichierBlast
    AskOptionsDeDrawBlast set $w BanqueId                      ""
    AskOptionsDeDrawBlast set $w TailleMaxDUneRegionNonAlignee $TailleMaxDUneRegionNonAlignee

    if {$w == "."} {set w ""}
    set    FrameGeneral                    "${w}.frame"
    set       FramePanel                   "$FrameGeneral.framepanel"
    set          FrameFileBlast            "$FramePanel.framefileblast"
    set             ButtonFileBlast        "$FrameFileBlast.buttonfileblast"
    set             EntryFileBlast         "$FrameFileBlast.entryfileblast"
    set       FrameCanva                   "$FrameGeneral.framecanva"
    set          FrameCanvaMosaic          "$FrameCanva.framecanvamosaic"  
    set          FrameCanvaMapping         "$FrameCanva.framecanvamapping"
    set    BoutonLaunch                    "${w}.boutonlaunch"
    set    BoutonFlatFile                  "${w}.boutonflatfile"
    set    BoutonOptions                   "${w}.boutonoptions"
    set    BoutonReset                     "${w}.boutonreset"
    set    BoutonClose                     "${w}.boutonclose"
    if {$w == ""} {set w "."}
    #############################################################
    frame  $FrameGeneral
    frame     $FramePanel 
    frame        $FrameFileBlast
    button          $ButtonFileBlast       -borderwidth 1 -text "Blast File"
    entry           $EntryFileBlast        -borderwidth 1 -textvariable TabOptionsDeDrawBlast($w,FichierBlast)
    frame     $FrameCanva
    frame        $FrameCanvaMosaic
    frame        $FrameCanvaMapping
    button $BoutonLaunch                   -borderwidth 1 -text "LAUNCH"
    button $BoutonFlatFile                 -borderwidth 1 -text "FLAT FILE"
    button $BoutonOptions                  -borderwidth 1 -text "OPTIONS"
    button $BoutonReset                    -borderwidth 1 -text "RESET MEM"
    button $BoutonClose                    -borderwidth 1 -text "CLOSE"
    
    ######################################################################
    
    pack   $FrameGeneral                   -side top -fill both -expand 1
    pack      $FramePanel                  -side top -fill x -expand 0
    pack         $FrameFileBlast           -side top -fill x -expand 0
    pack            $ButtonFileBlast       -side top -fill x -expand 0
    pack            $EntryFileBlast        -side top -fill x -expand 0
    pack      $FrameCanva                  -fill both -side right -expand 1
    pack         $FrameCanvaMosaic         -fill both -side top   -expand 1
    pack         $FrameCanvaMapping        -fill both -side top   -expand 0
    pack   $BoutonLaunch                   -side left -fill x -expand 1
    pack   $BoutonFlatFile                 -side left -fill x -expand 0
    pack   $BoutonOptions                  -side left -fill x -expand 0
    pack   $BoutonReset                    -side left -fill x -expand 0
    pack   $BoutonClose                    -side left -fill x -expand 1
    
    set LargeurFenetre        [expr {[winfo screenwidth  .] * 2.0 / 3}]
    set HauteurFenetreMosaic  [expr {[winfo screenheight .] * 2.0 / 9}]
    set HauteurFenetreMapping [expr {[winfo screenheight .] * 4.0 / 9}]
    
    AskOptionsDeDrawBlast set $w LargeurFenetre        $LargeurFenetre
    AskOptionsDeDrawBlast set $w HauteurFenetreMosaic  $HauteurFenetreMosaic
    AskOptionsDeDrawBlast set $w FrameCanvaMosaic      $FrameCanvaMosaic
    AskOptionsDeDrawBlast set $w HauteurFenetreMapping $HauteurFenetreMapping
    AskOptionsDeDrawBlast set $w FrameCanvaMapping     $FrameCanvaMapping

    if {$FichierBlast != "" && [file exists $FichierBlast]} {
	DrawMosaic  $FichierBlast    $ExpectMax $NbSubjectMax $IdMin ""               $LargeurFenetre $HauteurFenetreMosaic  $FrameCanvaMosaic  $w
	DrawMapping $FichierBlast "" $ExpectMax $IdMin $TailleMaxDUneRegionNonAlignee $LargeurFenetre $HauteurFenetreMapping $FrameCanvaMapping $w
    } else {
	set K [Canva $LargeurFenetre $HauteurFenetreMosaic  $LargeurFenetre $HauteurFenetreMosaic  "NoY" "" "" $FrameCanvaMosaic]
	$K configure -background white
	set K [Canva $LargeurFenetre $HauteurFenetreMapping $LargeurFenetre $HauteurFenetreMapping "NoY" "" "" $FrameCanvaMapping]
	$K configure -background white
    }

    $ButtonFileBlast  configure -command "ChooseBlastFileDrawBlast $w"
    $BoutonLaunch     configure -command "LaunchOrApplyDrawBlast   $w"
    $BoutonFlatFile   configure -command "FlatFileDrawBlast        $w"
    $BoutonOptions    configure -command "DrawBlastOptions         $w"
    $BoutonReset      configure -command "ResetFenetreDrawBlast    $w"
    $BoutonClose      configure -command "CloseFenetreDrawBlast    $w"
    return $w
}
proc ChooseBlastFileDrawBlast {w} {
    AskOptionsDeDrawBlast set $w FichierBlast [DrawChooseFile "Choose the Blast File"]
    return
}
proc LaunchOrApplyDrawBlast {w} {
    set FichierBlast                  [AskOptionsDeDrawBlast ask $w FichierBlast                 ]
    set BanqueId                      [AskOptionsDeDrawBlast ask $w BanqueId                     ]

    set NbSubjectMax                  [AskOptionsDeDrawBlast ask $w NbSubjectMax                 ]
    set ExpectMax                     [AskOptionsDeDrawBlast ask $w ExpectMax                    ]
    set IdMin                         [AskOptionsDeDrawBlast ask $w IdMin                        ]
    set LargeurFenetre                [AskOptionsDeDrawBlast ask $w LargeurFenetre               ]
    set FrameCanvaMosaic              [AskOptionsDeDrawBlast ask $w FrameCanvaMosaic             ]
    set HauteurFenetreMosaic          [AskOptionsDeDrawBlast ask $w HauteurFenetreMosaic         ]
    set FrameCanvaMapping             [AskOptionsDeDrawBlast ask $w FrameCanvaMapping            ]
    set HauteurFenetreMapping         [AskOptionsDeDrawBlast ask $w HauteurFenetreMapping        ]
    set TailleMaxDUneRegionNonAlignee [AskOptionsDeDrawBlast ask $w TailleMaxDUneRegionNonAlignee]
    
    if {![file exists $FichierBlast]} {return}
    if {![AskBlast $FichierBlast Exists]} {AskBlast unset}
    
    DrawMosaic  $FichierBlast           $ExpectMax $NbSubjectMax $IdMin ""               $LargeurFenetre $HauteurFenetreMosaic  $FrameCanvaMosaic  $w
    DrawMapping $FichierBlast $BanqueId $ExpectMax $IdMin $TailleMaxDUneRegionNonAlignee $LargeurFenetre $HauteurFenetreMapping $FrameCanvaMapping $w
    return
}
proc FlatFileDrawBlast {w} {
    set FichierBlast [AskOptionsDeDrawBlast ask $w FichierBlast]
    if {![file exists $FichierBlast]} {return}
    if {![AskOptionsDeDrawBlast exists $w ListBox]} {
	AskOptionsDeDrawBlast set $w ListBox [DrawInfo Brut ""]
    }
    set ListBox [AskOptionsDeDrawBlast ask $w ListBox]
    if {![winfo exists $ListBox]                        } {
	AskOptionsDeDrawBlast set $w ListBox [DrawInfo Brut ""]
    }
    set ListBox [AskOptionsDeDrawBlast ask $w ListBox]

    $ListBox delete 0 end
    foreach Ligne [LesLignesDuFichier $FichierBlast] {$ListBox insert end $Ligne}
    return
}
proc ResetFenetreDrawBlast {w} {
    set FichierBlast [AskOptionsDeDrawBlast ask $w FichierBlast]
    
    if {![file exists $FichierBlast]} {
	AskBlast unset
    } else {
	AskBlast unsetfile $FichierBlast
    }
    
    #DrawInfoDeDrawBlast unset
    return
}
proc CloseFenetreDrawBlast {w} {
    if {$w == ""} {set w "."}
    AskBlast  unset
    destroy   $w
    AskOptionsDeDrawBlast unsetall
    if {[winfo children .] == ""} {exit}
    return
}
proc DrawBlastOptions {v} {
    global TabOptionsDeDrawBlast

    if {![AskOptionsDeDrawBlast exists $v FenetreBlastOptions]} {
	AskOptionsDeDrawBlast set $v FenetreBlastOptions [NomDe drawblastoptions]
    }
    set w [AskOptionsDeDrawBlast ask $v FenetreBlastOptions]

    if {![winfo exists $w]} {
	toplevel $w
	set    FrameThreshold        "$w.framethreshold"
	set       LabelExpectMax     "$FrameThreshold.labelexpectmax"
	set       EntryExpectMax     "$FrameThreshold.entryexpectmax"
	set       LabelIdMin         "$FrameThreshold.labelidmax"
	set       EntryIdMin         "$FrameThreshold.entryidmax"
	set       LabelNbSubjectMax  "$FrameThreshold.labelnbsubjectmax"
	set       EntryNbSubjectMax  "$FrameThreshold.entrynbsubjectmax"
	set       LabelCompression   "$FrameThreshold.labelcompression"
	set       EntryCompression   "$FrameThreshold.entrycompression"
	set    BoutonApply           "$w.boutonapply"
	set    BoutonDefaut          "$w.boutondefaut"
	set    BoutonClose           "$w.boutonclose"
	
	frame  $FrameThreshold
	label     $LabelExpectMax        -borderwidth 1 -text "Expect Max"
	entry     $EntryExpectMax        -borderwidth 1 -textvariable TabOptionsDeDrawBlast($v,ExpectMax)
	label     $LabelIdMin            -borderwidth 1 -text "Id Min"
	entry     $EntryIdMin            -borderwidth 1 -textvariable TabOptionsDeDrawBlast($v,IdMin)
	label     $LabelNbSubjectMax     -borderwidth 1 -text "Nb Subject Max"
	entry     $EntryNbSubjectMax     -borderwidth 1 -textvariable TabOptionsDeDrawBlast($v,NbSubjectMax)
	label     $LabelCompression      -borderwidth 1 -text "Compression"
	entry     $EntryCompression      -borderwidth 1 -textvariable TabOptionsDeDrawBlast($v,TailleMaxDUneRegionNonAlignee)
	button $BoutonApply              -borderwidth 1 -text "APPLY"
	button $BoutonDefaut             -borderwidth 1 -text "DEFAULT"
	button $BoutonClose              -borderwidth 1 -text "CLOSE"
	
	pack   $FrameThreshold           -side top -fill both
	grid      $LabelExpectMax        -row 0 -column 0 -rowspan 1 -columnspan 1 -sticky nsew
	grid      $EntryExpectMax        -row 0 -column 1 -rowspan 1 -columnspan 1 -sticky nsew
	grid      $LabelIdMin            -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky nsew
	grid      $EntryIdMin            -row 1 -column 1 -rowspan 1 -columnspan 1 -sticky nsew
	grid      $LabelNbSubjectMax     -row 2 -column 0 -rowspan 1 -columnspan 1 -sticky nsew
	grid      $EntryNbSubjectMax     -row 2 -column 1 -rowspan 1 -columnspan 1 -sticky nsew
	grid      $LabelCompression      -row 3 -column 0 -rowspan 1 -columnspan 1 -sticky nsew
	grid      $EntryCompression      -row 3 -column 1 -rowspan 1 -columnspan 1 -sticky nsew
	pack   $BoutonApply              -side left -fill x -expand 1
	pack   $BoutonDefaut             -side left -fill x -expand 1
	pack   $BoutonClose              -side left -fill x -expand 1
	
	$BoutonApply    configure -command "LaunchOrApplyDrawBlast $v"
	$BoutonDefaut   configure -command "DefautOptionsDrawBlast $v"
	$BoutonClose    configure -command "destroy   $w"
    }
}
proc DefautOptionsDrawBlast {w} {
    AskOptionsDeDrawBlast set $w NbSubjectMax                  1.e200
    AskOptionsDeDrawBlast set $w ExpectMax                     1.e200
    AskOptionsDeDrawBlast set $w IdMin                         0.0
    AskOptionsDeDrawBlast set $w TailleMaxDUneRegionNonAlignee 1.e200
    return
}
proc DrawInfoDeDrawBlast {w {Kcanva ""} {PageInfoOuAliouToutLAliouToutLeBlast "Ali"}} {
    if {![AskOptionsDeDrawBlast exists LesBIdStockeAvecLeurPageInfo]} {
	AskOptionsDeDrawBlast set LesBIdStockeAvecLeurPageInfo {}
    }

    if {[string equal -nocase $w "unset"]} {
	foreach BId [AskOptionsDeDrawBlast ask LesBIdStockeAvecLeurPageInfo] {
	    AskOptionsDeDrawBlast unset $BId PageInfo
	}
	AskOptionsDeDrawBlast set LesBIdStockeAvecLeurPageInfo {}
	return
    }

    set Id           [$Kcanva  find withtag current]
    set MesTags      [$Kcanva  gettags  $Id]
    set FichierBlast [lindex   $MesTags 1  ]
    set BId          [lindex   $MesTags 2  ]
    set Segment      [lindex   $MesTags 3  ]

    if {$PageInfoOuAliouToutLAliouToutLeBlast == "ToutLeBlast"} {
	set Info [ContenuDuFichier $FichierBlast]
    }
    if {$PageInfoOuAliouToutLAliouToutLeBlast == "Ali"     } {
	if {$Segment == "All"} {
	    set Info [AskBlast $FichierBlast BanqueId $BId SubjectBrut]
	} else {
	    set Info [AskBlast $FichierBlast BanqueId $BId Segment $Segment AlignBrut]
	}
    }
    if {$PageInfoOuAliouToutLAliouToutLeBlast == "ToutLAli"} {
	set Info [AskBlast $FichierBlast BanqueId $BId SubjectBrut]
    }
    if {$PageInfoOuAliouToutLAliouToutLeBlast == "PageInfo"} {
	if {![info exists TabOptionsDeDrawBlast($BId,PageInfo)]} {
	    AskOptionsDeDrawBlast append LesBIdStockeAvecLeurPageInfo $BId
	    AskOptionsDeDrawBlast set    $BId PageInfo                [PageInfoLaTotale $BId "unknown"]
	}
	set Info [AskOptionsDeDrawBlast ask    $BId PageInfo]
    }
    if {$Info == ""} {return}
    
    DrawInfo Brut $Info

    return
    if {![AskOptionsDeDrawBlast exists $w ListBox]} {
	AskOptionsDeDrawBlast set $w ListBox [DrawInfo Brut ""]
    }
    set ListBox [AskOptionsDeDrawBlast ask $w ListBox]
    if {![winfo exists $ListBox]} {
	AskOptionsDeDrawBlast set $w ListBox [DrawInfo Brut ""]
    }
    set ListBox [AskOptionsDeDrawBlast ask $w ListBox]

    $ListBox delete 0 end
    foreach Ligne [split $Info "\n"] {$ListBox insert end $Ligne}
}
proc DrawMappingPourDrawBlast {w K} {
    set Id           [$K find withtag current]
    set MesTags      [$K gettags $Id]
    set BId          [lindex $MesTags 2]

    set FichierBlast                  [AskOptionsDeDrawBlast ask $w FichierBlast                 ]
    set ExpectMax                     [AskOptionsDeDrawBlast ask $w ExpectMax                    ]
    set IdMin                         [AskOptionsDeDrawBlast ask $w IdMin                        ]
    set TailleMaxDUneRegionNonAlignee [AskOptionsDeDrawBlast ask $w TailleMaxDUneRegionNonAlignee]
    set LargeurFenetre                [AskOptionsDeDrawBlast ask $w LargeurFenetre               ]
    set FrameCanvaMapping             [AskOptionsDeDrawBlast ask $w FrameCanvaMapping            ]
    set HauteurFenetreMapping         [AskOptionsDeDrawBlast ask $w HauteurFenetreMapping        ]

    AskOptionsDeDrawBlast set $w BanqueId $BId

    if {$FichierBlast == ""} {return}

    DrawMapping $FichierBlast $BId $ExpectMax $IdMin $TailleMaxDUneRegionNonAlignee $LargeurFenetre $HauteurFenetreMapping $FrameCanvaMapping $w
}
proc AskOptionsDeDrawBlast args {
    global TabOptionsDeDrawBlast

    set FirstArg [lindex $args 0]
    if {[string equal $FirstArg "unsetall"]} {
	if {[info exists TabOptionsDeDrawBlast]} {unset TabOptionsDeDrawBlast}
	return
    }

    if {[string equal $FirstArg "set"]} {
	set Qui [join [lrange $args 1 end-1] ","]
	set Quoi      [lindex $args   end  ]
	set TabOptionsDeDrawBlast($Qui) $Quoi
	return
    }
    if {[string equal $FirstArg "append"]} {
	set Qui [join [lrange $args 1 end-1] ","]
	set Quoi      [lindex $args   end  ]
	if {![info exists TabOptionsDeDrawBlast($Qui)]} {set TabOptionsDeDrawBlast($Qui) {}}
	lappend TabOptionsDeDrawBlast($Qui) $Quoi
	return
    }

    set Qui [join [lrange $args 1 end] ","]
    if {[string equal $FirstArg "unset"]} {
	if {![info exists TabOptionsDeDrawBlast($Qui)]} {return}
	unset TabOptionsDeDrawBlast($Qui)
	return
    }
    if {[string equal $FirstArg "exists"]} {
	if {![info exists TabOptionsDeDrawBlast($Qui)]} {return 0}
	return 1
    }
    if {[string equal $FirstArg "ask"]} {
	if {![info exists TabOptionsDeDrawBlast($Qui)]} {return}
	return [set TabOptionsDeDrawBlast($Qui)]
    }
    return
}
proc DrawMosaic {FichierBlast {ExpectMax 1e200} {NbSubjectMax 1e200} {IdMin 0.0} {LesBanqueId ""} {LargeurFenetre ""} {HauteurFenetre ""} {KCanva ""} {w ""}} {

    if {![file exists $FichierBlast]} {return}
    set TailleQ     [AskBlast $FichierBlast Taille]
    set TypeDeBlast [AskBlast $FichierBlast TypeDeBlast]
    set Query       [AskBlast $FichierBlast Query]
    
    if {$TailleQ == ""} {return}

    if {$LargeurFenetre == ""} {set LargeurFenetre     [expr {[winfo screenheight .] * 2.0 / 3}]}
    if {$HauteurFenetre == ""} {set HauteurFenetre     [expr {[winfo screenheight .] * 2.0 / 3}]}
    set LargeurMax       $LargeurFenetre
    set HauteurMax       $HauteurFenetre
    set BordX            [expr {$LargeurFenetre * 1.0 / 4}]
    set BordY            [expr {$HauteurFenetre * 1.0 / 10}]
    if {$BordY < 30} {set BordY 40}

    set EcartY           10.0
    set HBox             25.0
    set HSmallBox        15.0
    set LargeurAffichage [expr {$LargeurFenetre - 2 * $BordX}]
    set FacteurX         [expr {$LargeurAffichage * 1.0 / $TailleQ}]

    set LaFrequenceDesPetitesRegionsDuBlast [LaFrequenceDesPetitesRegionsDeLaQueryDuBlast $FichierBlast $ExpectMax $NbSubjectMax $IdMin "No" $LesBanqueId]
    if {$LaFrequenceDesPetitesRegionsDuBlast == {}} {
	Canva $LargeurFenetre $HauteurFenetre $LargeurFenetre $HauteurFenetre "NoY" "" "" $KCanva
	$KCanva configure -background white
	return
    }
    set LesFormesAPlacer {}

    lappend LesFormesAPlacer [list text  [expr {$LargeurFenetre*1.0/2}] [expr {$BordY*1.0/3}] "center" "QUERY : [string range $Query end-30 end]"]
    lappend LesFormesAPlacer [list text  [expr {$LargeurFenetre*1.0/2}] [expr {$BordY*2.0/3}] "center" "BLAST : $TypeDeBlast"                    ]

    set     YMin             $BordY
    set     XMin             $BordX
    set     YMax             [expr {$YMin + $HBox}]
    set     XMax             [expr {$XMin + $LargeurAffichage}]
    set     YText            [expr {$YMin + ($YMax - $YMin)/2}]

    lappend LesFormesAPlacer [list rectangle $XMin           $YMin $XMax $YMax "lightgrey" "black" 3]
    lappend LesFormesAPlacer [list text      [expr {$XMin-1}]  $YText "e"  1                                     ]
    lappend LesFormesAPlacer [list text      [expr {$XMax+1}]  $YText "w"  $TailleQ                              ]
    lappend LesFormesAPlacer [list text      $LargeurFenetre $YMax  "ne" [format "%10s %10s" "Length" "Expect"]]
    
    foreach LaFrequenceDeLaRegion $LaFrequenceDesPetitesRegionsDuBlast {
	set Deb     [lindex $LaFrequenceDeLaRegion 0]
	set Fin     [lindex $LaFrequenceDeLaRegion 1]
	set XMin    [expr {($Deb-1) * $FacteurX + $BordX}]
	set XMax    [expr {$Fin     * $FacteurX + $BordX}]

	set Freq    [lindex $LaFrequenceDeLaRegion 2    ]
	set Couleur [ColoriageDeLaFrequenceDeDrawBlast        $Freq]
	lappend LesFormesAPlacer [list rectangle $XMin $YMin $XMax $YMax $Couleur $Couleur 1]
    }
    set YMinDeRef [expr {$YMax + $EcartY}]

    foreach BId $LesBanqueId {set TabSelect($BId) 1}

    set k 0
    set NbSubject [AskBlast $FichierBlast NbSubject]
    for {set i 1;set NiveauMinDeRef 1} {$i <= $NbSubject} {incr i;incr NiveauMinDeRef} {
	set BId         [AskBlast $FichierBlast Subject  $i              ]
	set NbSegment   [AskBlast $FichierBlast BanqueId $BId NbSegment  ]
	set TailleS     [AskBlast $FichierBlast BanqueId $BId Taille     ]

	if {$LesBanqueId != "" && ![info exists TabSelect($BId)]} {incr NiveauMinDeRef -1;continue}

	set YLigne [expr {$YMinDeRef + ($NiveauMinDeRef-1) * ($HSmallBox + $EcartY) + $EcartY * 1.0 / 2}]

	if {[info exists TabNiveau]} {unset TabNiveau}

	set LesFormesAPlacerDuSubject {}
	set NiveauMaxDuSubject $NiveauMinDeRef
	for {set j 1} {$j <= $NbSegment} {incr j} {
	    set Expect [AskBlast $FichierBlast BanqueId $BId Segment $j Expect]
	    if {$ExpectMax  < $Expect} {continue}

	    set Identities [AskBlast $FichierBlast BanqueId $BId Segment $j Identities]
	    scan $Identities "%d/%d" n nTot
	    set Id [expr {$n*1.0/$nTot}]
	    if {$Id < $IdMin} {continue}

	    set ColorFill  [ColoriageDeLIdDeDrawBlast $Id]
	    
	    set TypeRegion [AskBlast $FichierBlast BanqueId $BId Segment $j Orientation]
	    if {$TypeRegion == "" && [regexp "BLASTP" $TypeDeBlast]} {set TypeRegion "Prot / Prot"}
	    set ColorOutLine [ColoriageDuMappingDeDrawBlast $TypeRegion]

	    set DQ [Mini [AskBlast $FichierBlast BanqueId $BId Segment $j DQ] [AskBlast $FichierBlast BanqueId $BId Segment $j FQ]]
	    set FQ [Maxi [AskBlast $FichierBlast BanqueId $BId Segment $j DQ] [AskBlast $FichierBlast BanqueId $BId Segment $j FQ]]

	    set AiJeRencontreUnNiveauAvecDeLaPlace 0
	    for {set Niv $NiveauMinDeRef} {$Niv <= $NiveauMaxDuSubject} {incr Niv} {
		if {![info exists TabNiveau($Niv,LesRegions)]} {set TabNiveau($Niv,LesRegions) {}}
		set AiJeRencontreUnePlaceACeNiveau 1
		foreach LaRegion [set TabNiveau($Niv,LesRegions)] {
		    set DR [lindex $LaRegion 0]
		    set FR [lindex $LaRegion 1]
		    if {[CesRegionsSontEllesOverlappantes $DQ $FQ $DR $FR]} {
			set AiJeRencontreUnePlaceACeNiveau 0
			break
		    }
		}
		if {$AiJeRencontreUnePlaceACeNiveau} {
		    set     AiJeRencontreUnNiveauAvecDeLaPlace 1
		    lappend TabNiveau($Niv,LesRegions) [list $DQ $FQ]
		    set     NiveauVide                         $Niv
		    break
		}
	    }
	    if {!$AiJeRencontreUnNiveauAvecDeLaPlace} {
		incr    NiveauMaxDuSubject
		set     NiveauVide                        $NiveauMaxDuSubject
		set     TabNiveau($NiveauVide,LesRegions) {}
		lappend TabNiveau($NiveauVide,LesRegions) [list $DQ $FQ]
	    }
	    
	    set XMin [expr {($DQ-1) * $FacteurX + $BordX}]
	    set YMin [expr {$YMinDeRef + $NiveauVide * ($HSmallBox + $EcartY) - $HSmallBox}]
	    set XMax [expr {$FQ    * $FacteurX + $BordX}]
	    set YMax [expr {$YMin      + $HSmallBox}]
	    
	    if {$HauteurMax < $YMax} {set HauteurMax $YMax}
	    
	    set LesTags   [list "Box" $FichierBlast $BId $j]
	    lappend LesFormesAPlacerDuSubject [list rectangle $XMin $YMin $XMax $YMax $ColorFill $ColorOutLine 2 $LesTags]
	}
	if {$LesFormesAPlacerDuSubject == {}} {incr NiveauMinDeRef -1;continue}
	
	incr k
	if {$NbSubjectMax < $k} {break}

	set NiveauMinDeRef $NiveauMaxDuSubject

	set Expect [AskBlast $FichierBlast BanqueId $BId Segment 1 Expect]
	
	set LesTags   [list "Box" $FichierBlast $BId All]

	set XMin 0
	set XMax $LargeurFenetre
	lappend LesFormesAPlacer [list line $XMin $YLigne $XMax $YLigne "black" "dash"]
	lappend LesFormesAPlacer [list text $XMin [expr {$YLigne+1}] "nw" $BId                                  $LesTags]
	lappend LesFormesAPlacer [list text $XMax [expr {$YLigne+1}] "ne" [format "%10s %10s" $TailleS $Expect] $LesTags]

	foreach LaForme $LesFormesAPlacerDuSubject {
	    lappend LesFormesAPlacer $LaForme
	}
    }
    #######CREATION DES FORMES##################
    set HauteurMax [expr {$HauteurMax + $EcartY}]
    set KCanva [Canva $LargeurMax $HauteurMax $LargeurFenetre $HauteurFenetre "NoY" "BlastTools" "" $KCanva]
    $KCanva configure -background white

    set i 0
    foreach LaForme $LesFormesAPlacer {
	set Forme    [lindex $LaForme 0]
	set XMin     [lindex $LaForme 1]
	set YMin     [lindex $LaForme 2]

	if {$Forme == "text"} {
	    set Anch    [lindex $LaForme 3]
	    set Text    [lindex $LaForme 4]
	    set LesTags [lindex $LaForme 5]

	    $KCanva create text $XMin $YMin -text $Text -anchor $Anch -font "courier 12" -tags $LesTags
	    continue
	}
	set XMax     [lindex $LaForme 3]
	set YMax     [lindex $LaForme 4]
	set Color    [lindex $LaForme 5]
	if {$Forme == "rectangle"} {
	    set COutLine [lindex $LaForme 6]
	    set Width    [lindex $LaForme 7]
	    set LesTags  [lindex $LaForme 8]

	    $KCanva create rectangle $XMin $YMin $XMax $YMax -fill $Color -outline $COutLine -width $Width -tags $LesTags
	    continue
	}
	if {$Forme == "line"} {
	    if {[lindex $LaForme 6] == "dash"} {
		$KCanva create line      $XMin $YMin $XMax $YMax -fill $Color -dash {4 2}
	    } else {
		$KCanva create line      $XMin $YMin $XMax $YMax -fill $Color
	    }
	    continue
	}
    }

    $KCanva bind "Box" <1>          "DrawInfoDeDrawBlast      $w $KCanva Ali     "
    $KCanva bind "Box" <2>          "DrawInfoDeDrawBlast      $w $KCanva ToutLAli"
    $KCanva bind "Box" <3>          "DrawInfoDeDrawBlast      $w $KCanva PageInfo"
    $KCanva bind "Box" <Shift-1>    "DrawMappingPourDrawBlast $w $KCanva" 
}
proc DrawMapping {FichierBlast {BanqueId ""} {ExpectMax 1e200} {IdMin 0.0} {TailleMaxDUneRegionNonAlignee 500} {LargeurFenetre ""} {HauteurFenetre ""} {KCanva ""} {w ""}} {

    if {![file exists $FichierBlast]} {return}
    set TypeDeBlast [AskBlast $FichierBlast TypeDeBlast]
    set Query       [AskBlast $FichierBlast Query      ]
    set TailleQ     [AskBlast $FichierBlast Taille     ]
    if {$TailleQ == ""} {return}


    if {$BanqueId != ""} {
	set NbSegment [AskBlast $FichierBlast BanqueId $BanqueId NbSegment]
	set AiJeRecontreUnSegment 0
	for {set j 1} {$j <= $NbSegment} {incr j} {
	    set Expect     [AskBlast $FichierBlast BanqueId $BanqueId Segment $j Expect    ]
	    set Identities [AskBlast $FichierBlast BanqueId $BanqueId Segment $j Identities]
	    if {$Expect == "" || $Identities == ""} {continue}
	    scan $Identities "%d/%d" n nTot
	    set Id [expr {$n*1.0/$nTot}]
	    if {$ExpectMax < $Expect} {continue}
	    if {$Id < $IdMin} {continue}
	    set AiJeRecontreUnSegment 1
	    break
	}
	if {!$AiJeRecontreUnSegment} {set BanqueId ""}
    }
    if {$BanqueId == ""} {
	set NbSubject [AskBlast $FichierBlast NbSubject]
	for {set i 1} {$i <= $NbSubject} {incr i} {
	    set BId       [AskBlast $FichierBlast Subject $i]
	    set NbSegment [AskBlast $FichierBlast BanqueId $BId NbSegment]
	    set AiJeRecontreUnSegment 0
	    for {set j 1} {$j <= $NbSegment} {incr j} {
		set Expect     [AskBlast $FichierBlast BanqueId $BId Segment $j Expect    ]
		set Identities [AskBlast $FichierBlast BanqueId $BId Segment $j Identities]
		if {$Expect == "" || $Identities == ""} {continue}
		scan $Identities "%d/%d" n nTot
		set Id [expr {$n*1.0/$nTot}]
		if {$ExpectMax < $Expect} {continue}
		if {$Id < $IdMin} {continue}
		
		set AiJeRecontreUnSegment 1
		break
	    }
	    if {!$AiJeRecontreUnSegment} {continue}
	    set BanqueId $BId
	    break
	}
    }
    if {$BanqueId == ""} {
	Canva $LargeurFenetre $HauteurFenetre $LargeurFenetre $HauteurFenetre "NoY" "" "" $KCanva
	$KCanva configure -background white
	return
    }

    set TailleS     [AskBlast $FichierBlast BanqueId $BanqueId Taille   ]
    set NbSegment   [AskBlast $FichierBlast BanqueId $BanqueId NbSegment]
    if {$NbSegment == "" || $TailleS == ""} {return}

    if {$LargeurFenetre == ""} {set LargeurFenetre     [expr {[winfo screenheight .] * 2.0 / 3}]}
    if {$HauteurFenetre == ""} {set HauteurFenetre     [expr {[winfo screenheight .] * 2.0 / 3}]}
    set LargeurMax       $LargeurFenetre
    set HauteurMax       $HauteurFenetre
    set BordX            [expr {$LargeurFenetre * 1.0 / 4} ]
    set BordY            [expr {$HauteurFenetre * 1.0 / 10}]
    if {$BordY < 30} {set BordY 40}
    set EcartY           10.0
    set HBox             25.0
    set HSmallBox        15.0
    set LargeurAffichage [expr {$LargeurFenetre - 2 * $BordX}]
    set FacteurXDeQ      [expr {$LargeurAffichage * 1.0 / $TailleQ}]
    set FacteurXDeS      [expr {$LargeurAffichage * 1.0 / $TailleS}]

    set TailleDansLeSensPlus  0
    set TailleDansLeSensMoins 0
    
    set LesRegionsDuSubject        {}
    set LesRegionsUniquesDuSubject {}
    set LesRegionsDeLaQuery        {}
    set LesRegionsUniquesDeLaQuery {}
    for {set Seg 1} {$Seg <= $NbSegment} {incr Seg} {
	set Expect     [AskBlast $FichierBlast BanqueId $BanqueId Segment $Seg Expect    ]
	if {$ExpectMax < $Expect} {continue}
	set Identities [AskBlast $FichierBlast BanqueId $BanqueId Segment $Seg Identities]
	scan $Identities "%d/%d" n nTot
	set Id [expr {$n*1.0/$nTot}]
	if {$Id < $IdMin} {continue}

	set Orientation  [AskBlast $FichierBlast BanqueId $BanqueId Segment $Seg Orientation]
	if {$Orientation == ""} {set Orientation "Prot / Prot"}

	set Sens [SensDeLaRegion $Orientation]
	set DQ [Mini [AskBlast $FichierBlast BanqueId $BanqueId Segment $Seg DQ] [AskBlast $FichierBlast BanqueId $BanqueId Segment $Seg FQ]]
	set FQ [Maxi [AskBlast $FichierBlast BanqueId $BanqueId Segment $Seg DQ] [AskBlast $FichierBlast BanqueId $BanqueId Segment $Seg FQ]]
	set DS [Mini [AskBlast $FichierBlast BanqueId $BanqueId Segment $Seg DS] [AskBlast $FichierBlast BanqueId $BanqueId Segment $Seg FS]]
	set FS [Maxi [AskBlast $FichierBlast BanqueId $BanqueId Segment $Seg DS] [AskBlast $FichierBlast BanqueId $BanqueId Segment $Seg FS]]

	if {$Sens == "+"} {incr TailleDansLeSensPlus [expr {$FQ - $DQ + 1}]} else {incr TailleDansLeSensMoins [expr {$FQ - $DQ + 1}]}

	if {![info exists Tab(Query,$DQ,$FQ,RegionOccupee)]} {
	    set     Tab(Query,$DQ,$FQ,RegionOccupee)   1
	    lappend LesRegionsUniquesDeLaQuery [list $DQ $FQ]
	}
	lappend LesRegionsDeLaQuery [list $DQ $FQ $DS $FS $Orientation $Id $Seg]
	if {![info exists Tab(Subject,$DS,$FS,RegionOccupee)]} {
	    set     Tab(Subject,$DS,$FS,RegionOccupee) 1
	    lappend LesRegionsUniquesDuSubject [list $DS $FS]
	}
	lappend LesRegionsDuSubject [list $DS $FS $DQ $FQ $Orientation $Id $Seg]
    }
    if {$LesRegionsDuSubject == {} || $LesRegionsDeLaQuery == {}} {return}
    set SensGeneral "+"
    if {$TailleDansLeSensMoins > $TailleDansLeSensPlus} {set SensGeneral "-"}

    set LesPetitesRegionsUniquesDeLaQuery  [LesPetitesRegionsDesGrandesRegions  $LesRegionsUniquesDeLaQuery]
    foreach LaRegion [LesRegionsAbsentesDesPetitesRegions $LesPetitesRegionsUniquesDeLaQuery 1 $TailleQ] {
	lappend LaRegion "Absente"
	lappend LesPetitesRegionsUniquesDeLaQuery $LaRegion
    }

    set TailleDeLaSeqReduiteQ       0
    set TailleDeLaReduction         0
    foreach LaRegion [lsort -index 0 -real $LesPetitesRegionsUniquesDeLaQuery] {
	set DQ     [lindex $LaRegion 0]
	set FQ     [lindex $LaRegion 1]
	set Type   [lindex $LaRegion 2]

	set Taille [expr   {$FQ-$DQ + 1}]

	set  DRed                      [expr {$DQ - $TailleDeLaReduction}]
	set  FRed                      [expr {$FQ - $TailleDeLaReduction}]
	if {$Type == "Absente" && $TailleMaxDUneRegionNonAlignee < $Taille} {
	    set  FRed                  [expr {$DRed + $TailleMaxDUneRegionNonAlignee - 1}]
	    incr TailleDeLaReduction   [expr {$Taille - $TailleMaxDUneRegionNonAlignee}]
	    incr TailleDeLaSeqReduiteQ $TailleMaxDUneRegionNonAlignee
	} else {
	    incr TailleDeLaSeqReduiteQ $Taille
	}
	set TabRed(Query,$DQ) $DRed
	set TabRed(Query,$FQ) $FRed
    }
    
    set FacteurQX [expr {$LargeurAffichage * 1.0 / $TailleDeLaSeqReduiteQ}]

    set LesFormesAPlacer {}
    lappend LesFormesAPlacer [list text  [expr {$LargeurFenetre*1.0/2}] [expr {$BordY*1.0/3}] "center" "QUERY : [string range $Query end-30 end]"]
    lappend LesFormesAPlacer [list text  [expr {$LargeurFenetre*1.0/2}] [expr {$BordY*2.0/3}] "center" "BLAST : $TypeDeBlast"                    ]
    
    set     XMin             $BordX
    set     YMin             $BordY
    set     XMax             [expr {$XMin + $LargeurAffichage}]
    set     YMax             [expr {$YMin + $HBox}]
    set     YText            [expr {$YMin + ($YMax - $YMin)/2}]
    lappend LesFormesAPlacer [list rectangle $XMin           $YMin $XMax $YMax "lightgrey" "black" 3]
    lappend LesFormesAPlacer [list text      [expr {$XMin-1}]  $YText "e"  1                          ]
    lappend LesFormesAPlacer [list text      [expr {$XMax+1}]  $YText "w"  $TailleQ                   ]
    
    set HauteurAffichage [expr {$HauteurMax - 2 * ($BordY + $HBox + $EcartY)}]
    set YMinDeRef [expr {$BordY + $HBox + $EcartY}]
    
    set NiveauMaxDeRef [expr int($HauteurAffichage * 1.0 / ($HSmallBox + $EcartY)/2)]
    set NiveauMin 0
    set NiveauMax 0

    foreach LaRegion $LesRegionsDeLaQuery {
	set DQ  [lindex $LaRegion 0]
	set FQ  [lindex $LaRegion 1]

	if {![info exists Tab(Query,$DQ,$FQ,RegionOccupee)]} {continue}
	if {![info exists TabRed(Query,$DQ)               ]} {continue}
	if {![info exists TabRed(Query,$FQ)               ]} {continue}

	set DQR [set TabRed(Query,$DQ)]
	set FQR [set TabRed(Query,$FQ)]

	set DS     [lindex $LaRegion 2]
	set FS     [lindex $LaRegion 3]
	set Orient [lindex $LaRegion 4]
	set Id     [lindex $LaRegion 5]
	set Seg    [lindex $LaRegion 6]

	set ColorFill    [ColoriageDeLIdDeDrawBlast     $Id    ]
	set ColorOutLine [ColoriageDuMappingDeDrawBlast $Orient]

	set AiJeRencontreUnNiveauAvecDeLaPlace 0
	for {set Niv $NiveauMin} {$Niv <= $NiveauMax} {incr Niv} {
	    if {![info exists TabNiveau($Niv,LesRegions)]} {set TabNiveau($Niv,LesRegions) {}}
	    set AiJeRencontreUnePlaceACeNiveau 1
	    foreach LaRegionDuNiveau [set TabNiveau($Niv,LesRegions)] {
		set DNiv [lindex $LaRegionDuNiveau 0]
		set FNiv [lindex $LaRegionDuNiveau 1]
		if {[CesRegionsSontEllesOverlappantes $DQ $FQ $DNiv $FNiv]} {
		    set AiJeRencontreUnePlaceACeNiveau 0
		    break
		}
	    }
	    if {$AiJeRencontreUnePlaceACeNiveau} {
		set     NiveauVide                         $Niv
		lappend TabNiveau($NiveauVide,LesRegions)  [list $DQ $FQ]
		set     AiJeRencontreUnNiveauAvecDeLaPlace 1
		break
	    }
	}
	if {!$AiJeRencontreUnNiveauAvecDeLaPlace} {
	    incr    NiveauMax
	    set     NiveauVide                        $NiveauMax
	    set     TabNiveau($NiveauVide,LesRegions) {}
	    lappend TabNiveau($NiveauVide,LesRegions) [list $DQ $FQ]
	}

	if {$NiveauMaxDeRef < $NiveauVide} {set NiveauVide $NiveauMaxDeRef}

	set XMin    [expr {($DQR-1)   * $FacteurQX + $BordX}]
	set YMin    [expr {$YMinDeRef + $NiveauVide * ($HSmallBox + $EcartY)}]
	set XMax    [expr {$FQR       * $FacteurQX + $BordX}]
	set YMax    [expr {$YMin      + $HSmallBox}]

	set LesTags [list "Box" $FichierBlast $BanqueId $Seg]

	set Tab(DQ,$DQ,FQ,$FQ,LesCoordonnees) [list $XMin $YMax $XMax $YMax]

	lappend LesFormesAPlacer [list rectangle $XMin $YMin $XMax $YMax $ColorFill $ColorOutLine 2 $LesTags]
    }


    set LesPetitesRegionsUniquesDuSubject  [LesPetitesRegionsDesGrandesRegions  $LesRegionsUniquesDuSubject]
    foreach LaRegion [LesRegionsAbsentesDesPetitesRegions $LesPetitesRegionsUniquesDuSubject 1 $TailleS] {
	lappend LaRegion "Absente"
	lappend LesPetitesRegionsUniquesDuSubject $LaRegion
    }

    set TailleDeLaSeqReduiteS       0
    set TailleDeLaReduction         0
    foreach LaRegion [lsort -index 0 -real $LesPetitesRegionsUniquesDuSubject] {
	set DS     [lindex $LaRegion 0]
	set FS     [lindex $LaRegion 1]
	set Type   [lindex $LaRegion 2]

	set Taille [expr   {$FS-$DS + 1}]

	set  DRed                      [expr {$DS - $TailleDeLaReduction}]
	set  FRed                      [expr {$FS - $TailleDeLaReduction}]
	if {$Type == "Absente" && $TailleMaxDUneRegionNonAlignee < $Taille} {
	    set  FRed                  [expr {$DRed + $TailleMaxDUneRegionNonAlignee - 1}]
	    incr TailleDeLaReduction   [expr {$Taille - $TailleMaxDUneRegionNonAlignee}]
	    incr TailleDeLaSeqReduiteS $TailleMaxDUneRegionNonAlignee
	} else {
	    incr TailleDeLaSeqReduiteS $Taille
	}
	set TabRed(Subject,$DS) $DRed
	set TabRed(Subject,$FS) $FRed
    }
    set FacteurSX [expr {$LargeurAffichage * 1.0 / $TailleDeLaSeqReduiteS}]
    
    set     XMin             $BordX
    set     YMin             [expr {$HauteurMax - $BordY - $HBox}]
    set     XMax             [expr {$XMin + $LargeurAffichage}]
    set     YMax             [expr {$YMin + $HBox}]
    set     YText            [expr {$YMin + ($YMax - $YMin)/2}]
    lappend LesFormesAPlacer [list rectangle $XMin           $YMin $XMax $YMax "lightgrey" "black" 3]
    lappend LesFormesAPlacer [list text  [expr {$LargeurFenetre*1.0/2}] [expr {$HauteurMax - $BordY*1.0/3}] "center" "SBJCT : $BanqueId"]
    
    if {$SensGeneral == "+"} {
	lappend LesFormesAPlacer [list text      [expr {$XMin-1}]  $YText "e"  1                          ]
	lappend LesFormesAPlacer [list text      [expr {$XMax+1}]  $YText "w"  $TailleS                   ]
    } else {
	lappend LesFormesAPlacer [list text      [expr {$XMin-1}]  $YText "e"  $TailleS                   ]
	lappend LesFormesAPlacer [list text      [expr {$XMax+1}]  $YText "w"  1                          ]
    }
    foreach LaRegion $LesRegionsDuSubject {
	set DS    [lindex $LaRegion 0]
	set FS    [lindex $LaRegion 1]

	if {![info exists Tab(Subject,$DS,$FS,RegionOccupee)]} {continue}
	if {![info exists TabRed(Subject,$DS)               ]} {continue}
	if {![info exists TabRed(Subject,$FS)               ]} {continue}

	set DSR    [set TabRed(Subject,$DS)]
	set FSR    [set TabRed(Subject,$FS)]
	set DQ     [lindex $LaRegion 2]
	set FQ     [lindex $LaRegion 3]
	set Orient [lindex $LaRegion 4]
	set Id     [lindex $LaRegion 5]
	set Seg    [lindex $LaRegion 6]

	set ColorFill    [ColoriageDeLIdDeDrawBlast    $Id    ]
	set ColorOutLine [ColoriageDuMappingDeDrawBlast $Orient]

	if {$SensGeneral == "-"} {
	    set XMax  [expr {$LargeurMax - (($DSR-1) * $FacteurSX + $BordX)}]
	    set XMin  [expr {$LargeurMax - ($FSR     * $FacteurSX + $BordX)}]
	} else {
	    set XMin  [expr {($DSR-1) * $FacteurSX + $BordX}]
	    set XMax  [expr {$FSR     * $FacteurSX + $BordX}]
	}

	set LesTags [list "Box" $FichierBlast $BanqueId $Seg]

	if {![info exists Tab(Subject,Box,$DS,$FS)]} {
	    set     Tab(Subject,Box,$DS,$FS) 1
	    lappend LesFormesAPlacer [list rectangle $XMin $YMin $XMax $YMax $ColorFill $ColorOutLine 2 $LesTags]
	}
	set XLineS1 $XMin
	set YLineS1 $YMin
	set XLineS2 $XMax
	set YLineS2 $YMin

	set LesCoordonnees [set Tab(DQ,$DQ,FQ,$FQ,LesCoordonnees)]
	set XLineQ1 [lindex $LesCoordonnees 0]
	set YLineQ1 [lindex $LesCoordonnees 1]
	set XLineQ2 [lindex $LesCoordonnees 2]
	set YLineQ2 [lindex $LesCoordonnees 3]
	
	lappend LesFormesAPlacer [list line $XLineQ1 $YLineQ1 $XLineS1 $YLineS1 $ColorOutLine]
	lappend LesFormesAPlacer [list line $XLineQ2 $YLineQ2 $XLineS2 $YLineS2 $ColorOutLine]
    }

    #######CREATION DES FORMES##################

    set KCanva [Canva $LargeurMax $HauteurMax $LargeurFenetre $HauteurFenetre "NoY" "BlastTools" "" $KCanva]
    $KCanva configure -background white

    set i 0
    foreach LaForme $LesFormesAPlacer {

	set Forme    [lindex $LaForme 0]
	set XMin     [lindex $LaForme 1]
	set YMin     [lindex $LaForme 2]

	if {$Forme == "text"} {
	    set Anch    [lindex $LaForme 3]
	    set Text    [lindex $LaForme 4]
	    set LesTags [lindex $LaForme 5]

	    $KCanva create text $XMin $YMin -text $Text -anchor $Anch -font "courier 10" -tags $LesTags
	    continue
	}
	set XMax     [lindex $LaForme 3]
	set YMax     [lindex $LaForme 4]
	set Color    [lindex $LaForme 5]
	if {$Forme == "rectangle"} {
	    set COutLine [lindex $LaForme 6]
	    set Width    [lindex $LaForme 7]
	    set LesTags  [lindex $LaForme 8]

	    $KCanva create rectangle $XMin $YMin $XMax $YMax -fill $Color -outline $COutLine -width $Width -tags $LesTags
	    continue
	}
	if {$Forme == "line"} {
	    if {[lindex $LaForme 6] == "dash"} {
		$KCanva create line      $XMin $YMin $XMax $YMax -fill $Color -dash {4 2}
	    } else {
		$KCanva create line      $XMin $YMin $XMax $YMax -fill $Color
	    }
	    continue
	}
    }

    $KCanva bind "Box" <1>     "DrawInfoDeDrawBlast $w $KCanva Ali     "
    $KCanva bind "Box" <2>     "DrawInfoDeDrawBlast $w $KCanva ToutLAli"
    $KCanva bind "Box" <3>     "DrawInfoDeDrawBlast $w $KCanva PageInfo"
}
proc ColoriageDuMappingDeDrawBlast {TypeRegion} {
    if {$TypeRegion == "Plus / Plus"} {
	set ColorFill "blue"
    } elseif {$TypeRegion == "Plus / Minus"} {
	set ColorFill "red"
    } elseif {$TypeRegion == "N"} {
	set ColorFill "black"
    } elseif {$TypeRegion == "RAS"} {
	set ColorFill "lightgrey"
    } elseif {$TypeRegion == "+1"} {
	set ColorFill "DodgerBlue1"
    } elseif {$TypeRegion == "+2"} {
	set ColorFill "blue1"
    } elseif {$TypeRegion == "+3"} {
	set ColorFill "blue4"
    } elseif {$TypeRegion == "-1"} {
	set ColorFill "red1"
    } elseif {$TypeRegion == "-2"} {
	set ColorFill "red3"
    } elseif {$TypeRegion == "-3"} {
	set ColorFill "red4"
    } elseif {$TypeRegion == "Prot / Prot"} {
	set ColorFill "orange"
    } elseif {[regexp "\[1-3\+]{2} / \[1-3\+]{2}" $TypeRegion]} {
	set ColorFill "DarkOrchid"
    } elseif {[regexp "\[1-3\+]{2} / \[1-3\-]{2}" $TypeRegion]} {
	set ColorFill "MediumOrchid"
    } elseif {[regexp "\[1-3\-]{2} / \[1-3\+]{2}" $TypeRegion]} {
	set ColorFill "SpringGreen3"
    } elseif {[regexp "\[1-3\-]{2} / \[1-3\-]{2}" $TypeRegion]} {
	set ColorFill "SpringGreen4"
    } else {
	set ColorFill "HotPink1"
    }
    return $ColorFill
}
proc ColoriageDeLaFrequenceDeDrawBlast {Frequence} {
    global LesCouleursDeLaGammeDeLaFreq
    if {![info exists LesCouleursDeLaGammeDeLaFreq]} {
	set LesCouleursDeLaGammeDeLaFreq [GammeDeCouleurBlancVersRouVouB B 8]
    }
    if {200 <=  $Frequence                    } {return [lindex $LesCouleursDeLaGammeDeLaFreq 7]}
    if {100 <= $Frequence && $Frequence <  200} {return [lindex $LesCouleursDeLaGammeDeLaFreq 6]}
    if {50  <= $Frequence && $Frequence <  100} {return [lindex $LesCouleursDeLaGammeDeLaFreq 5]}
    if {10  <= $Frequence && $Frequence <  50 } {return [lindex $LesCouleursDeLaGammeDeLaFreq 4]}
    if {4   <= $Frequence && $Frequence <  10 } {return [lindex $LesCouleursDeLaGammeDeLaFreq 3]}
    if {2   <= $Frequence && $Frequence <  4  } {return [lindex $LesCouleursDeLaGammeDeLaFreq 2]}
    if {1   <= $Frequence && $Frequence <  2  } {return [lindex $LesCouleursDeLaGammeDeLaFreq 1]}
    return [lindex $LesCouleursDeLaGammeDeLaFreq 0]
}
proc ColoriageDeLIdDeDrawBlast {Id} {
    global LesCouleursDeLaGammeDeLId
    if {![info exists LesCouleursDeLaGammeDeLId]} {
	set LesCouleursDeLaGammeDeLId [GammeDeCouleurBlancVersRouVouB V 5]
    }
    if {0.90 < $Id} {return [lindex $LesCouleursDeLaGammeDeLId 4]}
    if {0.80 < $Id} {return [lindex $LesCouleursDeLaGammeDeLId 3]}
    if {0.60 < $Id} {return [lindex $LesCouleursDeLaGammeDeLId 2]}
    if {0.30 < $Id} {return [lindex $LesCouleursDeLaGammeDeLId 1]}
    return [lindex $LesCouleursDeLaGammeDeLId 0]
}
