#################################################################################
# CADO4MI - CADO4MI: Computer Assisted Design of Oligonucleotide For MIcroarray #
#                                                                               #
# CADO4MI is a program to design oligonucleotide for microarrays or PCR.        #
#                                                                               #
# Copyright (C) 2006 Jean Muller (jean.muller@igbmc.u-strasbg.fr)               # 
#                                                                               #
# Please cite the following article:                                            #
#    Muller J et al                                                             #
#    Design and evaluation of Actichip,                                         #
#    a thematic microarray for the study of the actin cytoskeleton.             #
#    BMC Genomics 2007 Aug 29;8(1):294.                                         #
#                                                                               #
# This is part of CADO4MI source code.                                          #
#                                                                               #
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public License                   # 
# as published by the Free Software Foundation; either version 2                # 
# of the License, or (at your option) any later version.                        #
#                                                                               #
# This program is distributed in the hope that it will be useful,               # 
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
# GNU General Public License for more details.                                  #
#                                                                               #
# You should have received a copy of the GNU General Public License             #
# along with this program; if not, write to the Free Software                   #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                     #
#################################################################################

proc BlastSplitteParOrganisme {FileBlast Banque {LesOrganismesOuTaxonsPrioritaires ""} {LesOrganismesOuTaxonsAExclure ""}} {
    foreach Organisme $LesOrganismesOuTaxonsAExclure     {set TabAExclure($Organisme)     1}
    foreach Organisme $LesOrganismesOuTaxonsPrioritaires {set TabPrioritaires($Organisme) 1}

    AskBlast unset

    Espionne "$FileBlast -> Loading ..."
    set LesBId {}
    set NbSubject [AskBlast $FileBlast NbSubject]
    if {$NbSubject == ""} {return}
    for {set i 1} {$i <= $NbSubject} {incr i} {lappend LesBId [AskBlast $FileBlast Subject $i]}
    Espionne "$FileBlast -> Loaded ..."

    Espionne "$FileBlast -> Loading database ..."
    AskLesPagesInfos load $LesBId $Banque
    
    set LesOrganismes {}
    foreach Id $LesBId {
	foreach Orga [LesOrgasDeLaPage_PageInfo [AskLesPagesInfos ask $Id]] {
	    if {$Orga == ""} {continue}
	    Espionne "$Id -> $Orga"
	    if {![info exists Tab($Orga,LesBId)]} {
		set     Tab($Orga,LesBId) {} 
		lappend LesOrganismes     $Orga
	    }
	    lappend Tab($Orga,LesBId) $Id
	}
    }
    Espionne "$FileBlast -> database loaded..."

    foreach Orga $LesOrganismes {
	set EstCeOK                   1
	set AiJeLesTaxonsPrioritaires 0
	foreach Tax [InterrogeClassificationTaxonomique $Orga LesPhylums] {
	    if {[info exists  TabAExclure($Tax)    ]} {set EstCeOK 0                  ;continue}
	    if {[info exists  TabPrioritaires($Tax)]} {set AiJeLesTaxonsPrioritaires 1;continue}
	}
	if {!$EstCeOK} {continue}
	if {$LesOrganismesOuTaxonsPrioritaires != "" && !$AiJeLesTaxonsPrioritaires} {continue}

	Espionne $Orga

	set Ext $Orga
	regsub -all " " $Orga "_" Ext

	set NewFileBlast "${FileBlast}_${Ext}"

	set LesId [set Tab($Orga,LesBId)]
	FichierBlastReduit $FileBlast $NewFileBlast LesBIdToKeep $LesId
    }

    AskLesPagesInfos unset
    AskBlast         unset
    return
}
proc CorrigeFichierAnchors {FichierAnchors} {
    if {![file exists $FichierAnchors]} {return}

    File copy -force $FichierAnchors "$FichierAnchors.old"

    set F [open $FichierAnchors]
    while {[gets $F Ligne]>=0} {
	set Ligne [string toupper $Ligne]
	regsub -all {[^ \|]+\|} $Ligne "" Ligne 
	regsub -nocase "SEQ\:"    $Ligne "seq:"     Ligne
	regsub -nocase "POS\:"    $Ligne "pos:"     Ligne
	regsub -nocase "BEG\:"    $Ligne "beg:"     Ligne
	regsub -nocase "LEN\:"    $Ligne "len:"     Ligne
	regsub -nocase "WEIGHT\:" $Ligne "weight:"  Ligne
	lappend LesLignes $Ligne
    }
    close $F

    Sauve [join $LesLignes "\n"] dans $FichierAnchors
    return $FichierAnchors
}
proc AskAnchors args {
    #unset
    #$FileAnchors IdDeRef
    #$FileAnchors LesId
    #$FileAnchors Id $Id NbLMS
    #$FileAnchors Id $Id LMS $n Pos
    #$FileAnchors Id $Id LMS $n Beg
    #$FileAnchors Id $Id LMS $n End
    #$FileAnchors Id $Id LMS $n Len

    global TabDesAnchorsDeBallast

    if {$args == ""} {return}
    set PremArg [lindex $args 0]
    if {[string equal -nocase $PremArg "unset"]} {
	if {[info exists TabDesAnchorsDeBallast ]} {unset TabDesAnchorsDeBallast}
	return
    }
    if {![file exists $PremArg]} {return}
    set Quoi [join $args ","]

    if {[info exists TabDesAnchorsDeBallast($Quoi)]} {return [set TabDesAnchorsDeBallast($Quoi)]}
    if {[info exists TabDesAnchorsDeBallast($PremArg,EstCharge)]} {return}
    
    set         TabDesAnchorsDeBallast($PremArg,EstCharge) 1
    LoadAnchors TabDesAnchorsDeBallast $PremArg

    if {[info exists TabDesAnchorsDeBallast($Quoi)]} {return [set TabDesAnchorsDeBallast($Quoi)]}
    return
}
proc LoadAnchors {aTab FichierAnchors} {
    upvar $aTab Tab
    if {![file exists $FichierAnchors]} {return}

    set Tab($FichierAnchors,LesId) {}

    set F [open $FichierAnchors]
    while {[gets $F Ligne]>=0} {
	if {![regexp {^seq\:} $Ligne]} {continue}
	set LaLigne [split $Ligne "\t"]

	regsub -all "\t" $Ligne " " Ligne
	while {[regexp "  " $Ligne]} {regsub -all {[ ]{2,}} $Ligne " " Ligne}
	set LaLigne [split $Ligne " "]

	if {![info exists Tab($FichierAnchors,IdDeRef)]} {set Tab($FichierAnchors,IdDeRef) [lindex $LaLigne 1]}

	set Id  [lindex $LaLigne 2]

	if {![info exists Tab($FichierAnchors,Id,$Id,NbLMS)]} {
	    set     Tab($FichierAnchors,Id,$Id,NbLMS) 0
	    lappend Tab($FichierAnchors,LesId)        $Id
	}
	incr Tab($FichierAnchors,Id,$Id,NbLMS)

	set  nLMS                                      [set Tab($FichierAnchors,Id,$Id,NbLMS)]
	set  Tab($FichierAnchors,Id,$Id,LMS,$nLMS,Pos) [lindex $LaLigne 4] 
	set  Tab($FichierAnchors,Id,$Id,LMS,$nLMS,Beg) [lindex $LaLigne 6] 
	set  Tab($FichierAnchors,Id,$Id,LMS,$nLMS,End) [lindex $LaLigne 7] 
	set  Tab($FichierAnchors,Id,$Id,LMS,$nLMS,Len) [lindex $LaLigne 9] 
    }
    close $F
    return $aTab
}
proc LesTFAsDesHitsDuBlastP {FichierBlast {NbHitsMax 200} {NbPDBMax 30} {AvecFragment 0}} {
    set NbRequestInDBMax [expr {($NbHitsMax + $NbPDBMax) * 2}]
    set NbRequestInDB    0

    set LesHits [LesBanqueIdDuBlast $FichierBlast 1e200 1e200] 

    set LesBanqueId       {} 
    foreach BanqueId $LesHits {
	set BanqueId [string toupper $BanqueId]
	if {$NbRequestInDBMax <= $NbRequestInDB} {break}

	if {[regexp -nocase "UniRef" $BanqueId]} {
	    Espionne $BanqueId
	    regexp -nocase {UniRef[0-9]*} $BanqueId BanqueUniref
	    set BanqueUniref [string tolower $BanqueUniref]
	    set TFA [TFADeLaBanqueBlast "/blast/$BanqueUniref" $BanqueId]
	    if {$TFA == ""} {continue}
	    regsub -nocase {>[^ ]+} $TFA ">$BanqueId" TFA
	    set TabTmp($BanqueId) $TFA
	    incr NbRequestInDB
	    continue
	}
	if {[EstUnAccessPDB $BanqueId]} {
	    set TFA [TFADeLaBanqueBlast "/blast/pdb" $BanqueId]
	    if {$TFA == ""} {continue}
	    regsub -nocase {>[^ ]+} $TFA ">$BanqueId" TFA
	    set TabTmp($BanqueId) $TFA
	    incr NbRequestInDB
	    continue
	}
	if {[regexp {\-[0-9]+$} $BanqueId]} {
	    set TFA [TFADeLaBanqueBlast "/blast/varsplic" "VARSPLIC:$BanqueId"]
	    if {$TFA == ""} {continue}
	    regsub -nocase {>[^ ]+} $TFA ">$BanqueId" TFA
	    set TabTmp($BanqueId) $TFA
	    incr NbRequestInDB
	    continue
	}
	incr NbRequestInDB
	lappend LesBanqueId $BanqueId
    }
    foreach TFA [LesSeqTFAsDesBanqueId $LesBanqueId "protein" "" $AvecFragment] {
	set TFA [string toupper $TFA]
	scan [EnteteDuTexteTFA $TFA] ">%s" BId 
	if {$BId == ""} {continue}
	set BId [string toupper $BId]
	set TabTmp($BId) $TFA
    }
    set NbPDB  0
    set NbHits 0
    set LesTFAs {}
    foreach BanqueId $LesHits {
	set BanqueId [string toupper $BanqueId]
	if {![info exists TabTmp($BanqueId)]} {continue}
	if {[EstUnAccessPDB $BanqueId]} {
	    incr NbPDB
	    if {$NbPDBMax  < $NbPDB } {continue}
	} else {
	    incr NbHits
	    if {$NbHitsMax < $NbHits} {continue}
	}
	lappend LesTFAs [set TabTmp($BanqueId)]
    }
    return $LesTFAs
}
proc LesTFAsDesHitsDuBallast {FichierAnchors {NbHitsMax 200} {NbPDBMax 30} {AvecFragment 0}} {
    if {![file exists $FichierAnchors]} {return}

    AskAnchors unset 

    set IdDeRef [AskAnchors $FichierAnchors IdDeRef]

    set LesHits [list [list $IdDeRef 1e200]]
    foreach Id          [AskAnchors $FichierAnchors LesId          ] {
	set     n       [AskAnchors $FichierAnchors Id    $Id NbLMS]
	lappend LesHits [list $Id $n]
    }

    set NbRequestInDBMax [expr {($NbHitsMax + $NbPDBMax) * 2}]
    set NbRequestInDB    0

    set LesBanqueId       {} 
    foreach LeHit [lsort -index 1 -decreasing -real $LesHits] {
	if {$NbRequestInDBMax <= $NbRequestInDB} {break}

	set BanqueId [lindex $LeHit 0    ]
	
	if {[info exists TabDejaVu($BanqueId)]} {continue}
	set TabDejaVu($BanqueId) 1

	if {[regexp -nocase "UniRef" $BanqueId]} {
	    Espionne $BanqueId
	    regexp -nocase {UniRef[0-9]*} $BanqueId BanqueUniref
	    set BanqueUniref [string tolower $BanqueUniref]
	    set TFA [TFADeLaBanqueBlast "/blast/$BanqueUniref" $BanqueId]
	    if {$TFA == ""} {continue}
	    regsub -nocase {>[^ ]+} $TFA ">$BanqueId" TFA
	    set TabTmp($BanqueId) $TFA
	    incr NbRequestInDB
	    continue
	}
	if {[EstUnAccessPDB $BanqueId]} {
	    set TFA [TFADeLaBanqueBlast "/blast/pdb" $BanqueId]
	    if {$TFA == ""} {continue}
	    regsub -nocase {>[^ ]+} $TFA ">$BanqueId" TFA
	    set TabTmp($BanqueId) $TFA
	    incr NbRequestInDB
	    continue
	}
	if {[EstUnAccessVARSPLIC $BanqueId]} {
	    set TFA [TFADeLaBanqueBlast "/blast/varsplic" "VARSPLIC:$BanqueId"]
	    if {$TFA == ""} {continue}
	    regsub -nocase {>[^ ]+} $TFA ">$BanqueId" TFA
	    set TabTmp($BanqueId) $TFA
	    incr NbRequestInDB
	    continue
	}
	incr NbRequestInDB
	lappend LesBanqueId $BanqueId
    }
    foreach TFA [LesSeqTFAsDesBanqueId $LesBanqueId "protein" "" $AvecFragment] {
	set TFA [string toupper $TFA]
	scan [EnteteDuTexteTFA $TFA] ">%s" BId 
	if {$BId == ""} {continue}
	set BId [string toupper $BId]
	set TabTmp($BId) $TFA
    }

    set NbPDB  0
    set NbHits 0
    set LesTFAs {}
    foreach LeHit $LesHits {
	set BanqueId [string toupper [lindex $LeHit 0    ]]
	if {![info exists TabTmp($BanqueId)]} {continue}
	if {[EstUnAccessPDB $BanqueId]} {
	    incr NbPDB
	    if {$NbPDBMax  < $NbPDB } {continue}
	} else {
	    incr NbHits
	    if {$NbHitsMax < $NbHits} {continue}
	}
	lappend LesTFAs [set TabTmp($BanqueId)]
    }
    AskAnchors unset

    return $LesTFAs
}
proc RepBlastDBAlternatif {} {
    return "/home/chalmel/BlastDB"
}
proc PipeAlign {FichierTFA {Source "TFA"} {FileIn ""} {ExpectMax "100"} {NbSubjectMax "2000"} {NbProtMax "200"} {NbPDBMax "30"} {Banque "protall"}} {
    #Source -> TFA
    #Source -> BLASTP
    #Source -> BALLAST (needs .anchors too)
    #Source -> MSF
    #Source -> MSFFinal

    if {[string equal -nocase $Source "TFA"]} {set FileIn $FichierTFA}
    if {![file exists $FileIn] || ![file exists $FichierTFA]} {return}    
  
    set Seq [string toupper [QueLaSequenceDuFichierTFA $FichierTFA]]
    set TFA [SequenceFormatTFA $Seq "QUERY_PROTEIN"]

    set FaireLeBlast   0
    set FaireLeBallast 0
    set FaireLeMSF     0
    set FaireLeRascal  0
    set FaireLeClust   0
    if {[string equal -nocase $Source "TFA"    ]} {
	set FaireLeBlast 1;set FaireLeBallast 1;set FaireLeMSF 1;set FaireLeRascal 1;set FaireLeClust 1
    }

    if {[string equal -nocase $Source "BLASTP" ]} {
	set FichierBlast   $FileIn
	set                    FaireLeBallast 1;set FaireLeMSF 1;set FaireLeRascal 1;set FaireLeClust 1
    }
    if {[string equal -nocase $Source "BALLAST"]} {
	set FichierBallast $FileIn
	set                                         FaireLeMSF 1;set FaireLeRascal 1;set FaireLeClust 1
    }
    if {[string equal -nocase $Source "MSF"    ]} {
	set FichierMSFDB   $FileIn
	set                                                          FaireLeRascal 1;set FaireLeClust 1
    }
    if {[string equal -nocase $Source "MSFFinal"]} {
	set FichierMSF     $FileIn
	set                                                                              FaireLeClust 1
    }
    
    #BLAST#
    if {$FaireLeBlast} {
	Espionne "$FichierTFA -> PipeBlast"
	set FichierBlast   "$FichierTFA.blastp"
	PipeBlast $FichierTFA $Banque $FichierBlast P $NbSubjectMax $ExpectMax T 4
	if {![file exists $FichierBlast]} {return}    
    }	
    #BALLAST (Ballast - TFAs)#
    if {$FaireLeBallast} {
	Espionne "$FichierTFA -> PipeBallast"
	set FichierBallast "$FichierTFA.ballast"
	
	PipeBallast $FichierBlast $FichierBallast
	set FileAnchors    "$FichierTFA.ballast.anchors"
	set FichierTFAs    "$FichierTFA.tfas"
	if {      [file exists $FileAnchors]} {
	    CorrigeFichierAnchors $FileAnchors
	    Espionne "TFAs from Ballast"
	    Sauve [join [linsert [LesTFAsDesHitsDuBallast $FileAnchors   $NbProtMax $NbPDBMax 0] 0 $TFA]  "\n"] dans $FichierTFAs 
	} elseif {[file exists $FichierBlast]} {
	    Espionne "TFAs from BlastP"
	    Sauve [join [linsert [LesTFAsDesHitsDuBlastP  $FichierBlast  $NbProtMax $NbPDBMax 0] 0 $TFA]  "\n"] dans $FichierTFAs
	}
	if {![file exists $FichierTFAs]} {return}
    }

    #MSF (DbClustal - Rascal - NorMD - Selection) 
    if {$FaireLeMSF} {
	Espionne "$FichierTFA -> PipeDbClustal"
	set FichierMSFDB  "$FichierTFA.msfdbclustal"
	PipeDbClustal $FichierTFAs $FileAnchors $FichierMSFDB
	if {![file exists $FichierMSFDB]} {return}
    }
    if {$FaireLeRascal} {
	Espionne "$FichierTFA -> PipeRascal"
	set FichierMSFRascal  "$FichierTFA.msfrascal"
	PipeRascal $FichierMSFDB $FichierMSFRascal
	if {![file exists $FichierMSFRascal]} {return}

	Espionne "$FichierTFA -> PipeNorMD"
	set FichierMSF    "$FichierTFA.msf"
	if {[PipeNorMD $FichierMSFDB] <= [PipeNorMD $FichierMSFRascal]} {
	    File copy -force $FichierMSFRascal $FichierMSF
	} else {
	    File copy -force $FichierMSFDB     $FichierMSF
	}
    }
    #Clustering (Secator or DPC)
    if {$FaireLeClust} {
	Espionne "$FichierTFA -> PipeClustering"
	set FichierClust "$FichierTFA.clust"
	PipeClustering $FichierMSF $FichierClust
	if {![file exists $FichierClust]} {return}
    }
    return
}
proc PipeBlast {FileTFA Database {FileOUT ""} {NorPorXorTNorTX "P"} {NbSubjectInOutPut "500"} {ExpectMin "10"} {Filtre "T"} {NbProc "4"}} {
    if {![file exists $FileTFA]} {return}

    if {[string equal -nocase $NorPorXorTNorTX "N"]} {
	set BLAST "blastn"
    } elseif {[string equal -nocase $NorPorXorTNorTX "P"]} {
	set BLAST "blastp"
    } elseif {[string equal -nocase $NorPorXorTNorTX "X"]} {
	set BLAST "blastx"
    } elseif {[string equal -nocase $NorPorXorTNorTX "TN"]} {
	set BLAST "tblastn"
    } elseif {[string equal -nocase $NorPorXorTNorTX "TX"]} {
	set BLAST "tblastx"
    } else {
	set BLAST "blastp"
    }
    if {$FileOUT == ""} {set FileOUT "$FileTFA.$BLAST"}

    if {![string equal -nocase $Filtre "T"]} {set Filtre "F"}
    
    set NomDeLaDatabase ""    
    if {    [file exists "/blast/$Database.nhr"] || [file exists "/blast/$Database.nal"] || \
	    [file exists "/blast/$Database.phr"] || [file exists "/blast/$Database.pal"] } {
	set NomDeLaDatabase "/blast/$Database"
    }
    if {[file exists "[RepBlastDBAlternatif]/$Database"]} {
	set NomDeLaDatabase "[RepBlastDBAlternatif]/$Database"
    }
    if {    [file exists "$Database.nhr"       ] || [file exists "$Database.nal"       ] || \
	    [file exists "$Database.phr"       ] || [file exists "$Database.pal"       ] } {
	set NomDeLaDatabase "$Database"
    }
    if {$NomDeLaDatabase == ""} {Espionne "$NomDeLaDatabase does not exist";return}

    set CommandeBlast "eval exec [ProgPathway blastall] \
		-p \$BLAST \
		-i \$FileTFA \
		-o \$FileOUT \
		-d \$NomDeLaDatabase   \
		-v \$NbSubjectInOutPut \
		-K 0 \
		-b \$NbSubjectInOutPut \
		-e \$ExpectMin \
		-F \$Filtre \
		-a \$NbProc"

    if {[catch {eval $CommandeBlast} Message]} {
	Espionne "Problem with $FileOUT"
	Espionne $Message
    }
    return $FileOUT
}
proc PipeBallast {FileBlast {FileOut ""} {QueryName ""}} {
    if {![file exists $FileBlast]} {return}
    if {$FileOut == ""} {set FileOut "$FileBlast.ballast"}
    set                           CommandeBallast "eval exec [ProgPathway ballast] -p $FileBlast -o $FileOut -anchors -noext"
    if {$QueryName != ""} {append CommandeBallast " -queryname $QueryName"}
    if {[catch {eval $CommandeBallast} Message]} {Espionne $Message}
    set FileAnchors "$FileBlast.ballast.anchors"
    if {[file exists $FileAnchors]} {CorrigeFichierAnchors $FileAnchors}
    return $FileOut
}
proc PipeDbClustal {FichierTFAs FichierAnchors FichierOut} {
    if {![file exists $FichierTFAs]} {return}
    set FichierLog "$FichierOut.log"

    if {[file exists $FichierAnchors]} {
	if {[catch {exec [ProgPathway dbclustal] $FichierTFAs -motifs=$FichierAnchors -output=gscope -outfile=$FichierOut -propagate >& $FichierLog} Message]} {Espionne $Message;return}
    } else {
	if {[catch {exec [ProgPathway dbclustal] $FichierTFAs                         -output=gscope -outfile=$FichierOut -propagate >& $FichierLog} Message]} {Espionne $Message;return} 
    }

    if {![file exists $FichierOut]} {return}

    set NorMD         [PipeNorMD             $FichierOut]
    set LeMSF         [LesLignesDuFichier $FichierOut]
    set DeuxiemeLigne [string trimright [lindex $LeMSF 1]]

    append DeuxiemeLigne " NorMD $NorMD"
    set LeMSF [linsert $LeMSF 1 $DeuxiemeLigne]
    Sauve [join $LeMSF "\n"] dans $FichierOut

    return $FichierOut
}
proc PipeNorMD {FichierMSF} {
    set CommandeMDScore "[ProgPathway normd] $FichierMSF /biolo/normd/gon250.bla 0 0.1"
    set MD ""
    if { ! [catch {set MD [eval exec $CommandeMDScore]} Message]} {
	if { ! [regexp {[0-9]} $MD]} {
	    #MDScore ne rend pas de nombre mais $Message
	    return -9999.4
	}		
	set Score ""
	scan $MD "%s %f" NomDuFich Score
	if {[regexp {^[0-9\-\+\.eE]+$} $Score]} {
	    set MD $Score
	} elseif {[regexp {^[0-9\-\+\.eE]+$} $NomDuFich]} {
	    set MD $NomDuFich
	} else {
	    set MD $MD
	}
	return $MD
    }
    #MDScore se plante par $Message
    return -9999.0
}
proc PipeRascal {FichierMSF FichierOut} {
    set OldNorMD [PipeNorMD $FichierMSF]
    if {[catch {exec [ProgPathway rascal] $FichierMSF $FichierOut} Message]} {
	Warne "Error from rascal. Message follows \n$Message"
	return ""
    }
    if { ! [file exists $FichierOut]} {
	Warne "Error from rascal. File not created"
	return ""
    }	
    if {[catch {exec [ProgPathway clustalw] $FichierOut -convert -output=gscope -outfile=$FichierOut.100} Message]} {
	Warne "Clustalw couldn't convert the msf to 100 : \n$Message"
    }
    if {[file exists "$FichierOut.100"]} {File rename -force "$FichierOut.100" $FichierOut}
        
    set NewNorMD [PipeNorMD             $FichierOut]
    set LeRascal [LesLignesDuFichier $FichierOut]
    set DeuxiemeLigne [string trimright [lindex [LesLignesDuFichier $FichierMSF] 1]]
    if {$OldNorMD<=$NewNorMD} {set Changed "improved"} else {set Changed "reduced"}
    regexp -nocase {(ANCHORS[ ]+[0-9]+)} $DeuxiemeLigne tmp DeuxiemeLigne

    append DeuxiemeLigne " Rascal $Changed NorMD from $OldNorMD to $NewNorMD"
    set LeRascal [linsert $LeRascal 1 $DeuxiemeLigne]

    return [Sauve [join $LeRascal "\n"] dans $FichierOut]
}
proc PipeClustering {FichierMSF FichierClustering {DPCouSecator ""}} {
    if {![file exists $FichierMSF]} {return}

    set NameFile [file tail $FichierMSF]
    regexp {^[^\.]+} $NameFile NameFile
    set FichierMSFTmp "[RepertoireDeTravail]/$NameFile"
    File copy -force $FichierMSF $FichierMSFTmp 

    if {$DPCouSecator == ""} {
        set NbAccess 0
	set F [open $FichierMSF]
	while {[gets $F Ligne]>=0} {    
	    set Entete [lindex [split [string trim $Ligne] " "] 0]
	    if {[string equal -nocase $Entete "Name:"]} {incr NbAccess}
	    if {$Entete == "//"} {break}
	}
	close $F

	if {100 < $NbAccess} {
	    set Moi    "dpc"
	    set LAutre "secator"
	    set Commande "eval exec [ProgPathway dpc]     $FichierMSFTmp alignment"
	    set FileClustering "${FichierMSFTmp}.dpc"
	} else {
	    set Moi    "secator"
	    set LAutre "dpc"
	    set Commande "eval exec [ProgPathway secator] $FichierMSFTmp"
	    set FileClustering "${FichierMSFTmp}.sec"
	}
	if {[catch {eval $Commande} Message]} {
	    Espionne $Message
	    Espionne "Problem with $FichierMSFTmp with $Moi je vais utiliser $LAutre"
	    return [ClusteringDuMSF $FichierMSF $FichierClustering $LAutre]
	}
	if {![file exists $FileClustering ]} {return}
	File rename -force $FileClustering $FichierClustering
    } else {
	if {[string equal -nocase $DPCouSecator "dpc"]} {
	    set Commande "eval exec [ProgPathway dpc]     $FichierMSFTmp alignment"
	    set FileClustering "${FichierMSFTmp}.dpc"
	}
	if {[string equal -nocase $DPCouSecator "secator"]} {
	    set Commande "eval exec [ProgPathway secator] $FichierMSFTmp"
	    set FileClustering "${FichierMSFTmp}.sec"
	}
	if {[catch {eval $Commande} Message]} {
	    Espionne $Message
	    Espionne "Problem with $FichierMSFTmp with $DPCouSecator"
	}
	if {![file exists $FileClustering ]} {return}
	File rename -force $FileClustering $FichierClustering
    }
    return $FichierClustering
}
proc FichierTFAsDesHitsDuBallast {FichierTFAs FichierBallast {NbHitsMax 2000} {NbPDBMax 30} {AvecFragment 0}} {
    set LesTFAs [LesTFAsDesHitsDuBallast $FichierBallast $NbHitsMax $NbPDBMax $AvecFragment]
    if {$LesTFAs != {}} {Sauve [join $LesTFAs "\n"] dans $FichierTFAs}
    return
}
proc LesHitsDuBallast {FichierBallast} {
    if {![file exists $FichierBallast]} {
	Espionne "$FichierBallast does not exist"
	return ""
    }
    set F [open $FichierBallast]
    set SuisJeSurLesHits 0
    set LesHits {}
    while {[gets $F Ligne]>=0} {
	if {$Ligne == ""} {continue}
	if {[regexp -nocase "^Sequences producing High-scoring Segment Pairs" $Ligne]} {
	    set SuisJeSurLesHits 1
	    continue
	}
	if {[regexp "^>" $Ligne]} {set SuisJeSurLesHits 0;break}
	if {!$SuisJeSurLesHits || [regexp "^\-" $Ligne]} {continue}

	set SuisJeUneReference 0

	if {![regexp "^   " $Ligne]} {set SuisJeUneReference 1}

	set Ligne [string trim $Ligne]
	regsub -all "\t" $Ligne " " Ligne
	while {[regexp "  " $Ligne]} {regsub -all "  " $Ligne " " Ligne}
	set LaLigne [split $Ligne " "]
	set Id      [string toupper [lindex $LaLigne 0]]
	set IdCourt $Id
	regsub {^[^\:]+\:} $Id "" IdCourt
	if {[EstUnAccessPDB $Id]} {set Access $IdCourt} else {set Access [string toupper [lindex $LaLigne 1]]}

	if {$SuisJeUneReference} {set Expect [lindex $LaLigne end]}

	set LesLMS [AskLMS $FichierBallast Access $IdCourt LesLMS]
	if {$LesLMS != ""} {
	    lappend LesHits [concat [list $IdCourt $Access $Expect] $LesLMS]
	    continue
	}
	set LesLMS [AskLMS $FichierBallast Access $Access LesLMS]
	if {$LesLMS != ""} {
	    lappend LesHits [concat [list $Access $Access $Expect] $LesLMS]
	    continue
	}
	lappend LesHits [list $Access $Access $Expect]
    }
    close $F
    return $LesHits
}
proc AskLMS args {
    global TabDesLMSEtDesMotifsDuBallast

    if {$args == ""} {return}
    set PremArg [lindex $args 0]
    if {[string equal -nocase $PremArg "unset"]} {
	unset TabDesLMSEtDesMotifsDuBallast
	return
    }
    if {![file exists $PremArg]} {return}
    set Quoi [join $args ","]
    if {[info exists TabDesLMSEtDesMotifsDuBallast($Quoi)]} {return [set TabDesLMSEtDesMotifsDuBallast($Quoi)]}
    if {[info exists TabDesLMSEtDesMotifsDuBallast($PremArg,EstCharge)]} {return}
    
    set     TabDesLMSEtDesMotifsDuBallast($PremArg,EstCharge) 1
    LoadLMS TabDesLMSEtDesMotifsDuBallast $PremArg

    if {[info exists TabDesLMSEtDesMotifsDuBallast($Quoi)]} {return [set TabDesLMSEtDesMotifsDuBallast($Quoi)]}
    return
}
proc LoadLMS {aTabLMS FichierBallast} {
    upvar $aTabLMS TabLMS

    set FichierMotifs "$FichierBallast.motifs"

    if {![file exists $FichierMotifs]} {return}

    set NbLMS 0
    set TabLMS($FichierBallast,LesLMS) {}
    set TabLMS($FichierBallast,LesAccess) {}

    set LesDebutsEtFins {}
    set F [open $FichierMotifs]
    while {[gets $F Ligne]>=0} {
	regsub -all { |\t} $Ligne "" Ligne
	if {![regexp {^[0-9]+\-[0-9]+} $Ligne]} {continue}

	set LesElements [split $Ligne ":"]
	set LesBornes [split [lindex $LesElements 0] "-"]

	set Debut     [Mini [lindex $LesBornes   0] [lindex $LesBornes   1]]
	set Fin       [Maxi [lindex $LesBornes   0] [lindex $LesBornes   1]]

	incr NbLMS
	lappend TabLMS($FichierBallast,LesLMS) $NbLMS

	set     TabLMS($FichierBallast,LMS,$NbLMS,Debut) $Debut
	set     TabLMS($FichierBallast,LMS,$NbLMS,Fin)   $Fin
	set     TabLMS($FichierBallast,LMS,$NbLMS,Motif) [lindex $LesElements 1]
	set     TabLMS($FichierBallast,LMS,$NbLMS,Score) [lindex $LesElements 2]
	set     TabLMS($FichierBallast,LMS,$NbLMS,LesAccess) {}
	if {![info exists TabLMS($FichierBallast,Debut,$Debut,Fin,$Fin,LesLMS)]} {set TabLMS($FichierBallast,Debut,$Debut,Fin,$Fin,LesLMS) {}}
	lappend           TabLMS($FichierBallast,Debut,$Debut,Fin,$Fin,LesLMS) $NbLMS
	lappend           LesDebutsEtFins                                      [list $Debut $Fin]
    }
    close $F
    set         TabLMS($FichierBallast,NbLMS) $NbLMS

    set FichierAnchors "$FichierBallast.anchors"
    if {![file exists $FichierAnchors]} {return}
    set F [open $FichierAnchors]
    while {[gets $F Ligne]>=0} {
	while {[regexp {  |\t} $Ligne]} {regsub -all {  |\t} $Ligne " " Ligne}
	if {$Ligne == ""} {continue}
	set LaLigne [split $Ligne " "]
	if {![string equal -nocase [lindex $LaLigne 0] "seq:"]} {continue}
	set Access1 [string toupper [lindex $LaLigne 1 ]]
	regsub {^[^\|]+\|} $Access1 "" Access1
	set Access2 [string toupper [lindex $LaLigne 2 ]]
	regsub {^[^\|]+\|} $Access2 "" Access2

	if {![info exists TabLMS($FichierBallast,Access,$Access1,LesLMS)]} {
	    lappend TabLMS($FichierBallast,LesAccess)              $Access1
	    set     TabLMS($FichierBallast,Access,$Access1,LesLMS) {}
	}
	if {![info exists TabLMS($FichierBallast,Access,$Access2,LesLMS)]} {
	    lappend TabLMS($FichierBallast,LesAccess)              $Access2
	    set     TabLMS($FichierBallast,Access,$Access2,LesLMS) {}
	}

	set Deb     [Mini [lindex $LaLigne 4 ] [lindex $LaLigne 6 ]] 
	set Fin     [Maxi [lindex $LaLigne 4 ] [lindex $LaLigne 6 ]] 
	set Score   [lindex $LaLigne 11]

	foreach LeDebutEtLaFin $LesDebutsEtFins {
	    set DLMS [lindex $LeDebutEtLaFin 0]
	    set FLMS [lindex $LeDebutEtLaFin 1]
	    if {![CesRegionsSontEllesOverlappantes $Deb $Fin $DLMS $FLMS]} {continue}
	    
	    foreach nLMS [set  TabLMS($FichierBallast,Debut,$DLMS,Fin,$FLMS,LesLMS)] {
		if {![info exists TabTmp($nLMS,$Access1)]} {
		    lappend TabLMS($FichierBallast,LMS,$nLMS,LesAccess) $Access1
		    set     TabTmp($nLMS,$Access1) 1
		    
		    lappend TabLMS($FichierBallast,Access,$Access1,LesLMS)          $nLMS
		    set     TabLMS($FichierBallast,Access,$Access1,LMS,$nLMS,Debut) $Deb
		    set     TabLMS($FichierBallast,Access,$Access1,LMS,$nLMS,Fin)   $Fin
		    set     TabLMS($FichierBallast,Access,$Access1,LMS,$nLMS,Score) $Score
		}
		if {![info exists TabTmp($nLMS,$Access2)]} {
		    lappend TabLMS($FichierBallast,LMS,$nLMS,LesAccess) $Access2
		    set     TabTmp($nLMS,$Access2) 1

		    lappend TabLMS($FichierBallast,Access,$Access2,LesLMS)          $nLMS
		    set     TabLMS($FichierBallast,Access,$Access2,LMS,$nLMS,Debut) $Deb
		    set     TabLMS($FichierBallast,Access,$Access2,LMS,$nLMS,Fin)   $Fin
		    set     TabLMS($FichierBallast,Access,$Access2,LMS,$nLMS,Score) $Score
		}
	    }
	}
    }
    close $F
    return $aTabLMS
}
proc LesProteinesDansLeMemeClusterDuMSF {FichierClust {ProteinQuery "QUERY_PROTEIN"}} {
    if {![file exists $FichierClust]} {return}
    set Cluster [InterrogeLesClustersDesMSF $FichierClust Access $ProteinQuery Cluster]
    if {$Cluster == ""} {return}
    return [InterrogeLesClustersDesMSF $FichierClust Cluster $Cluster LesAccess]
}
proc LesProteinesIdentiquesDuMSF {FichierMSF {ProteinQuery "QUERY_PROTEIN"} {IdMin 0.98}} {
    if {![file exists $FichierMSF]} {return}

    set LesAccess [AskMSF $FichierMSF LesAccess]

    set LesProteines {}
    foreach Access $LesAccess {
	set LesId [IDdansMSF $FichierMSF $ProteinQuery $Access SelonAEtB]
	set Id1 [lindex $LesId 0]
	set Id2 [lindex $LesId 1]

	if {$Id1 == "" || $Id1 < $IdMin} {continue}
	if {$Id2 == "" || $Id2 < $IdMin} {continue}
	lappend LesProteines $Access
    }
    return [lsort -unique $LesProteines]
}

proc AskBlast args {
    #$FileBlast Exists
    #unset 
    #unsetfile  $FileBlast
    #$FileBlast TypeDeBlast
    #$FileBlast Database
    #$FileBlast Query
    #$FileBlast Taille 

    #$FileBlast NbSubject

    #$FileBlast Subject $nSubject
    #$FileBlast BanqueId $BId Access
    #$FileBlast BanqueId $BId Definition
    #$FileBlast BanqueId $BId Taille
    #$FileBlast BanqueId $BId SubjectBrut

    #$FileBlast BanqueId $BId NbSegment
    #$FileBlast BanqueId $BId Segment $nSegment AlignBrut
    #$FileBlast BanqueId $BId Segment $nSegment Score
    #$FileBlast BanqueId $BId Segment $nSegment Expect
    #$FileBlast BanqueId $BId Segment $nSegment Identities
    #$FileBlast BanqueId $BId Segment $nSegment Positives
    #$FileBlast BanqueId $BId Segment $nSegment Gaps
    #$FileBlast BanqueId $BId Segment $nSegment Orientation
    #$FileBlast BanqueId $BId Segment $nSegment Align
    #$FileBlast BanqueId $BId Segment $nSegment AlignLineaire
    #$FileBlast BanqueId $BId Segment $nSegment AlignBrut
    #$FileBlast BanqueId $BId Segment $nSegment DQ
    #$FileBlast BanqueId $BId Segment $nSegment FQ
    #$FileBlast BanqueId $BId Segment $nSegment DS
    #$FileBlast BanqueId $BId Segment $nSegment FS

    global TableauContennantLesInformationsDuBlast

    if {$args == ""} {return}
    if {[string equal -nocase [lindex $args 0] "unset"]} {
	if {[info exists TableauContennantLesInformationsDuBlast]} {unset TableauContennantLesInformationsDuBlast}
	return
    }

    if {[string equal -nocase [lindex $args 0] "unsetfile"]} {
	set File [lindex $args 1]
	if {$File == "" || ![file exists $File] || ![info exists TableauContennantLesInformationsDuBlast]} {return}
	array unset TableauContennantLesInformationsDuBlast "$File*" 
	if {0} {
	    foreach Index [array name TableauContennantLesInformationsDuBlast] {
		if {![regexp "^$File" $Index]} {continue}
		unset TableauContennantLesInformationsDuBlast($Index)
	    }
	}
	return
    }

    set FileBlast [lindex $args 0]
    if {![file exists $FileBlast]} {return}

    set Quoi [join $args ","]

    if {[info exists TableauContennantLesInformationsDuBlast($Quoi)]} {return [set TableauContennantLesInformationsDuBlast($Quoi)]}
    if {[info exists TableauContennantLesInformationsDuBlast($FileBlast,Exists)]} {return}
    set              TableauContennantLesInformationsDuBlast($FileBlast,Exists) 1

    LoadBlast TableauContennantLesInformationsDuBlast $FileBlast
    if {[info exists TableauContennantLesInformationsDuBlast($Quoi)]} {return [set TableauContennantLesInformationsDuBlast($Quoi)]}
    return
}
proc LoadBlast {aTabBlast FichierBlast} {
    upvar $aTabBlast TabBlast

    set Compteur 0

    set ExpectDeRef  1e200
    set NbSubjectTot 1e200
    
    if {![file exists $FichierBlast]} {return}

    set SuisJeSurLesAlignements 0
    set SuisJeSurUnAlignement 0
    set ToutLeSubject ""
    set TousLesSubjects {}
    set j 0

    set SuisJeSurLeNomDeLaQuery 0

    set TailleMaxDUnFichier [expr {25*1e6}]
    if {$TailleMaxDUnFichier < [file size $FichierBlast]} {
	set NbHitsMax 10000
	Espionne "$FichierBlast is too long, we are going to study only the $NbHitsMax first hits"
    } else {
	set NbHitsMax "NoLimit"
    }
    #set NbHitsMax "NoLimit"

    set NomQuery {}
    set F [open $FichierBlast]
    set nHits 0
    while {[gets $F Ligne]>=0} {
	if {[string range $Ligne 0 6] == "Query= "} {
	    set SuisJeSurLeNomDeLaQuery 1
	    set Ligne [string range $Ligne 7 end]
	}
	if {!$SuisJeSurLesAlignements && [regexp "\[0-9\.]* letters" $Ligne]} {
	    set SuisJeSurLeNomDeLaQuery 0 
	}
	if {$SuisJeSurLeNomDeLaQuery} {
	    regsub "\n" $Ligne " " Ligne
	    lappend NomQuery [string trim $Ligne]
	}
	incr j
	if {$j == 1 || $j == 2} {
	    if {[string range $Ligne 0 4] == "BLAST" || [string range $Ligne 0 5] == "TBLAST"} {
		set LigneTrim   [string trim  $Ligne]
		set TypeDeBlast [string range $LigneTrim 0 [expr {[string first " " $LigneTrim]-1}]]
		set TabBlast($FichierBlast,TypeDeBlast) $TypeDeBlast
	    }
	}
	if {[string range $Ligne 0 11] == "  Database: " || [string range $Ligne 0 7] == "Matrix: "} {
	    if {$ToutLeSubject != ""} {lappend TousLesSubjects $ToutLeSubject}
	    regsub "\n" $Ligne "" Ligne
	    regsub " Database: " $Ligne "" Ligne
	    set TabBlast($FichierBlast,Database) [string trim $Ligne]
	    set SuisJeSurLesAlignements 0
	    break
	}
	if {[string range $Ligne 0 0] != ">" && $SuisJeSurLesAlignements == 0} {continue}
	if {[string range $Ligne 0 8] == " Score = "} {incr nHits}
	if {$NbHitsMax != "NoLimit" && $NbHitsMax < $nHits} {
	    if {$ToutLeSubject != ""} {lappend TousLesSubjects $ToutLeSubject}
	    break
	}
	if {[string range $Ligne 0 0] == ">"} {
	    if {$ToutLeSubject != ""} {lappend TousLesSubjects $ToutLeSubject}
	    set ToutLeSubject ""

	    set SuisJeSurLesAlignements 1
	    set SuisJeSurUnAlignement 1
	    append ToutLeSubject "$Ligne"
	} elseif {$SuisJeSurUnAlignement && $SuisJeSurLesAlignements} {
	    append ToutLeSubject "\n$Ligne"
	}	
    }
    close $F

    #Espionne ">> $TousLesSubjects"
    #Espionne ">> $nHits"
    
    set NbSubject 0
    foreach Subject  $TousLesSubjects {
	regsub -all {Expect[ 0-9\(\)]* =} $Subject "Expect =" Subject

	set AiJeAuMoinsUnExpectCorrect 0
	set Tmp $Subject
	while {[regexp { Expect =([ e0-9\.\-]*)} $Tmp ToutLExpect Expect]} {
	    set Expect  [string trim $Expect]
	    if {[string range $Expect 0 0] == "e"} {set Expect "1$Expect"}
	    if {$Expect <= $ExpectDeRef} {set AiJeAuMoinsUnExpectCorrect 1;break}
	    regsub { Expect =[ e0-9\.\-]*} $Tmp "" Tmp
	}
	unset Tmp

	if {!$AiJeAuMoinsUnExpectCorrect} {continue}
	set SubjectBrut $Subject
	####BanqueId et Access et definition et Taille######
	regexp -indices " Length = " $Subject DebFin
	set EnteteSubject [string trim [string range $Subject 1                  [lindex $DebFin 0]]]
	set Subject                    [string range $Subject [lindex $DebFin 0] end               ]

	regsub  -all "\n"    $EnteteSubject " " EnteteSubject
	regsub  -all {\(|\)} $EnteteSubject "_" EnteteSubject
	while {[regexp "  " $EnteteSubject]} {regsub -all "  " $EnteteSubject " " EnteteSubject}
	set LEnteteSubject [split $EnteteSubject " "]
	set BanqueId       [lindex $LEnteteSubject 0]
	#set BANQUEID       [string toupper $BanqueId]
	set Access         [lindex $LEnteteSubject 1]
	regsub "^_" $Access "" Access
	regsub "_$" $Access "" Access
	#set ACCESS         [string toupper $Access  ]

	set Definition     [join [lrange $LEnteteSubject 2 end] " "]
	if {$Access == ""} {
	    set Access     $BanqueId
	    set Definition ""
	} elseif {![regexp -nocase {^[a-z]} $Access] || [EstUnAccessPDB $BanqueId]} {
	    set Access     $BanqueId
	    set Definition [join [lrange $LEnteteSubject 1 end] " "]
	}
	#########Elimination en fonction de l'access ou du banqueid##################
	
	incr NbSubject
	if {$NbSubjectTot < $NbSubject} {incr NbSubject -1;break} 
	#############################################################################
	regexp -nocase { Length = ([0-9]+)[ ]*\n} $Subject tmp Taille 
	regsub         { Length = [0-9]+[ ]*\n}   $Subject ""  Subject

	set TabBlast($FichierBlast,Subject,$NbSubject)            $BanqueId
	set TabBlast($FichierBlast,BanqueId,$BanqueId,Access)      $Access
	set TabBlast($FichierBlast,BanqueId,$BanqueId,Definition)  $Definition
	set TabBlast($FichierBlast,BanqueId,$BanqueId,Taille)      $Taille
	set TabBlast($FichierBlast,BanqueId,$BanqueId,SubjectBrut) $SubjectBrut

	##############Split les alignements du subject##############
	set SuisJeSurUnAlignement 0
	set LesAlignements {}
	set LAlignement    {}
	foreach Ligne [split $Subject "\n"] {
	    if {[regexp -nocase "Score = " $Ligne]} {
		if {$LAlignement != {}} {
		    lappend LesAlignements [join $LAlignement "\n"]
		    set LAlignement {}
		}
		lappend LAlignement $Ligne 
		set SuisJeSurUnAlignement 1
	    } elseif {$SuisJeSurUnAlignement} {
		lappend LAlignement $Ligne
	    }
	}
	lappend LesAlignements [join $LAlignement "\n"]

	##############Decomposition des alignements###########
	set NbSegment 0
	foreach Alignement $LesAlignements {
	    set AlignementBrut $Alignement
	    incr NbSegment
	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,AlignBrut) $Alignement

	    set Score ""
	    set Expect ""
	    set Identities ""
	    set Gaps ""
	    set Orientation ""
	    set Positives ""
	    set Query ""
	    set Sbjct ""
	    set Inter ""
	    set AlignementLineaire "" 
	    set DebutQuery ""
	    set FinQuery ""
	    set DebutSbjct "" 
	    set FinSbjct ""
	    
	    regexp { Score = ([e0-9a-z\.\+\(\) ]+)} $Alignement tmp Score 
	    regsub { Score = [e0-9a-z\.\+\(\) ]+}   $Alignement "" Alignement
	    set Score [string trim $Score]

	    regexp { Expect =([ e0-9\.\-]+)} $Alignement tmp Expect
	    regsub { Expect =[ e0-9\.\-]+}   $Alignement ""  Alignement
	    set Expect [string trim $Expect]
	    if {[regexp "^e" $Expect]} { set Expect "1$Expect"}

	    regexp { Identities = ([0-9a-z\/\%\(\) ]+)} $Alignement tmp Identities 
	    regsub { Identities = [0-9a-z\/\%\(\) ]+}   $Alignement ""  Alignement
	    set Identities [string trim $Identities]

	    if {[regexp { Positives = ([0-9\/\%\(\) ]+)} $Alignement tmp Positives]} {
		regsub  { Positives = [0-9\/\%\(\) ]+}   $Alignement "" Alignement
	    }
	    set Positives [string trim $Positives]

	    if {[regexp { Gaps = ([0-9\/\%\(\) ]+)} $Alignement tmp Gaps]} {
		regsub  { Gaps = [0-9\/\%\(\) ]+}   $Alignement "" Alignement
	    }
	    set Gaps [string trim $Gaps]

	    regexp -nocase { [a-z]+ = ([0-9a-z\/\-\+ ]+)} $Alignement tmp Orientation 
	    regsub -nocase { [a-z]+ = [0-9a-z\/\-\+ ]+}   $Alignement ""  Alignement
	    set Orientation [string trim $Orientation]

	    regsub -all "," $Alignement "" Alignement
	    set Alignement [string trim $Alignement]
	    foreach Ligne  [split $Alignement "\n"] {
		if {$Query != "" && [regexp {^Query: [0-9 ]+} $Ligne]} {
		    regsub {[0-9 ]+$}   $Query ""  Query
		    append Query [string range $Ligne $n end]
		}
		if {$Sbjct != "" && [regexp {^Sbjct: [0-9 ]+} $Ligne]} {
		    regsub {[0-9 ]+$}   $Sbjct ""  Sbjct
		    append Sbjct [string range $Ligne $n end]
		}
		if {      $Inter != ""                          && \
			  [regexp -nocase {^  }         $Ligne] && \
			  [regexp -nocase {[a-z\|\-\+]} $Ligne]} {
		    append Inter [string range $Ligne $n end]
		}

		if {      $Query == "" && \
			  [regexp {^Query: ([0-9 ]+)} $Ligne tmp DebutQuery]} {
		    set n  [string length $tmp]
		    append Query $Ligne
		}
		if {      $Sbjct == "" && \
			  [regexp {^Sbjct: ([0-9 ]+)}  $Ligne tmp DebutSbjct]} {
		    append Sbjct $Ligne
		}
		if {      $Inter == ""                  && \
			  [regexp {^  }         $Ligne] && \
			  [regexp {[a-z\|\-\+]} $Ligne] } {
		    append Inter $Ligne
		}
	    }
	    regexp {([0-9]+)[ ]*$} $Query tmp FinQuery
	    regexp {([0-9]+)[ ]*$} $Sbjct tmp FinSbjct

	    if {$AlignementLineaire != ""} {set AlignementLineaire "\n"}
	    append AlignementLineaire      "$Query\n$Inter\n$Sbjct"

	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,Score)         $Score
	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,Expect)        $Expect
	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,Identities)    $Identities
	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,Positives)     $Positives
	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,Gaps)          $Gaps
	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,Orientation)   $Orientation
	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,Align)         $Alignement
	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,AlignLineaire) $AlignementLineaire
	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,AlignBrut)     $AlignementBrut
	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,DQ)            [string trim $DebutQuery]
	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,FQ)            [string trim $FinQuery  ]
	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,DS)            [string trim $DebutSbjct]
	    set TabBlast($FichierBlast,BanqueId,$BanqueId,Segment,$NbSegment,FS)            [string trim $FinSbjct  ]
	}
	set TabBlast($FichierBlast,BanqueId,$BanqueId,NbSegment) $NbSegment
    }
    set NomQuery [join $NomQuery " "]
    while {[regexp "  " $NomQuery]} {regsub -all "  " $NomQuery " " NomQuery}
    set TabBlast($FichierBlast,Query) $NomQuery

    set TabBlast($FichierBlast,Taille) [TailleDeLaQueryDuBlast $FichierBlast]

    set TabBlast($FichierBlast,NbSubject) $NbSubject

    return $aTabBlast
}

proc TailleDeLaQueryDuBlast {FichierBlast} {
    if {[file exists $FichierBlast]} {
	set F [open $FichierBlast]
	set nHits 0
	set Taille ""
	while {[gets $F Ligne]>=0} {
	    if {[regexp "\[0-9\,]* letters" $Ligne tmp]} {
		regsub " letters" $tmp "" Taille
		regsub "\," $Taille "" Taille
		break
	    }
	}
	close $F
	return "$Taille"
    } else {
	return ""
    }
}
proc FichierBlastReduit {FichierBlast NewFichierBlast {EnFonctionDe "Expect"} {SeuilOuValeurs "0.001"} {Keep ""}} {
    #EnFonctionDe -> 
    # LesBIdToKeep 
    # LesBIdToEliminate 
    # LesSegmentsDesBIdToKeep
    # LesAccessToKeep 
    # LesAccessToEliminate 
    # LesOrgaToKeep 
    # LesOrgaToEliminate 
    # LesmRNAsToKeep 
    # LesmRNAsToEliminate
    # SensToKeep 
    # SensToEliminate 
    # Expect 
    # Score     
    # JusteLePremierAlignementDesSubjects
    # LesRegionsToEliminate (Elimination des segments possedant ces regions)
    # LesRegionsToKeep (Segments possedant les regions a conserver)
    # NbSubjectMax
    # NbAliMax
    # TailleAlignementMin
    # TailleAlignementMax 
    # Positives 
    # PositivesWithoutFilter
    # Identity
    # IdentityWithoutFilter 
    # NbPositivesSurTailleQuery
    # NbPositivesSurTailleQueryWithoutFilter
    # RegionsAligneesSurRegionsAlignables
    # RegionsAligneesSurRegionsDansAlignement
    # RegionsAligneesSurRegionsDansAlignementSubject
    # RegionsAligneesSurTailleQuery
    # RegionsAligneesSurTailleSubject
    # RegionsAligneesSurTailleQueryDansLeMeilleurDesSens
    # RegionsAligneesSurTailleSubjectDansLeMeilleurDesSens

    set FichierTmp "[TmpFile].reduceblast"

    if {$FichierBlast != $NewFichierBlast} {
	if {[file exists $NewFichierBlast]} {
	    if {$Keep == ""} {
		#if {[OuiOuNon "$NewFichierBlast exists, Would you want to conserv it?"]} {set Keep 1} else {set Keep 0}
		set Keep 0
	    }
	    if {$Keep == 1 } {return}
	    File delete -force $NewFichierBlast
	}
    }

    set TypeDeBlast [AskBlast $FichierBlast TypeDeBlast]


    set LesBId {}
    foreach LeSubject    [LaSelectionDesSegmentsDuBlast $FichierBlast $EnFonctionDe $SeuilOuValeurs] {
	set BId          [lindex $LeSubject 0]
	lappend LesBId   $BId
	set ExpectDeRef  1e200
	set ScoreDeRef   0
	set LesSegments  [lindex $LeSubject 1]
	set TabTmp($BId) $LesSegments
	foreach Segment  $LesSegments {
	    set TabTmp($BId,$Segment) 1
	    set Score  [AskBlast $FichierBlast BanqueId $BId Segment $Segment Score ]
	    set Expect [AskBlast $FichierBlast BanqueId $BId Segment $Segment Expect]
	    scan $Score "%f" Score

	    if {$Expect     < $ExpectDeRef} {set ExpectDeRef $Expect}
	    if {$ScoreDeRef < $Score      } {set ScoreDeRef  $Score }
	}
	set TabTmp($BId,Score)  $ScoreDeRef
	set TabTmp($BId,Expect) $ExpectDeRef
    }

    set F [open $FichierBlast]
    set LeTexte {}
    while {[gets $F Ligne]>=0} {
	lappend LeTexte $Ligne
	if {$LesBId == {} && [regexp -nocase "^Searching......." $Ligne]} {break}
	if {[regexp -nocase "^Sequences producing significant" $Ligne]  } {break}
    }
    close $F
    AppendAuFichier $FichierTmp "[join $LeTexte "\n"]\n"

    if {$LesBId == {}} {
	AppendAuFichier $FichierTmp "\n***** No hits found ******\n"
    } else {
	set LeTexte {}
	foreach BId $LesBId {
	    set Access     [AskBlast $FichierBlast BanqueId $BId Access]
	    set Definition [AskBlast $FichierBlast BanqueId $BId Definition]
	    set Score [set TabTmp($BId,Score)]
	    set Expect [set TabTmp($BId,Expect)]
	    set Entete "$BId $Access $Definition"
	    if {[string length $Entete] < 67} {
		set Taille    [string length $Entete]
		set nDiff     [expr {67 - $Taille}]
		append Entete [string repeat " " $nDiff]
	    } else {
		set Entete [string range $Entete 0 66]
	    }
	    set ScoreEtExpect [format " %5s  %-6s" $Score $Expect]
	    append Entete $ScoreEtExpect
	    lappend LeTexte $Entete
	}
	AppendAuFichier $FichierTmp [join $LeTexte "\n"]

	set LeTexte {}
	foreach BId $LesBId {
	    set Access     [AskBlast $FichierBlast BanqueId $BId Access]
	    set Definition [AskBlast $FichierBlast BanqueId $BId Definition]
	    set Taille     [AskBlast $FichierBlast BanqueId $BId Taille]
	    set Description "\n>$BId $Access $Definition"
	    set AiJeDejaEcris 0
	    while {$Description != ""} {
		if {$AiJeDejaEcris} {set Description "          $Description"}
		set n [string length $Description]
		if {$n < 80} {
		    lappend LeTexte "$Description"
		    set Description ""
		} else {
		    lappend LeTexte [string range $Description 0 80]
		    set Description [string range $Description 81 end]
		}
		set AiJeDejaEcris 1
	    }
	    lappend LeTexte "          Length = $Taille\n"

	    if {![info exists TabTmp($BId)]} {continue}
	    foreach Seg [set TabTmp($BId)] {
		lappend LeTexte [AskBlast $FichierBlast BanqueId $BId Segment $Seg AlignBrut]
	    }
	}
	AppendAuFichier $FichierTmp [join $LeTexte "\n"]
    }
    if {[info exists TabTmp]} {unset TabTmp}
   
    AskBlast unsetfile $FichierBlast

    set F [open $FichierBlast]
    set LeTexte {}
    set SuisJeSurLaFinDuBlast 0
    while {[gets $F Ligne]>=0} {
	if {[regexp -nocase "^  Database: " $Ligne]} {set SuisJeSurLaFinDuBlast 1}
	if {$SuisJeSurLaFinDuBlast}                  {lappend LeTexte $Ligne}
    }
    close $F
    AppendAuFichier $FichierTmp [join $LeTexte "\n"]
    File rename -force $FichierTmp $NewFichierBlast
    return $NewFichierBlast
}
proc LaSelectionDesSegmentsDuBlast {FichierBlast {EnFonctionDe "Expect"} {SeuilOuValeurs "0.001"}} {
    Wup "EnFonctionDe -> LesBIdToKeep , LesBIdToEliminate , LesSegmentsDesBIdToKeep, LesAccessToKeep , LesAccessToEliminate , LesOrgaToKeep , LesOrgaToEliminate , LesmRNAsToKeep, LesmRNAsToEliminate, Expect , Score , SensToKeep, SensToEliminate , Identity, IdentityWithoutFilter, Positives, PositivesWithoutFilter, NbPositivesSurTailleQuery, NbPositivesSurTailleQueryWithoutFilter, TailleAlignementMin, TailleAlignementMax , LesRegionsToEliminate (Elimination des segments possedant ces regions),  LesRegionsToKeep (Segments possedant les regions a conserver), NbSubjectMax, NbAliMax, JusteLePremierAlignementDesSubjects,RegionsAligneesSurRegionsAlignables, RegionsAligneesSurRegionsDansAlignement, RegionsAligneesSurRegionsDansAlignementSubject, RegionsAligneesSurTailleQuery, RegionsAligneesSurTailleSubject, RegionsAligneesSurTailleQueryDansLeMeilleurDesSens, RegionsAligneesSurTailleSubjectDansLeMeilleurDesSens"
    

    set LesSegmentsAConserver {}

    set TypeDeBlast        [AskBlast $FichierBlast TypeDeBlast]
    if {$TypeDeBlast == ""} {return}

    set Mask "X"
    if {[string equal -nocase "BLASTN" $TypeDeBlast]} {set Mask "N"}

    set NbSubjectDansBlast [AskBlast $FichierBlast NbSubject]
    if {$NbSubjectDansBlast == "" || $NbSubjectDansBlast < 1} {return}

    set TailleQuery        [AskBlast $FichierBlast Taille   ]

    if {$EnFonctionDe == "LesSegmentsDesBIdToKeep"} {
	foreach LeBId $SeuilOuValeurs {
	    set BId         [lindex $LeBId 0]
	    set LesSegments [lrange $LeBId 1 end]
	    lappend LesSegmentsAConserver [list $BId $LesSegments]
	}
	return $LesSegmentsAConserver
    }

    if {$EnFonctionDe == "LesmRNAsToKeep"} {
	set LesBId {}
	for {set i 1} {$i <= $NbSubjectDansBlast} {incr i} {lappend LesBId [AskBlast $FichierBlast Subject $i]}
	foreach BId [LesmRNAsDesAcNucs $LesBId] {
	    set LesSegments {}
	    set NbSegment [AskBlast $FichierBlast BanqueId $BId NbSegment]
	    for {set j 1} {$j <= $NbSegment} {incr j} {lappend LesSegments $j}
	    Espionne $BId
	    lappend LesSegmentsAConserver [list $BId $LesSegments]
	}
	return $LesSegmentsAConserver
    }
    if {$EnFonctionDe == "LesmRNAsToEliminate"} {
	set LesBId {}
	for {set i 1} {$i <= $NbSubjectDansBlast} {incr i} {lappend LesBId [AskBlast $FichierBlast Subject $i]}

	foreach BId [LesNonmRNAsDesAcNucs $LesBId] {
	    set LesSegments {}
	    set NbSegment [AskBlast $FichierBlast BanqueId $BId NbSegment]
	    for {set j 1} {$j <= $NbSegment} {incr j} {lappend LesSegments $j}
	    Espionne $BId
	    lappend LesSegmentsAConserver [list $BId $LesSegments]
	}
	return $LesSegmentsAConserver
    }

    set k 0
    set l 0
    for {set i 1} {$i <= $NbSubjectDansBlast} {incr i} {
	incr k 
	if {$EnFonctionDe == "NbSubjectMax" && $SeuilOuValeurs < $k} {break}

	set BId    [AskBlast $FichierBlast Subject $i]

	set LesSegments {}
	set NbSegment [AskBlast $FichierBlast BanqueId $BId NbSegment]
	for {set j 1} {$j <= $NbSegment} {incr j} {lappend LesSegments $j}

	if {$EnFonctionDe == "LesBIdToEliminate"} {
	    set LAiJeVu 0
	    set BIdCourt $BId
	    regsub -nocase {^[^\:]+\:} $BId "" BIdCourt
	    foreach BIdDeRef $SeuilOuValeurs {
		regsub {^[^\:]+\:} $BIdDeRef "" BIdDeRef
		if {[string equal -nocase $BIdDeRef $BIdCourt]} {set LAiJeVu 1;break}
	    }
	    if {!$LAiJeVu} {lappend LesSegmentsAConserver [list $BId $LesSegments]}
	    continue
	}
	if {$EnFonctionDe == "LesBIdToKeep"} {
	    set LAiJeVu 0
	    set BIdCourt $BId
	    regsub -nocase {^[^\:]+\:} $BId "" BIdCourt
	    foreach BIdDeRef $SeuilOuValeurs {
		regsub {^[^\:]+\:} $BIdDeRef "" BIdDeRef
		if {[string equal -nocase $BIdDeRef $BIdCourt]} {set LAiJeVu 1;break}
	    }
	    if {$LAiJeVu} {lappend LesSegmentsAConserver [list $BId $LesSegments]}
	    continue
	}
	set Access [AskBlast $FichierBlast BanqueId $BId Access]
	if {$EnFonctionDe == "LesAccessToEliminate"} {
	    set LAiJeVu 0
	    foreach AccDeRef $SeuilOuValeurs {
		if {[string equal -nocase $AccDeRef $Access]} {set LAiJeVu 1;break}
	    }
	    if {!$LAiJeVu} {lappend LesSegmentsAConserver [list $BId $LesSegments]}
	    continue
	}
	if {$EnFonctionDe == "LesAccessToKeep"} {
	    set LAiJeVu 0
	    foreach AccDeRef $SeuilOuValeurs {
		#Espionne $Access
		if {[string equal -nocase $AccDeRef $Access]} {set LAiJeVu 1;break}
	    }
	    if {$LAiJeVu} {lappend LesSegmentsAConserver [list $BId $LesSegments]}
	    continue
	}
	if {$EnFonctionDe == "LesOrgaToEliminate"} {
	    set EnAiJeVuUn 0
	    foreach Orga [LesOrgasDeLAccess $BId] {
		if {[regexp -nocase $Orga [join $SeuilOuValeurs "\t"]]} {set EnAiJeVuUn 1;break}
	    }
	    if {!$EnAiJeVuUn} {lappend LesSegmentsAConserver [list $BId $LesSegments]}
	    continue
	}
	if {$EnFonctionDe == "LesOrgaToKeep"} {
	    set EnAiJeVuUn 0
	    foreach Orga [LesOrgasDeLAccess $BId] {
		#Espionne "$BId\t$Orga\t[join $SeuilOuValeurs "\t"]"
		if {[regexp -nocase $Orga [join $SeuilOuValeurs "\t"]]} {set EnAiJeVuUn 1;break}
	    }
	    if {$EnAiJeVuUn} {lappend LesSegmentsAConserver [list $BId $LesSegments]}
	    continue
	}

	if {0 && $EnFonctionDe == "LesmRNAsToKeep"} {
	    set PageInfo [PageInfo $BId genbank]
	    if {[EstCeUnmRNACompleteCDS_PageInfo $PageInfo]} {
		#Espionne $BId
		lappend LesSegmentsAConserver [list $BId $LesSegments]
	    }
	    continue
	}
	if {0 && $EnFonctionDe == "LesmRNAsToEliminate"} {
	    set PageInfo [PageInfo $BId genbank]
	    if {![EstCeUnmRNACompleteCDS_PageInfo $PageInfo]} {
		#Espionne $BId
		lappend LesSegmentsAConserver [list $BId $LesSegments]
	    }
	    continue
	}
	
	if {$EnFonctionDe == "RegionsAligneesSurRegionsAlignables"} {
	    set P [PourcentageDesRegionsAligneesSurToutesLesRegionsAlignables $FichierBlast $BId]
	    if {$SeuilOuValeurs < $P} {
		#Espionne "$BId RegionsAligneesSurRegionsAlignables $SeuilOuValeurs $P"
		lappend LesSegmentsAConserver [list $BId $LesSegments]
	    }
	    continue
	}
	if {$EnFonctionDe == "RegionsAligneesSurRegionsDansAlignement"} {
	    set P [PourcentageDIdentiteGlobal $FichierBlast $BId "Query"]
	    if {$SeuilOuValeurs < $P} {
		#Espionne "$BId RegionsAligneesSurRegionsDansAlignement $SeuilOuValeurs $P"
		lappend LesSegmentsAConserver [list $BId $LesSegments]
	    }
	    continue
	}
	if {$EnFonctionDe == "RegionsAligneesSurRegionsDansAlignement_Subject"} {
	    set P [PourcentageDIdentiteGlobal $FichierBlast $BId "Subject"]
	    if {$SeuilOuValeurs < $P} {
		#Espionne "$BId RegionsAligneesSurRegionsDansAlignement_Subject $SeuilOuValeurs $P"
		lappend LesSegmentsAConserver [list $BId $LesSegments]
	    }
	    continue
	}
	if {$EnFonctionDe == "RegionsAligneesSurTailleQuery"} {
	    set P [PourcentageDeRecouvrement  $FichierBlast $BId "Query"]
	    if {$SeuilOuValeurs < $P} {
		#Espionne "$BId RegionsAligneesSurTailleQuery $SeuilOuValeurs $P"
		lappend LesSegmentsAConserver [list $BId $LesSegments]
	    }
	    continue
	}
	if {$EnFonctionDe == "RegionsAligneesSurTailleSubject"} {
	    set P [PourcentageDeRecouvrement  $FichierBlast $BId "Subject"]
	    if {$SeuilOuValeurs < $P} {
		#Espionne "$BId RegionsAligneesSurTailleSubject $SeuilOuValeurs $P"
		lappend LesSegmentsAConserver [list $BId $LesSegments]
	    }
	    continue
	}
	if {$EnFonctionDe == "RegionsAligneesSurTailleQueryDansLeMeilleurDesSens"} {
	    set nSPlus  [NombreDeBasesEnCommun $FichierBlast $BId "Query" "+"]
	    set nSMoins [NombreDeBasesEnCommun $FichierBlast $BId "Query" "-"]
	    if {$nSPlus < $nSMoins} {
		set P  [expr {$nSMoins * 1.0 / $TailleQuery}]
		set SensSelect "-"
	    } else {
		set SensSelect "+"
		set P  [expr {$nSPlus  * 1.0 / $TailleQuery}]
	    }
	    if {$SeuilOuValeurs < $P} {
		#Espionne "$BId RegionsAligneesSurTailleQuery $SeuilOuValeurs $P"
		set LesSegments {}
		for {set j 1} {$j <= $NbSegment} {incr j} {
		    set Orient [AskBlast $FichierBlast BanqueId $BId Segment $j Orientation] 
		    if {[SensDeLaRegion $Orient] == $SensSelect} {lappend LesSegments $j}
		}
		lappend LesSegmentsAConserver [list $BId $LesSegments]
	    }
	    continue
	}                     
	if {$EnFonctionDe == "RegionsAligneesSurTailleSubjectDansLeMeilleurDesSens"} {
	    set nSPlus  [NombreDeBasesEnCommun $FichierBlast $BId "Subject" "+"]
	    set nSMoins [NombreDeBasesEnCommun $FichierBlast $BId "Subject" "-"]

	    if {$nSPlus < $nSMoins} {
		set P  [expr {$nSMoins * 1.0 / [AskBlast $FichierBlast BanqueId $BId Taille]}]
		set SensSelect "-"
	    } else {
		set SensSelect "+"
		set P  [expr {$nSPlus  * 1.0 / [AskBlast $FichierBlast BanqueId $BId Taille]}]
	    }
	    if {$SeuilOuValeurs < $P} {
		#Espionne "$BId RegionsAligneesSurTailleSubjectDansLeMeilleurDesSens $SeuilOuValeurs $P"
		set LesSegments {}
		for {set j 1} {$j <= $NbSegment} {incr j} {
		    set Orient [AskBlast $FichierBlast BanqueId $BId Segment $j Orientation] 
		    if {[SensDeLaRegion $Orient] == $SensSelect} {lappend LesSegments $j}
		}
		lappend LesSegmentsAConserver [list $BId $LesSegments]
	    }
	    continue
	}
	set LesSegments {}
	for {set j 1} {$j <= $NbSegment} {incr j} {

	    if {$EnFonctionDe == "JusteLePremierAlignementDesSubjects"} {
		if {$j == 1} {
		    #Espionne "$BId JusteLePremierAlignementDesSubjects segment $j"
		    lappend LesSegments $j
		    break
		}
		continue
	    }

	    incr l
	    if {$EnFonctionDe == "NbAliMax" && $SeuilOuValeurs < $l} {break}

	    if {$EnFonctionDe == "Expect"} {
		if {[AskBlast $FichierBlast BanqueId $BId Segment $j Expect] <= $SeuilOuValeurs}       {lappend LesSegments $j}
		continue
	    }
	    if {$EnFonctionDe == "Score"} {
		if {[AskBlast $FichierBlast BanqueId $BId Segment $j Score] <= $SeuilOuValeurs}        {lappend LesSegments $j}
		continue
	    }
	    if {$EnFonctionDe == "SensToKeep"} {
		if {[AskBlast $FichierBlast BanqueId $BId Segment $j Orientation] == $SeuilOuValeurs} {lappend LesSegments $j}
		continue
	    }
	    if {$EnFonctionDe == "SensToEliminate"} {
		if {[AskBlast $FichierBlast BanqueId $BId Segment $j Orientation] != $SeuilOuValeurs} {lappend LesSegments $j}
		continue
	    }

	    if {$EnFonctionDe == "Identity"} {
		set Identities [AskBlast $FichierBlast BanqueId $BId Segment $j Identities]
		scan $Identities "%d/%d" n nTot
		set  Id        [expr {$n*1.0/$nTot}]
		#Espionne "$BId\t$Id\t$SeuilOuValeurs"
		if {$SeuilOuValeurs <= $Id} {lappend LesSegments $j}
		continue
	    }
	    if {$EnFonctionDe == "IdentityWithoutFilter"} {
		set NbBasesFiltrees [NombreDeBasesFiltrees $FichierBlast $BId Segment $j]
		set Identities [AskBlast $FichierBlast BanqueId $BId Segment $j Identities]
		scan $Identities "%d/%d" n nTot
		set  Id        [expr {$n*1.0/($nTot- $NbBasesFiltrees)}]
		#Espionne "$BId\t$Id\t$SeuilOuValeurs"
		if {$SeuilOuValeurs <= $Id} {lappend LesSegments $j}
		continue
	    }
	    if {$EnFonctionDe == "Positives"} {
		set Positives [AskBlast $FichierBlast BanqueId $BId Segment $j Positives]
		scan $Positives "%d/%d" n nTot
		set  Pos       [expr {$n*1.0/$nTot}]
		#Espionne "$BId\t$Pos\t$SeuilOuValeurs"
		if {$SeuilOuValeurs <= $Pos} {lappend LesSegments $j}
		continue
	    }
	    if {$EnFonctionDe == "PositivesWithoutFilter"} {
		set NbBasesFiltrees [NombreDeBasesFiltrees $FichierBlast $BId Segment $j]
		set Positives [AskBlast $FichierBlast BanqueId $BId Segment $j Positives]
		scan $Positives "%d/%d" n nTot
		set  Pos        [expr {$n*1.0/($nTot- $NbBasesFiltrees)}]
		#Espionne "$BId\t$Pos\t$SeuilOuValeurs"
		if {$SeuilOuValeurs <= $Pos} {lappend LesSegments $j}
		continue
	    }
	    if {$EnFonctionDe == "NbPositivesSurTailleQueryWithoutFilter"} {
		set Positives [AskBlast $FichierBlast BanqueId $BId Segment $j Positives]
		set NbBasesFiltrees [NombreDeBasesFiltrees $FichierBlast $BId Segment $j]
		scan $Positives "%d" n
		set Pos [expr {$n * 1.0 / ($TailleQuery - $NbBasesFiltrees)}]
		#Espionne "$BId\t$Pos\t$SeuilOuValeurs"
		if {$SeuilOuValeurs <= $Pos} {lappend LesSegments $j}
		continue
	    }

	    if {$EnFonctionDe == "NbPositivesSurTailleQuery"} {
		set Positives [AskBlast $FichierBlast BanqueId $BId Segment $j Positives]
		scan $Positives "%d" n
		set Pos [expr {$n * 1.0 / $TailleQuery}]
		#Espionne "$BId\t$Pos\t$SeuilOuValeurs"
		if {$SeuilOuValeurs <= $Pos} {lappend LesSegments $j}
		continue
	    }

	    set DQ          [Mini [AskBlast $FichierBlast BanqueId $BId Segment $j DQ] [AskBlast $FichierBlast BanqueId $BId Segment $j FQ]]
	    set FQ          [Maxi [AskBlast $FichierBlast BanqueId $BId Segment $j DQ] [AskBlast $FichierBlast BanqueId $BId Segment $j FQ]]
	    if {$EnFonctionDe == "TailleAlignementMin"} {
		set TailleAli   [expr {$FQ-$DQ+1}]
		if {$SeuilOuValeurs <= $TailleAli     }                      {
		    #Espionne "$BId segment $j TailleAli $TailleAli $SeuilOuValeurs"
		    lappend LesSegments $j
		}
		continue
	    }
	    if {$EnFonctionDe == "TailleAlignementMax"} {
		set TailleAli   [expr {$FQ-$DQ+1}]
		if {$TailleAli      <= $SeuilOuValeurs}                      {lappend LesSegments $j}
		continue
	    }
	    if {$EnFonctionDe == "LesRegionsToKeep"} {
		set EnAiJeUne 0
		foreach LaRegion $SeuilOuValeurs {
		    set Debut [Mini [lindex $LaRegion 0] [lindex $LaRegion 1]]
		    set Fin   [Maxi [lindex $LaRegion 0] [lindex $LaRegion 1]]
		    #Espionne "$DQ $FQ $Debut $Fin"
		    if {[CesRegionsSontEllesOverlappantes $DQ $FQ $Debut $Fin]} {
			set EnAiJeUne 1
			break
		    }
		}
		if {$EnAiJeUne}                                          {lappend LesSegments $j}
		continue
	    }
	    if {$EnFonctionDe == "LesRegionsToEliminate"} {
		set EnAiJeUne 0
		foreach LaRegion $SeuilOuValeurs {
		    set Debut [Mini [lindex $LaRegion 0] [lindex $LaRegion 1]]
		    set Fin   [Maxi [lindex $LaRegion 0] [lindex $LaRegion 1]]
		    #Espionne "$DQ $FQ $Debut $Fin"
		    if {[CesRegionsSontEllesOverlappantes $DQ $FQ $Debut $Fin]} {
			set EnAiJeUne 1
			break
		    }
		}
		if {!$EnAiJeUne}                                          {lappend LesSegments $j}
		continue
	    }
	    lappend LesSegments $j
	}
	if {$LesSegments != {}} {lappend LesSegmentsAConserver [list $BId $LesSegments]}
    }
    return $LesSegmentsAConserver
}

################################
proc PourcentageDIdentiteDuSegmentDeLId {FichierBlast BanqueId {Segment 1} {WithoutFilter 1}} {
    if {![file exists $FichierBlast]} {return ""}

    set nF 0
    if {$WithoutFilter} {set nF [NombreDeBasesFiltrees $FichierBlast $BanqueId $Segment]}
    set Id [AskBlast $FichierBlast BanqueId $BanqueId Segment $Segment Identities]
    if {$Id == ""} {return}
    regexp {([0-9]+)/([0-9]+)} $Id tmp n nTot

    return [expr {$n * 1.0 / ($nTot - $nF)}]
}
proc PourcentageDeRecouvrementDuSegment {FichierBlast BanqueId {Segment 1} {QueryOuSubject "Query"}} {
    Wup "Nombre de bases alignees de la query/subject divisee par la taille de la query/subject"

    set Identities [AskBlast $FichierBlast BanqueId $BanqueId Segment $Segment Identities]
    if {$Identities == ""} {return}
    set  n 0
    scan $Identities "%d/%d" n nTot

    if {[string equal -nocase "Query" $QueryOuSubject]} {
	set Taille [AskBlast $FichierBlast Taille]
    } else {
	set Taille [AskBlast $FichierBlast BanqueId $BanqueId Taille]
    }
    return [expr {$n * 1.0 / $Taille}]
}
proc PourcentageDeRecouvrement {FichierBlast BanqueId {QueryOuSubject "Query"}} {
    Wup "Nombre de bases alignees de la query/subject divisee par la taille de la query/subject"

    set NbBasesEnCommun [NombreDeBasesEnCommun $FichierBlast $BanqueId $QueryOuSubject]
    if {$NbBasesEnCommun == ""} {return}
    if {[string equal -nocase "Query" $QueryOuSubject]} {
	set Taille [AskBlast $FichierBlast Taille]
    } else {
	set Taille [AskBlast $FichierBlast BanqueId $BanqueId Taille]
    }
    return [expr {$NbBasesEnCommun * 1.0 / $Taille}]
}
proc PourcentageDIdentiteGlobal {FichierBlast BanqueId {Quoi "Query"}} {
    set n    [NombreDeBasesEnCommun           $FichierBlast $BanqueId $Quoi]
    set NTot [NombreDeBasesDansLesAlignements $FichierBlast $BanqueId $Quoi]
    if {$NTot == 0 || $n == "" || $NTot == ""} {return}
    return   [expr {$n * 1.0 / $NTot}]
}
proc PourcentageDesBasesAligneesSurToutesLesBasesAlignables {FichierBlast BanqueId} {
    set N    [NombreDeBasesEnCommun   $FichierBlast $BanqueId]
    set NTot [NombreDeBasesAlignables $FichierBlast $BanqueId]
    if {$NTot == 0 || $N == "" || $NTot == ""} {return 0}
    return [expr {$N * 1.0 / $NTot}]
}
proc PourcentageDesRegionsAligneesSurToutesLesRegionsAlignables {FichierBlast BanqueId} {
    set N    [NombreDeBasesDansLesAlignements $FichierBlast $BanqueId]
    set NTot [NombreDeBasesAlignables         $FichierBlast $BanqueId]
    if {$NTot == 0 || $N == "" || $NTot == ""} {return 0}
    return [expr {$N * 1.0 / $NTot}]
}
proc NombreDeBasesAlignables {FichierBlast BanqueId} {

    Wup "Nb de bases que l'on aurait pu aligner au maximum (Max - Min + 1)"
    Wup "-------------XXXXX----XXXXXX---------------"
    Wup "       |                                  |"
    Wup "       ------XXXXX----XXXXXX--------------------"
    Wup "       |                                  |"
    Wup "      Min                                Max"

    set NbSegments [AskBlast $FichierBlast BanqueId $BanqueId NbSegment]
    if {$NbSegments == ""} {return}

    set TailleQ [AskBlast $FichierBlast Taille]
    set TailleS [AskBlast $FichierBlast BanqueId $BanqueId Taille]
    set DQMin 1e200
    set FQMax 0
    set DSMin 1e200
    set FSMax 0
    for {set Segment 1} {$Segment <= $NbSegments} {incr Segment} {
	set DQ [Mini [AskBlast $FichierBlast BanqueId $BanqueId Segment $Segment DQ] [AskBlast $FichierBlast BanqueId $BanqueId Segment $Segment FQ]]
	set FQ [Maxi [AskBlast $FichierBlast BanqueId $BanqueId Segment $Segment DQ] [AskBlast $FichierBlast BanqueId $BanqueId Segment $Segment FQ]]
	set DS [Mini [AskBlast $FichierBlast BanqueId $BanqueId Segment $Segment DS] [AskBlast $FichierBlast BanqueId $BanqueId Segment $Segment FS]]
	set FS [Maxi [AskBlast $FichierBlast BanqueId $BanqueId Segment $Segment DS] [AskBlast $FichierBlast BanqueId $BanqueId Segment $Segment FS]]

	if {$DQ    < $DQMin} {set DQMin $DQ}
	if {$FQMax < $FQ   } {set FQMax $FQ}	
	if {$DS    < $DSMin} {set DSMin $DS}
	if {$FSMax < $FS   } {set FSMax $FS}	
    }
    set DistFQMaxToEnd [expr {$TailleQ - $FQMax + 1}]
    set DistFSMaxToEnd [expr {$TailleS - $FSMax + 1}]
    if {$DSMin          < $DQMin         } {set Debut [expr {$DQMin - $DSMin + 1}                         ]} else {set Debut 1       }
    if {$DistFSMaxToEnd < $DistFQMaxToEnd} {set Fin   [expr {$TailleQ - $DistFQMaxToEnd + $DistFSMaxToEnd}]} else {set Fin   $TailleQ}

    return [expr {$Fin - $Debut + 1}]
}
proc NombreDeBasesFiltrees {FichierBlast BanqueId {Segment 1} {Quoi "Query"}} {
    if {![file exists $FichierBlast]} {return 0}

    Wup "Nb de bases filtrees dans le Segment aligne (A) par blast ou repeatmasker (X)"
    Wup "   ----------AAAAXXXAAAAAAAAAA---------------"
    Wup "   000000000000001110000000000000000000000000  "  
    Wup " "  
    Wup "   NbFiltree = Somme des 1"

    set TypeDeBlast [AskBlast $FichierBlast TypeDeBlast]
    if {$TypeDeBlast == ""} {return}
    if {$TypeDeBlast == "BLASTX" || $TypeDeBlast == "BLASTP" || $TypeDeBlast == "TBLASTN" || $TypeDeBlast == "TBLASTX"} {
	set BaseFiltree "X"
    } elseif {$TypeDeBlast == "BLASTN"} {
	set BaseFiltree "n"
    }
    if {[AskBlast $FichierBlast BanqueId $BanqueId NbSegment                     ] == ""} {return 0}
    set AlignLineaire [AskBlast $FichierBlast BanqueId $BanqueId Segment $Segment AlignLineaire]
    if {$AlignLineaire == ""} {return 0}
    if {[string equal -nocase $Quoi "Query"]} {
	set LAli       [split [lindex [split $AlignLineaire "\n"] 0] ""]
    } else {
	set LAli       [split [lindex [split $AlignLineaire "\n"] 2] ""]
    }
    set nFiltres 0
    foreach Base $LAli {if {$Base == $BaseFiltree} {incr nFiltres}}
    return $nFiltres
}

proc NombreDeBasesDansLesAlignements {FichierBlast BanqueId {Quoi "Query"}} {
    if {![file exists $FichierBlast]} {return 0}

    Wup "   Nb de bases dans les alignements de la query/subject"
    Wup "   -------------XXX----XXXX--------------- "
    Wup "                | |    |  |                "
    Wup "           -----XXX----XXXX---------       "
    Wup "   000000000000011100001111000000000000000 "   
    Wup "   "
    Wup "   Nb de bases dans les alignements = Somme des 1"

    set NbBasesDansLesAlignements 0

    set NbSubjectDansBlast [AskBlast $FichierBlast NbSubject]
    if {$NbSubjectDansBlast == ""} {return}

    for {set i 1} {$i <= $NbSubjectDansBlast} {incr i} {
	set BId         [AskBlast $FichierBlast Subject $i]

	set BId_Court $BId
	#Modif Jean
	#^[0-9]+_[0-9]+$ pour les access de jean
	if {[regexp "_" $BId_Court] && [regexp {^[a-zA-Z]{,2}[0-9]+_[0-9]+$} $BId_Court]} {
	    regsub {_[0-9]+$} $BId "" BId_Court
	}
	#Espionne "2 $BId_Court vs $BId vs $BanqueId"
	if {$BId_Court != $BanqueId} {continue}

	set NbSegments [AskBlast $FichierBlast BanqueId $BId NbSegment]
	for {set Segment 1} {$Segment <= $NbSegments} {incr Segment} {
	    if {[string equal -nocase $Quoi "Query"]} {
		set D [AskBlast $FichierBlast BanqueId $BId Segment $Segment DQ]
		set F [AskBlast $FichierBlast BanqueId $BId Segment $Segment FQ]
	    } else {
		set D [AskBlast $FichierBlast BanqueId $BId Segment $Segment DS]
		set F [AskBlast $FichierBlast BanqueId $BId Segment $Segment FS]
	    }
	    if {$F < $D} {set tmp $D;set D $F;set F $tmp}
	    for {set j $D} {$j <= $F} {incr j} {
		if {![info exists TabTmp($j)]} {incr NbBasesDansLesAlignements}
		set TabTmp($j) 1
	    }
	}
    }
    if {[info exists TabTmp]} {unset TabTmp}

    return $NbBasesDansLesAlignements
}
proc NombreDeBasesEnCommun {FichierBlast BanqueId {Quoi "Query"} {WhatSens "BothSens"}} {
    if {![file exists $FichierBlast]} {return 0}

    Wup "   Nb de bases alignes (A) dans les segments alignes (X) de la query/subject"
    Wup "   ----------AAAXAAXA----AAXXAAXXA---------------  "
    Wup "             ||| || |    ||  ||  |                 "
    Wup "          ---AAAXAAXA----AAXXAAXXA-------          "
    Wup "   0000000000111011010000110011001000000000000000  Ex par rapport  Query"  
    Wup "   "
    Wup "   Nb de bases alignees = Somme des 1"


    if {[string equal -nocase $Quoi "Query"]} {
	set LeProfil [LeProfilDeLAlignementDeLaQuery $FichierBlast $BanqueId $WhatSens]
    } else {
	set LeProfil [LeProfilDeLAlignementDuSubject $FichierBlast $BanqueId $WhatSens]
    }    

    set NbBasesEnCommun 0    
    foreach Base $LeProfil {if {$Base != 0} {incr NbBasesEnCommun}}
    return $NbBasesEnCommun
}
proc LeProfilDeLAlignementDeLaQuery {FichierBlast BanqueId {WhatSens "BothSens"}} {
    if {![file exists $FichierBlast]} {return ""}

    set TypeDeBlast [AskBlast $FichierBlast TypeDeBlast]
    if {$TypeDeBlast == ""} {return}

    set Indice 1
    if {$TypeDeBlast == "BLASTX" || $TypeDeBlast == "TBLASTX"} {set Indice 3}

    set TailleQuery [AskBlast $FichierBlast Taille]

    #^[0-9]+_[0-9]+$ pour les access de jean
    #regsub {_[0-9]+$} $BanqueId "" BanqueId
    if {[regexp "_" $BanqueId] && [regexp {^[a-zA-Z]{,2}[0-9]+_[0-9]+$} $BanqueId]} {
	regsub {_[0-9]+$} $BanqueId "" BanqueId
    }

    for {set i 1} {$i <= $TailleQuery} {incr i} {set TabProfil($i) 0}

    set NbSubjectDansBlast [AskBlast $FichierBlast NbSubject]
    for {set j 1} {$j <= $NbSubjectDansBlast} {incr j} {
	set BId         [AskBlast $FichierBlast Subject $j]

	if {[info exists BId_Court]} {unset BId_Court}
	#^[0-9]+_[0-9]+$ pour les access de jean
	#regsub {_[0-9]+$} $BId "" BId_Court
	if {[regexp "_" $BId] && [regexp {^[a-zA-Z]{,2}[0-9]+_[0-9]+$} $BId]} {
	    regsub {_[0-9]+$} $BId "" BId_Court
	} else {
	    set BId_Court $BId
	}
	if {![string equal -nocase $BId_Court $BanqueId]} {continue}

	set NbSegments [AskBlast $FichierBlast BanqueId $BId NbSegment]
	for {set Segment 1} {$Segment <= $NbSegments} {incr Segment} {
	    set AlignLineaire [AskBlast $FichierBlast BanqueId $BId Segment $Segment AlignLineaire]
	    if {$AlignLineaire == ""} {return}
	    set Sens [SensDeLaRegion [AskBlast $FichierBlast BanqueId $BId Segment $Segment Orientation]]
	    if {$WhatSens != "BothSens" && $Sens != $WhatSens} {continue}

	    set DQ [AskBlast $FichierBlast BanqueId $BId Segment $Segment DQ]
	    set FQ [AskBlast $FichierBlast BanqueId $BId Segment $Segment FQ]

	    if {[info exists AliQuery]} {unset AliQuery}
	    if {[info exists AliSbjct]} {unset AliSbjct}

	    set Query [lindex [split $AlignLineaire "\n"] 0]
	    scan $Query "%s %d %s %d" Entete D AliQuery F
	    set LAliQuery [split $AliQuery ""]
	    
	    set Sbjct [lindex [split $AlignLineaire "\n"] 2]
	    scan $Sbjct "%s %d %s %d" Entete D AliSbjct F
	    set LAliSbjct [split $AliSbjct ""]
	    
	    if {![info exists AliQuery]} {continue}
	    if {![info exists AliSbjct]} {continue}
	    
	    set Sens "+"
	    if {$FQ < $DQ} {set Sens "-"}
	    
	    set PosIni $DQ
	    foreach AQ $LAliQuery AS $LAliSbjct {
		if {$AQ == "-"} {continue}
		if {$AQ == $AS} {
		    for {set i 0} {$i < $Indice} {incr i} {
			if {$Sens == "-"} {
			    set Pos [expr {$PosIni - $i}]
			} else {
			    set Pos [expr {$PosIni + $i}]
			}
			if {![info exists TabProfil($Pos)]} {continue}
			incr TabProfil($Pos) 
		    }
		}
		if {$Sens == "-"} {
		    set PosIni [expr {$PosIni - $Indice}]
		} else {
		    set PosIni [expr {$PosIni + $Indice}]
		}
	    }
	}
    }
    set LeProfil {}
    for {set i 1} {$i <= $TailleQuery} {incr i} {lappend LeProfil [set TabProfil($i)]}

    return $LeProfil
}
proc LeProfilDeLAlignementDuSubject {FichierBlast BanqueId {WhatSens "BothSens"}} {
    if {![file exists $FichierBlast]} {return ""}

    set TypeDeBlast [AskBlast $FichierBlast TypeDeBlast]
    if {$TypeDeBlast == ""} {return}

    set Indice 1
    if {$TypeDeBlast == "TBLASTN" || $TypeDeBlast == "TBLASTX"} {set Indice 3}

    set TailleSbjct [AskBlast $FichierBlast BanqueId $BanqueId Taille]
    for {set i 1} {$i <= $TailleSbjct} {incr i} {set TabProfil($i) 0}

    set NbSubjectDansBlast [AskBlast $FichierBlast NbSubject]
    for {set j 1} {$j <= $NbSubjectDansBlast} {incr j} {
	
	set BId         [AskBlast $FichierBlast Subject $j]
	if {0} {
	    if {[info exists BId_Court]} {unset BId_Court}
	    regsub {_[0-9]+$} $BId "" BId_Court
	    if {$BId_Court != $BanqueId} {continue}
	} else {
	    if {![string equal -nocase $BId $BanqueId]} {continue}
	}

	set NbSegments [AskBlast $FichierBlast BanqueId $BId NbSegment]
	for {set Segment 1} {$Segment <= $NbSegments} {incr Segment} {
	    set AlignLineaire [AskBlast $FichierBlast BanqueId $BId Segment $Segment AlignLineaire]
	    if {$AlignLineaire == ""} {return}
	    set Orientation   [AskBlast $FichierBlast BanqueId $BId Segment $Segment Orientation]
	    set Sens         [SensDeLaRegion $Orientation]
	    if {$WhatSens != "BothSens" && $Sens != $WhatSens} {continue}

	    set DS   [AskBlast $FichierBlast BanqueId $BId Segment $Segment DS           ]
	    set FS   [AskBlast $FichierBlast BanqueId $BId Segment $Segment FS           ]
	    
	    set Query [lindex [split $AlignLineaire "\n"] 0]
	    scan $Query "%s %d %s %d" Entete D AliQuery F
	    set LAliQuery [split $AliQuery ""]
	    
	    set Sbjct [lindex [split $AlignLineaire "\n"] 2]
	    scan $Sbjct "%s %d %s %d" Entete D AliSbjct F
	    set LAliSbjct [split $AliSbjct ""]

	    set Sens "+"
	    if {$FS < $DS} {set Sens "-"}
	    
	    set PosIni $DS
	    foreach AQ $LAliQuery AS $LAliSbjct {
		if {[regexp -nocase "\\\-" $AS]} {continue}
		
		if {$AQ == $AS} {
		    for {set i 0} {$i < $Indice} {incr i} {
			if {$Sens == "-"} {
			    set Pos [expr {$PosIni - $i}]
			} else {
			    set Pos [expr {$PosIni + $i}]
			}
			if {![info exists TabProfil($Pos)]} {continue}
			incr TabProfil($Pos) 
		    }
		}
		if {$Sens == "-"} {
		    set PosIni [expr {$PosIni - $Indice}]
		} else {
		    set PosIni [expr {$PosIni + $Indice}]
		}
	    }
	}
    }
    set LeProfil {}
    for {set i 1} {$i <= $TailleSbjct} {incr i} {lappend LeProfil [set TabProfil($i)]}

    return $LeProfil
}
proc LeProfilDeLaQuerySurToutLeBlast {FichierBlast {BaseAligneOuBaseDansAlignementOuLesDeux "BaseAligne"} {WhatSens "BothSens"} {LesBanqueId ""} {Expect 1e200} } {
    if {![file exists $FichierBlast]} {return ""}

    foreach BId $LesBanqueId {
	regsub {_[0-9]+$} $BId "" BId
#	if {[regexp "_" $BId] && [regexp {^[a-zA-Z]{,2}[0-9]+_[0-9]+$} $BId]} {
#	    regsub {_[0-9]+$} $BId "" BId
#	}
	set TabBIdOk($BId) 1
    }

    Espionne "LeProfilDeLaQuerySurToutLeBlast"

    set TypeDeBlast [AskBlast $FichierBlast TypeDeBlast]
    if {$TypeDeBlast == ""} {return}

    set Indice 1
    if {$TypeDeBlast == "BLASTX" || $TypeDeBlast == "TBLASTX"} {set Indice 3}

    set TailleQuery [AskBlast $FichierBlast Taille]
    for {set i 1} {$i <= $TailleQuery} {incr i} {
	set TabProfil(BaseAligne,$i)         0
	set TabProfil(BaseDansAlignement,$i) 0
    }

    set NbSubjectDansBlast [AskBlast $FichierBlast NbSubject  ]
    for {set j 1} {$j <= $NbSubjectDansBlast} {incr j} {
	set BId         [AskBlast $FichierBlast Subject $j]

	if {[info exists BId_Court]} {unset BId_Court}
	regsub {_[0-9]+$} $BId "" BId_Court
#	if {[regexp "_" $BId_Court] && [regexp {^[a-zA-Z]{,2}[0-9]+_[0-9]+$} $BId_Court]} {
#	    regsub {_[0-9]+$} $BId "" BId_Court
#	}
	if {$LesBanqueId != "" && ![info exists TabBIdOk($BId_Court)]} {continue}

	set NbSegments [AskBlast $FichierBlast BanqueId $BId NbSegment]
	for {set Segment 1} {$Segment <= $NbSegments} {incr Segment} {
	    set AlignLineaire [AskBlast $FichierBlast BanqueId $BId Segment $Segment AlignLineaire]
	    if {$AlignLineaire == ""} {return}
	    set DQ   [AskBlast $FichierBlast BanqueId $BId Segment $Segment DQ         ]
	    set FQ   [AskBlast $FichierBlast BanqueId $BId Segment $Segment FQ         ]

	    set Orientation [AskBlast $FichierBlast BanqueId $BId Segment $Segment Orientation]
	    set Sens        [SensDeLaRegion $Orientation]
	    if {$WhatSens != "BothSens" && $Sens != $WhatSens} {continue}

	    set Query [lindex [split $AlignLineaire "\n"] 0]
	    scan $Query "%s %d %s %d" Entete D AliQuery F
	    set LAliQuery [split $AliQuery ""]
	    
	    set Sbjct [lindex [split $AlignLineaire "\n"] 2]
	    scan $Sbjct "%s %d %s %d" Entete D AliSbjct F
	    set LAliSbjct [split $AliSbjct ""]
	    
	    set Sens "+"
	    if {$FQ < $DQ} {set Sens "-"}
	    
	    set PosIni $DQ
	    foreach AQ $LAliQuery AS $LAliSbjct {
		if {$AQ == "-"} {continue}
		for {set i 0} {$i < $Indice} {incr i} {
		    if {$Sens == "-"} {set Pos [expr {$PosIni-$i}]} else {set Pos [expr {$PosIni+$i}]}
		    incr TabProfil(BaseDansAlignement,$Pos)
		    if {$AQ == $AS} {
			if {![info exists TabProfil(BaseAligne,$Pos)]} {continue}
			incr TabProfil(BaseAligne,$Pos) 
		    }
		}
		if {$Sens == "-"} {set PosIni [expr {$PosIni-$Indice}]} else {set PosIni [expr {$PosIni+$Indice}]}
	    }
	}
    }
    set LeProfil {}
    for {set i 1} {$i <= $TailleQuery} {incr i} {
	if {[string equal -nocase $BaseAligneOuBaseDansAlignementOuLesDeux "BaseAligne"        ]} {
	    lappend LeProfil [set TabProfil(BaseAligne,$i)]
	    continue
	}
	if {[string equal -nocase $BaseAligneOuBaseDansAlignementOuLesDeux "BaseDansAlignement"]} {
	    lappend LeProfil [set TabProfil(BaseDansAlignement,$i)]
	    continue
	}
    if {    [string equal -nocase $BaseAligneOuBaseDansAlignementOuLesDeux "LesDeux"           ]} {
	    lappend LeProfil [list [set TabProfil(BaseAligne,$i)] [set TabProfil(BaseDansAlignement,$i)]]
	    continue
	}
    }
    return $LeProfil
}
proc LesProfilsDeLaQuerySurToutLeBlast {FichierBlast {BaseAligneOuBaseDansAlignementOuLesDeux "BaseAligne"} {WhatSens "BothSens"} {LesBanqueId ""} {Expect 1e200} } {
    if {![file exists $FichierBlast]} {return ""}

    foreach BId $LesBanqueId {
	#Modif Jean
	#regsub {_[0-9]+$} $BId "" BId
	if {[regexp "_" $BId] && [regexp {^[a-zA-Z]{,2}[0-9]+_[0-9]+$} $BId]} {
	    regsub {_[0-9]+$} $BId "" BId
	}

	set TabBIdOk($BId) 1
    }

    set TypeDeBlast [AskBlast $FichierBlast TypeDeBlast]
    if {$TypeDeBlast == ""} {return}

    set Indice 1
    if {$TypeDeBlast == "BLASTX" || $TypeDeBlast == "TBLASTX"} {set Indice 3}

    set TailleQuery [AskBlast $FichierBlast Taille]
    set LesBId     {}
    set NbSubjectDansBlast [AskBlast $FichierBlast NbSubject  ]
    for {set j 1} {$j <= $NbSubjectDansBlast} {incr j} {
	set BId         [AskBlast $FichierBlast Subject $j]

	if {[info exists BId_Court]} {unset BId_Court}
	#Modif Jean
	#regsub {_[0-9]+$} $BId "" BId_Court
	if {[regexp "_" $BId] && [regexp {^[a-zA-Z]{,2}[0-9]+_[0-9]+$} $BId]} {
	    regsub {_[0-9]+$} $BId "" BId_Court
	} else {
	    set BId_Court $BId
	}

	if {$LesBanqueId != "" && ![info exists TabBIdOk($BId_Court)]} {continue}
	
	if {![info exists  TabProfil($BId_Court,1)]} {
	    lappend LesBId $BId_Court
	    for {set i 1} {$i <= $TailleQuery} {incr i} {
		set TabProfil($BId_Court,BaseAligne,$i)         0
		set TabProfil($BId_Court,BaseDansAlignement,$i) 0
	    }
	}

	set NbSegments [AskBlast $FichierBlast BanqueId $BId NbSegment]
	for {set Segment 1} {$Segment <= $NbSegments} {incr Segment} {
	    set AlignLineaire [AskBlast $FichierBlast BanqueId $BId Segment $Segment AlignLineaire]
	    if {$AlignLineaire == ""} {return ""}
	    set Orientation [AskBlast $FichierBlast BanqueId $BId Segment $Segment Orientation]
	    set Sens        [SensDeLaRegion $Orientation]
	    if {$WhatSens != "BothSens" && $Sens != $WhatSens} {continue}

	    set DQ   [AskBlast $FichierBlast BanqueId $BId Segment $Segment DQ         ]
	    set FQ   [AskBlast $FichierBlast BanqueId $BId Segment $Segment FQ         ]


	    set Query [lindex [split $AlignLineaire "\n"] 0]
	    scan $Query "%s %d %s %d" Entete D AliQuery F
	    set LAliQuery [split $AliQuery ""]
	    
	    set Sbjct [lindex [split $AlignLineaire "\n"] 2]
	    scan $Sbjct "%s %d %s %d" Entete D AliSbjct F
	    set LAliSbjct [split $AliSbjct ""]
	    
	    set Sens "+"
	    if {$FQ < $DQ} {set Sens "-"}
	    
	    set PosIni $DQ
	    foreach AQ $LAliQuery AS $LAliSbjct {
		if {$AQ == "-"} {continue}
		for {set i 0} {$i < $Indice} {incr i} {
		    if {$Sens == "-"} {set Pos [expr {$PosIni-$i}]} else {set Pos [expr {$PosIni+$i}]}
		    incr TabProfil($BId_Court,BaseDansAlignement,$Pos)
		    if {$AQ == $AS} {
			if {![info exists TabProfil($BId_Court,BaseAligne,$Pos)]} {continue}
			incr TabProfil($BId_Court,BaseAligne,$Pos)
		    }
		}
		if {$Sens == "-"} {set PosIni [expr {$PosIni-$Indice}]} else {set PosIni [expr {$PosIni+$Indice}]}
	    }
	}
    }

    set LesProfils {}
    foreach BId $LesBId {
	set LeProfil {}
	for {set i 1} {$i <= $TailleQuery} {incr i} {
	    if {[string equal -nocase $BaseAligneOuBaseDansAlignementOuLesDeux "BaseAligne"]} {
		lappend LeProfil [set TabProfil($BId,BaseAligne,$i)]
		continue
	    }
	    if {[string equal -nocase $BaseAligneOuBaseDansAlignementOuLesDeux "BaseDansAlignement"]} {
		lappend LeProfil [set TabProfil($BId,BaseDansAlignement,$i)]
		continue
	    }
	    if {[string equal -nocase $BaseAligneOuBaseDansAlignementOuLesDeux "LesDeux"]} {
		lappend LeProfil [list [set TabProfil($BId,BaseAligne,$i)] [set TabProfil($BId,BaseDansAlignement,$i)]]
		continue
	    }
	}
	lappend LesProfils [list $BId $LeProfil]
    }
    return $LesProfils
}
proc LaFrequenceDesPetitesRegionsDeLaQueryDuBlast {FichierBlast {ExpectMax 1e200} {NbSubjectMax 1e200} {IdMin 0.0} {SplitWithGap "Yes"} {LesBanqueId ""}} {
    set TailleQ [AskBlast $FichierBlast Taille]

    set LesGrandesRegions  [LesGrandesRegionsDeLaQueryDuBlast   $FichierBlast      $ExpectMax $NbSubjectMax $IdMin $SplitWithGap $LesBanqueId]

    set LesPetitesRegions  [LesPetitesRegionsDesGrandesRegions  $LesGrandesRegions                                                           ]
    foreach LaRegion       [LesRegionsAbsentesDesPetitesRegions $LesPetitesRegions 1 $TailleQ                                                ] {
	lappend LesPetitesRegions $LaRegion
    }

    set LesFrequencesDesPetitesRegions {}
    foreach LaPetiteRegion [lsort -index 0 -real $LesPetitesRegions] {
	set DPR [lindex $LaPetiteRegion 0]
	set FPR [lindex $LaPetiteRegion 1]
	set i 0
	foreach LaGrandeRegion $LesGrandesRegions {
	    set DGR [lindex $LaGrandeRegion 0]
	    set FGR [lindex $LaGrandeRegion 1]
	    if {[CesRegionsSontEllesOverlappantes $DPR $FPR $DGR $FGR]} {incr i}
	}
	lappend LesFrequencesDesPetitesRegions [list $DPR $FPR $i]
    }
    return $LesFrequencesDesPetitesRegions
}
proc LesGrandesRegionsDeLaQueryDuBlast {FichierBlast {ExpectMax 1e200} {NbSubjectMax 1e200} {IdMin 0.0} {SplitWithGap "Yes"} {LesBanqueId ""}} {
    foreach BId $LesBanqueId {set TabSelect($BId) 1}

    set NbSubject   [AskBlast $FichierBlast NbSubject  ]

    set TypeDeBlast [AskBlast $FichierBlast TypeDeBlast]
    if {$TypeDeBlast == "" || $NbSubject == ""} {return}

    set k 0
    set LesGrandesRegions {}
    for {set i 1} {$i <= $NbSubject} {incr i} {
	set BId [AskBlast $FichierBlast Subject $i]

	if {$LesBanqueId != "" && ![info exists TabSelect($BId)]} {continue}
	set Expect [AskBlast $FichierBlast BanqueId $BId Segment 1 Expect]
	if {$Expect == ""} {continue}

	incr k
	
	if {$NbSubjectMax < $k     } {break}
	if {$ExpectMax    < $Expect} {continue}

	set NbSegment [AskBlast $FichierBlast BanqueId $BId NbSegment]
	for {set j 1} {$j <= $NbSegment} {incr j} {
	    set Expect [AskBlast $FichierBlast BanqueId $BId Segment $j Expect]
	    if {$ExpectMax < $Expect} {continue}
	    if {0 < $IdMin} {
		set Identities [AskBlast $FichierBlast BanqueId $BId Segment $j Identities]
		scan $Identities "%d/%d" nmRNA nmRNATot
		set  Id        [expr {$nmRNA*1.0/$nmRNATot}]
		if {$Id < $IdMin} {continue}
	    }
	    
	    set DQ [Mini [AskBlast $FichierBlast BanqueId $BId Segment $j DQ] [AskBlast $FichierBlast BanqueId $BId Segment $j FQ]] 
	    set FQ [Maxi [AskBlast $FichierBlast BanqueId $BId Segment $j DQ] [AskBlast $FichierBlast BanqueId $BId Segment $j FQ]] 
	    
	    if {$SplitWithGap == "Yes"} {
		if {$FQ < $DQ} {set FactSensQ -1 } else {set FactSensQ 1 }
		
		set AlignLineaire  [AskBlast $FichierBlast BanqueId $BId Segment $j AlignLineaire]
		set LAlignLineaire [split $AlignLineaire "\n"]

		set Query [lindex $LAlignLineaire 0]
		scan $Query "%s %d %s %d" Entete D AliQuery F
		set LAliQuery [split $AliQuery ""]

		set Sbjct [lindex $LAlignLineaire 2]
		scan $Sbjct "%s %d %s %d" Entete D AliSbjct F
		set LAliSbjct [split $AliSbjct ""]
		
		if {[info exists Debut]} {unset Debut}
		set PosIni $DQ
		foreach AQ $LAliQuery AS $LAliSbjct {
		    if {0 && $AQ == "$BaseFiltree"} {
			if {[info exists Debut]} {
			    lappend LesGrandesRegions [list $Debut $Fin]
			    unset Debut
			}
			set PosIni [expr {$PosIni + 1 * $FactSensQ}]
			continue
		    }
		    if {$AS == "-" || $AS == "*" || $AQ == "*"} {
			if {[info exists Debut]} {
			    lappend LesGrandesRegions [list $Debut $Fin]
			    unset Debut
			}
			set PosIni [expr {$PosIni + 1 * $FactSensQ}]
			continue
		    }
		    if {$AQ == "-"} {
			if {[info exists Debut]} {
			    lappend LesGrandesRegions [list $Debut $Fin]
			    unset Debut
			}
			continue
		    }
		    if {![info exist Debut]} {set Debut $PosIni}
		    set Fin    [expr {$PosIni}                 ]
		    set PosIni [expr {$PosIni + 1 * $FactSensQ}]
		}
		if {[info exists Debut]} {lappend LesGrandesRegions [list $Debut $Fin];unset Debut}
	    } else {
		lappend LesGrandesRegions [list $DQ $FQ]
	    }
	}
    }
    return $LesGrandesRegions
}
proc LesRegionsProchesFusionnees {LesRegions DistanceMax} {
    
    foreach LaRegion [lsort -index 0 -real [LesPetitesRegionsDesGrandesRegions $LesRegions]] {
	




    }


}



