#################################################################################
# CADO4MI - CADO4MI: Computer Assisted Design of Oligonucleotide For MIcroarray #
#                                                                               #
# CADO4MI is a program to design oligonucleotide for microarrays or PCR.        #
#                                                                               #
# Copyright (C) 2006 Jean Muller (jean.muller@igbmc.u-strasbg.fr)               # 
#                                                                               #
# Please cite the following article:                                            #
#    Muller J et al                                                             #
#    Design and evaluation of Actichip,                                         #
#    a thematic microarray for the study of the actin cytoskeleton.             #
#    BMC Genomics 2007 Aug 29;8(1):294.                                         #
#                                                                               #
# This is part of CADO4MI source code.                                          #
#                                                                               #
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public License                   # 
# as published by the Free Software Foundation; either version 2                # 
# of the License, or (at your option) any later version.                        #
#                                                                               #
# This program is distributed in the hope that it will be useful,               # 
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
# GNU General Public License for more details.                                  #
#                                                                               #
# You should have received a copy of the GNU General Public License             #
# along with this program; if not, write to the Free Software                   #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                     #
#################################################################################

## These traces also work on array routines, but you have to be careful,
## because ALL elements of the array will be traced with the same procedure.
## You can place multiple traces on each variable (ie - int and len 8).
## Mail jeff.hobbs@acm.org with problems/questions.
##
## This first routine would be used for ensuring that an array value
## would always exist.  It would be used with a read trace like so:
## % set default 1
## % set array(0) $default
## % trace variable array r "forceValue $default"
## Now any other accesses to the array will ensure at least the default value.
proc forceValue {default name el op} {
    global $name
    if [string comp {} $el] { set name $name\($el) }
    if ![info exists $name] { set $name $default }
}

proc forceInt {name el op} {
    global $name ${name}_int
    if [string comp {} $el] {
	set old  ${name}_int\($el)
	set name $name\($el)
    } else { set old ${name}_int }
    if ![regexp {^[-+]?[0-9]*$} [set $name]] {
	set $name [set $old]
	bell; return
    }
    set $old [set $name]
}

proc forceReal {name el op} {
    global $name ${name}_real
    if [string comp {} $el] {
	set old  ${name}_real\($el)
	set name $name\($el)
    } else { set old ${name}_real }
    if ![regexp {^[-+]?[0-9]*\.?[0-9]*([0-9]\.?e[-+]?[0-9]*)?$} [set $name]] {
	set $name [set $old]
	bell; return
    }
    set $old [set $name]
}

proc forceRegexp {regexp name el op} {
    global $name ${name}_regexp
    if [string comp {} $el] {
	set old  ${name}_regexp\($el)
	set name $name\($el)
    } else { set old ${name}_regexp }
    if ![regexp $regexp [set $name]] {
	set $name [set $old]
	bell; return
    }
    set $old [set $name]
}

proc forceAlpha {name el op} {
    global $name ${name}_alpha
    if [string comp {} $el] {
	set old  ${name}_alpha\($el)
	set name $name\($el)
    } else { set old ${name}_alpha }
    if ![regexp {^[a-zA-Z]*$} [set $name]] {
	set $name [set $old]
	bell; return
    }
    set $old [set $name]
}

proc forceLen {len name el op} {
    global $name ${name}_len
    if [string comp $el {}] {
	set old  ${name}_len\($el)
	set name $name\($el)
    } else { set old ${name}_len }
    if {[string length [set $name]] > $len} {
	set $name [set $old]
	bell; return
    }
    set $old [set $name]
}

## entryLimit - a convenience proc for managing traces on entry widgets
## Perhaps to make this even nicer, it could create a textvar for the
## user with the name the same as the entry?
# ARGS:	entry		- the entry widget to trace
#	traceProc	- trace procedure
#	init		- initial value to use, defaults to {}
## If traceProc=={}, then all write traces on the entry's textvar are
## deleted.  This doesn't delete array traces.
##
proc entryLimit {entry traceProc {init {}}} {
    if [string match {} [set var [$entry cget -textvariable]]] {
	return -code error "-textvariable not set for entry \"$entry\""
    }

    if {[string compare $traceProc {}]} {
	## TextVars are always considered global
	uplevel \#0 [list set $var $init]
	uplevel \#0 [list trace variable $var w $traceProc]
	if {[catch {uplevel \#0 [list set $var $init]} err]} {
	    ## If this errors out, the $init value was bad, or
	    ## something was wrong with the traceProc
	    return -code error "an error was received setting the initial\
		    value of \"$var\" to \"$init\".  Make sure the value is\
		    valid and the traceProc is functional:\n$err"
	} else {
	    ## Do you really want to delete the trace when
	    ## destroying the entry?
	    #bind $entry <Destroy> [list + trace vdelete $var w $traceProc]
	    return
	}
    }
    foreach p [uplevel \#0 [list trace vinfo $var]] {
	if {[string match w [lindex $p 0]]} {
	    uplevel \#0 trace vdelete [list $var] $p
	}
    }
}
