#################################################################################
# CADO4MI - CADO4MI: Computer Assisted Design of Oligonucleotide For MIcroarray #
#                                                                               #
# CADO4MI is a program to design oligonucleotide for microarrays or PCR.        #
#                                                                               #
# Copyright (C) 2006 Jean Muller (jean.muller@igbmc.u-strasbg.fr)               # 
#                                                                               #
# Please cite the following article:                                            #
#    Muller J et al                                                             #
#    Design and evaluation of Actichip,                                         #
#    a thematic microarray for the study of the actin cytoskeleton.             #
#    BMC Genomics 2007 Aug 29;8(1):294.                                         #
#                                                                               #
# This is part of CADO4MI source code.                                          #
#                                                                               #
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public License                   # 
# as published by the Free Software Foundation; either version 2                # 
# of the License, or (at your option) any later version.                        #
#                                                                               #
# This program is distributed in the hope that it will be useful,               # 
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
# GNU General Public License for more details.                                  #
#                                                                               #
# You should have received a copy of the GNU General Public License             #
# along with this program; if not, write to the Free Software                   #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                     #
#################################################################################

proc LesIdDeLaBanqueRelieALAccess {Access de Banque1 dans Banque2} {
    set Commande "[ProgPathway getz] \"\\\[${Banque1}-acc:$Access\\\] > $Banque2 \""
    catch {set Out [eval exec $Commande]} Message
    set Out [string trim $Out]
    if {$Out == ""} {
	set Commande "ProgPathway getz] \"\\\[${Banque1}-id:$Access\\\] > $Banque2 \""
	catch {set Out [eval exec $Commande]} Message
	if {$Out == ""} {return}
    }
    set LesId {}
    foreach Ligne [split $Out "\n"] {
	if {$Ligne == ""} {continue}
	if {[info exists Id]} {unset Id}
	regsub {^[^:]+:} $Ligne "" Id
	if {![info exists Id]} {continue}
	lappend LesId $Id
    }
    return $LesId
}
proc InfoDuChampsDeLaPageGenBank {PageGenBank NOMDUCHAMPS} { 
    global RepertoireDuGenome

    set LesInfosNettoyes {}
    foreach Ligne [LeChampsDeLaPageGenBank $PageGenBank $NOMDUCHAMPS] {
	regsub "$NOMDUCHAMPS [ ]*" $Ligne "" Ligne
	set Ligne [string trim $Ligne]
	while {[regexp "  " $Ligne] } {regsub "  " $Ligne " " Ligne}

	lappend LesInfosNettoyes $Ligne
    }
    return [join $LesInfosNettoyes " "]
}

proc LeChampsDeLaPageGenBank {PageGenBank NOMDUCHAMPS} { 
    global RepertoireDuGenome
    
    set LaPageGenBank [split $PageGenBank "\n"]

    set SuisJeSurLeBonChamps 0
    set LAiJeVu              0
    set LesInfosDuChamps {}
    foreach Ligne $LaPageGenBank {
	if {!$SuisJeSurLeBonChamps && $LAiJeVu} {break}

	if { $SuisJeSurLeBonChamps == 1 && [regexp "        " $Ligne]} {
	    lappend LesInfosDuChamps " $Ligne"
	} else {
	    set SuisJeSurLeBonChamps 0
	}
	if {    [regexp "$NOMDUCHAMPS " $Ligne] && $LAiJeVu == 0} {
	    lappend LesInfosDuChamps $Ligne
	    set LAiJeVu              1
	    set SuisJeSurLeBonChamps 1
	}
    }
    return "$LesInfosDuChamps"
}
proc LesAccessDeLaPageInfo {PageInfo} {
    set LesAccess {}
    foreach Ligne [split $PageInfo "\n"] {
	if {[regexp -nocase {^ACCESSION  } $Ligne tmp]} {
	    regsub $tmp $Ligne "" Ligne
	    set Ligne [string trim $Ligne]
	    foreach Acc [split $Ligne " "] {
		if {$Acc == ""} {continue}
		lappend LesAccess [string trim $Acc]
	    }
	    break
	}
	if {[regexp -nocase {^AC  } $Ligne tmp]} {
	    regsub $tmp $Ligne "" Ligne
	    set Ligne [string trim $Ligne]
	    regsub -all ";" $Ligne "" Ligne 
	    foreach Acc [split $Ligne " "] {
		if {$Acc == ""} {continue}
		lappend LesAccess [string trim $Acc]
	    }
	}
    }
    return $LesAccess
}
proc BanqueIdDeLaPageInfo {PageInfo} {
    foreach Ligne [split $PageInfo "\n"] {
	if {[regexp -nocase {^ID   |^LOCUS  } $Ligne]} {
	    set Id ""
	    scan $Ligne "%s %s" EnteteID Id
	    return $Id
	} 
    }
    return
}
proc LesNonmRNAsDesAcNucs {LesAccess} {
    AskLesPagesInfos load $LesAccess genbankfull

    set LesNonmRNAs  {}
    foreach Access $LesAccess {
	set PageInfo [AskLesPagesInfos ask $Access]
	if {![EstCeUnmRNACompleteCDS_PageInfo $PageInfo]} {
	    lappend LesNonmRNAs $Access
	}
    }
    AskLesPagesInfos unset

    return $LesNonmRNAs
}
proc LesmRNAsDesAcNucs {LesAccess} {
    AskLesPagesInfos load $LesAccess genbankfull

    set LesmRNAs  {}
    foreach Access $LesAccess {
	set PageInfo [AskLesPagesInfos ask $Access]
	if {$PageInfo == ""} {continue}
	if {![EstCeUnmRNACompleteCDS_PageInfo $PageInfo]} {continue}
	lappend LesmRNAs $Access
    }
    AskLesPagesInfos unset

    return $LesmRNAs
}
proc AskLesPagesInfos args {
    global TabDesPagesInfosEnMemoire

    set FirstArg [lindex $args 0]

    if {$FirstArg == "set"} {
	set Access   [lindex $args 1]
	set PageInfo [lindex $args 2]
	set TabDesPagesInfosEnMemoire($Access) $PageInfo
	return
    }

    if {$FirstArg == "unset"} {
	if {[info exists TabDesPagesInfosEnMemoire]} {unset TabDesPagesInfosEnMemoire}
	return
    }
    if {$FirstArg == "ask"} {
	set Id [lindex $args 1]
	if {![info exists TabDesPagesInfosEnMemoire($Id)]} {return}
	return [set TabDesPagesInfosEnMemoire($Id)]
    }
    if {$FirstArg == "load"} {
	set LesId  [lindex $args 1]
	set Banque [lindex $args 2]
	
	foreach ListeId [LaListeSplittee $LesId "200"] {
	    TabLesPageInfos TabDesPagesInfosEnMemoire $ListeId $Banque
	}
	return
    }
    return
}
proc TabLesPageInfos {aTab {LesId {}} {Banque ""}} {
    upvar $aTab Tab
    if {$LesId == {}} {return}

    set LesIdModif {}
    foreach Id $LesId {
	set IdModif $Id
	regsub -nocase {^[^\:]+\:} $Id      "" IdModif
	regsub -nocase {\-[0-9]+$} $IdModif "" IdModif
	set IdModif [string toupper $IdModif]

	set TabTmp($Id,modif)    $IdModif
	set TabTmp($IdModif,ini) $Id

	lappend LesIdModif $IdModif
    }
    if {$Banque == ""} {set Banque "\{genbankfull protein est\}"}

    set Ids [join $LesIdModif "|"]
    set Ids "\($Ids\)"

    set Commande "[ProgPathway getz] \" \( \\\[$Banque-acc:$Ids\\\] \| \\\[$Banque-id:$Ids\\\] \) \" -e"

    catch {set GetzOut [eval exec $Commande]} Message
    if {![info exists GetzOut] || $GetzOut == ""} {return}

    set LaPageInfo   {}
    foreach Ligne [split $GetzOut "\n"] {
	lappend LaPageInfo $Ligne
	if {[string trim $Ligne] != "//"} {continue}

	set PageInfo [join $LaPageInfo "\n"]
	set     LesAccess [LesAccessDeLaPageInfo $PageInfo]
	lappend LesAccess [BanqueIdDeLaPageInfo  $PageInfo]
	foreach Access $LesAccess {
	    set Access [string toupper $Access]
	    set TabTmp($Access,PageInfo) $PageInfo
	}

	set LaPageInfo {}
    }
    foreach Id $LesId {
	set IdModif [set TabTmp($Id,modif)]
	if {![info exists TabTmp($IdModif,PageInfo)]} {continue}
	set Tab($Id) [set TabTmp($IdModif,PageInfo)]
    }
    return $aTab
}

proc PageInfo {Id {Banque ""} {Field ""}} {
    set Commande ""
    ###Si l'id est un contig humain#######
    if {[EstUnAccessDeHumanContig $Id]} {
	set Commande "[ProgPathway fetchcat] $Id"
	catch {set GetzOut [eval exec $Commande]} Message
	if {[info exists GetzOut] && $GetzOut != ""} {return $GetzOut}
    }
    set Commande ""
    ####Si l'id est un access de PDB########
    if {[EstUnAccessPDB $Id]} {
	if {[string equal -nocase $Field "seq"]} {
	    set TFA [TFADeLaBanqueBlast PDB $Id] 
	    if {$TFA != ""} {return $TFA}
	    regsub -nocase {_[a-z]*} $Id "" Id
	    set Commande "[ProgPathway getz] \"\\\[PDB-id:$Id\\\]\)\" -f seq -sf fasta" 
	} else {
	    regsub -nocase {_[a-z]*} $Id "" Id
	    set Commande "[ProgPathway getz] \"\\\[PDB-id:$Id\\\]\)\" -e"
	}
    }
    ####Si l'id est un access d'INTERPRO########
    if {[EstUnAccessINTERPRO $Id]} {
	set Commande "[ProgPathway getz] \"\\\[interpro-id:$Id\\\]\)\" -e" 
    }
    set IdCourt $Id
    regsub -nocase {^[0-9a-z_]*:} $Id "" IdCourt
    
    set Banque [string tolower $Banque]
    if {[string equal -nocase $Banque "genembl"]} {set Banque "genbankfull"}
    if {[string equal -nocase $Banque "genbank"]} {set Banque "genbankfull"}
    if {$Banque == "genbankfull"} {
	##Mise en attente si la banque est en mise  jour
	set FileUpdateGenbank "/gcg/Update/.genbank_update"
	if {[file exists $FileUpdateGenbank]} {
	    Espionne "SRS is updating the genbank database, please wait"
	    AttendreTantQueExiste "/gcg/Update/.genbank_update"
	    Espionne "The genbank database updating is done"
	}
    }

    set OptionFastaSiSeq "" 
    if {[regexp -nocase "seq" $Field]} {set OptionFastaSiSeq "-sf fasta"}

    ####Si l'id est un access de VARPLIC########
    if {[EstUnAccessVARSPLIC $Id]} {
	if {[string equal -nocase $Field "seq"]} {
	    set Commande "[ProgPathway getz] \"\\\[VARSPLIC-ID:$IdCourt]\)\" -e"
	} else {	
	    regsub {\-[0-9]+$} $IdCourt "" IdCourt
	    
	    if {$Field == ""} {
		set Commande "[ProgPathway getz] \"\\\[protein-acc:$IdCourt]\)\" -e $OptionFastaSiSeq" 
	    } else {
		set Commande "[ProgPathway getz] \"\\\[protein-acc:$IdCourt]\)\" -f \{$Field\} $OptionFastaSiSeq"
	    }
	}
    }

    if {$Commande != ""} {
	catch {set GetzOut [eval exec $Commande]} Message
	if {[info exists GetzOut] && $GetzOut != ""} {return $GetzOut}
    }

    set LesChamps [list "acc" "id" "prd" "gid" "gen"]

    ####Si l'id n'a pas de banque connu########
    set Commande ""
    if {$Banque == "" || [string equal -nocase $Banque "unknown"]} {
	foreach Champs $LesChamps {
	    set Commande ""
	    if {$Champs == "acc"} {set Commande "[ProgPathway getz] \"\(\\\[\{protein genbankfull est\}-acc:$IdCourt\\\]\)\" "}
	    if {$Champs == "id" } {set Commande "[ProgPathway getz] \"\(\\\[\{protein genbankfull est\}-id:$IdCourt\\\]\)\" "}
	    if {$Champs == "prd"} {set Commande "[ProgPathway getz] \"\(\\\[protein-prd:$IdCourt\\\]\)\" "}
	    if {$Champs == "gid"} {set Commande "[ProgPathway getz] \"\(\\\[\{genbankfull est\}-gid:$IdCourt\\\]\)\" "}
	    if {$Champs == "gen"} {set Commande "[ProgPathway getz] \"\(\\\[protein-gen:$IdCourt\\\]\)\" "}
	    if {$Commande == ""} {continue}

	    if {$Field == ""} {
		append Commande " -e $OptionFastaSiSeq"
	    } else {
		append Commande " -f \{$Field\} $OptionFastaSiSeq"
	    }

	    catch {set GetzOut [eval exec $Commande]} Message
	    if {[info exists GetzOut] && $GetzOut != ""} {return $GetzOut}
	}
    }

    ####Si l'id est un access ou id normal je cherche dans getz########
    set Commande ""
    foreach Champs $LesChamps {
	set Commande ""
	if {$Champs == "acc"} {set Commande "[ProgPathway getz] \"\(\\\[$Banque-acc:$IdCourt\\\]\)\" "}
	if {$Champs == "id" } {set Commande "[ProgPathway getz] \"\(\\\[$Banque-id:$IdCourt\\\]\)\" "}
	if {$Champs == "prd"} {set Commande "[ProgPathway getz] \"\(\\\[$Banque-prd:$IdCourt\\\]\)\" "}
	if {$Champs == "gid"} {set Commande "[ProgPathway getz] \"\(\\\[$Banque-gid:$IdCourt\\\]\)\" "}
	if {$Champs == "gen"} {set Commande "[ProgPathway getz] \"\(\\\[$Banque-gen:$IdCourt\\\]\)\" "}
	if {$Commande == ""} {continue}
	
	if {$Field == ""} {
	    append Commande " -e $OptionFastaSiSeq"
	} else {
	    append Commande " -f \{$Field\} $OptionFastaSiSeq"
	}
	
	catch {set GetzOut [eval exec $Commande]} Message
	if {[info exists GetzOut] && $GetzOut != ""} {return $GetzOut}
    }
    global tcl_platform
    if {[set tcl_platform(platform)] == "windows"} {
	set PageInfo [join [WgetzSurWeb $IdCourt] "\n"]
	if {$PageInfo != "" && ![string equal -nocase $PageInfo "ERREUR"]} {return $PageInfo}
    }
    return
}
proc PageInfoLaTotale {Id {Banque ""} {Field ""}} {
    set IdCourt $Id
    regsub -nocase {^[0-9a-z_]*:} $Id "" IdCourt

    set PageInfo [PageInfo $Id $Banque $Field]
    if {$PageInfo != ""} {return $PageInfo}
        
    ####Si l'id est un access ou id normal je cherche dans GCG########
    set Commande "[ProgPathway fetchcat] $Id"
    catch {set GetzOut [eval exec $Commande]} Message
    if {[info exists GetzOut] && $GetzOut != ""} {return $GetzOut}
    set Commande "[ProgPathway fetchcat] $IdCourt"
    catch {set GetzOut [eval exec $Commande]} Message
    if {[info exists GetzOut] && $GetzOut != ""} {return $GetzOut}

    ###Sinon je cherche dans les banques blast
    foreach Banque [list protein genembl est varsplic human genomes unigene] {
	set Commande "[ProgPathway fastacmd] -d $Banque -s $Id"
	catch {set GetzOut [eval exec $Commande]} Message
	if {[info exists GetzOut] && $GetzOut != ""} {return $GetzOut}
    }
    return
}
proc EstUnAccessDeHumanContig {AccessouId} {
    if {[regexp -nocase {^HS[0-9]{2}\:nt[0-9]{6}} $AccessouId]} {return 1}
    if {[regexp -nocase {^nt[0-9]{6}_[0-9]{,3}}   $AccessouId]} {return 1}
    if {[regexp -nocase {^nt[0-9]{6}$}            $AccessouId]} {return 1}
    return 0
}
proc EstUnAccessVARSPLIC {AccessouId} {
    if {[regexp -nocase {^varsplic\:}             $AccessouId]} {return 1}
    if {[regexp -nocase {^[0-9a-z]{6,8}\-[1-9]+} $AccessouId]} {return 1}
    return 0
}
proc EstUnAccessINTERPRO {Access} {
    if {[regexp -nocase {^ipr[0-9]{6}$} $Access]} {return 1}
    return 0
}
proc TFADeLaBanqueBlast {Banque Access} {
    set Banque [string tolower $Banque]
    if {[catch {set TFADeLaBanqueBlast [exec [ProgPathway fastacmd] -d $Banque -s $Access 2> /dev/null]} Message]} {return}
    return $TFADeLaBanqueBlast
}
