#################################################################################
# CADO4MI - CADO4MI: Computer Assisted Design of Oligonucleotide For MIcroarray #
#                                                                               #
# CADO4MI is a program to design oligonucleotide for microarrays or PCR.        #
#                                                                               #
# Copyright (C) 2006 Jean Muller (jean.muller@igbmc.u-strasbg.fr)               # 
#                                                                               #
# Please cite the following article:                                            #
#    Muller J et al                                                             #
#    Design and evaluation of Actichip,                                         #
#    a thematic microarray for the study of the actin cytoskeleton.             #
#    BMC Genomics 2007 Aug 29;8(1):294.                                         #
#                                                                               #
# This is part of CADO4MI source code.                                          #
#                                                                               #
# This program is free software; you can redistribute it and/or                 #
# modify it under the terms of the GNU General Public License                   # 
# as published by the Free Software Foundation; either version 2                # 
# of the License, or (at your option) any later version.                        #
#                                                                               #
# This program is distributed in the hope that it will be useful,               # 
# but WITHOUT ANY WARRANTY; without even the implied warranty of                #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
# GNU General Public License for more details.                                  #
#                                                                               #
# You should have received a copy of the GNU General Public License             #
# along with this program; if not, write to the Free Software                   #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                     #
#################################################################################

#########################################################
#                 Melting Temperature                   #
# 1.Hybridation et calcul du Melting TemperatureTm (Tm) #
# 2.Meilleur Melting temperature                        #
#                                                       #
#                 Guanosine and Cytosine content        # 
# 1.Hybridation et calcul du GC Content                 #
# 2.Meilleur pourcentage en GC                          #
#########################################################
############################################################
#   Hybridation et calcul de la Melting Temperature (Tm)   #
############################################################
proc CalculTmDeLaSeq {{Seq ""} {Methode ""} {Salt ""} {Formamide ""} {DNA ""}} {

    Wup "Calcul le Tm selon la formule voulue"
    Wup "avec M concentration en sel, F en Formamide"
    Wup "avec L la longueur de Seq, XG et XC la fraction molaire de GC dans la Seq"
    Wup "La concentration est exprimee en molaire. Pour le formamide certains considerent le %"
    Wup "16/03/2006 Mise a disposition des parametres par defaut et integration dans le Panel (acces au user)"
    Wup "17/03/2006 Reorganisation et optimisation des Tm"

    #Formule selon Wallace et al. 1979
    ##################################
    #Taille < 20 nts
    #Tm= 4*(G+C)+2*(A+T)
    #
    #Formule selon Meintoth and Wahl 1984
    #####################################
    #Taille > 20 nts
    #Tm ADN-ADN stability +
    #Tm= 81.5 + 16.6 log M + 41.0(%G+C) - 500/L - 0.62%F
    #
    #Tm ADN-ARN stability ++
    #Tm= 79.8 + 18.5 log M + 58.4(%G+C) + 11.8(%G+C)^2 - 820/L - 0.35%F
    #
    #Tm ARN-ARN stability +++
    #Tm= 79.8 + 18.5 log M + 58.4(%G+C) + 11.8(%G+C)^2 - 820/L - 0.50%F
    #
    #Nearest Neighbor Method
    ########################
    #Taille < 70 nts
    #Tm= 1000(DH)/     (DS)  + Rln([ADN]/4) -273.15) 
    #Correction pour le Na+
    #Tm= 1000(DH)/((A + DS)) + Rln([ADN]/4) -273.15 + 16.6log([Na+])
    #Correction comprise dans les parametres DH et DS 
    #R gaz cst = 1.987 cal.deg-1.mol-1, A correction for initiation -10.8
    #DNA = concentration des brins non auto complementaires 
    #        DNA/4 si meme concentration
    #(DNA1-DNA2)/2 si pas la meme concentraion
    #Ici DNA = 10E-6 M a voir 
    
    if {$Seq       == ""} {return ""}
    set  Seq [string toupper $Seq]

    if {$Methode   == ""} {
	if {!       [LesParametresDuDesign TmMethod          "Existe"]} {
	             LesParametresDuDesign TmMethod "DNADNA" "Positionne"
	}
	set Methode [LesParametresDuDesign TmMethod          "LaValeurMerci"]
    }

    if {$Methode   == "NearestNeighbor"} {
	if {$DNA   == ""} {
	    if {![LesParametresDuDesign TmConcentration            "Existe"]} {
		  LesParametresDuDesign TmConcentration "0.000001" "Positionne"
	    }
	    set DNA [LesParametresDuDesign TmConcentration         "LaValeurMerci"]
	}
	set Tm [NearestNeighborTm $Seq $DNA]
    } elseif {$Methode == "Wallace"} {
	#set xA [NombreDeBaseDe $Seq "A"]
	#set xT [NombreDeBaseDe $Seq "T"]
	#set xG [NombreDeBaseDe $Seq "G"]
	#set xC [NombreDeBaseDe $Seq "C"]
	set xAT [NombreDeBaseDe $Seq "A|T"]
	set xGC [NombreDeBaseDe $Seq "G|C"]
	set Tm  [expr {4*($xGC)+2*($xAT)}] 
    } else {
    	if {$Salt      == ""} {
	    if {![LesParametresDuDesign TmSalt       "Existe"]} {
		LesParametresDuDesign TmSalt "1.0" "Positionne"
	    }
	    set Salt [LesParametresDuDesign TmSalt "LaValeurMerci"]
	}
	if {$Formamide == ""} {
	    if {![LesParametresDuDesign TmFormamide        "Existe"]} {
		LesParametresDuDesign TmFormamide "12.3" "Positionne"
	    } 
	    set Formamide [LesParametresDuDesign TmFormamide "LaValeurMerci"]
	}
	#Shortest length of the two partners in duplex
	set TailleSeq [string length  $Seq] 
	set xGC       [NombreDeBaseDe $Seq "G|C"]
	set GCP       [expr {($xGC)/($TailleSeq*1.0)}]    

	#Autre formule pour le Salt
	#set NewSalt [expr $Salt/(1.0+(0.7*$Salt))]
	#set Tm4 [expr 81.5 + 16.6*log($NewSalt) + 41.0*($GCP) - 500/$TailleSeq - 0.62*$FormamidePercent]
	switch -exact -- $Methode {
	    "DNADNA"  {set Tm [expr {81.5 + 16.6*log($Salt) + 41.0*($GCP)                    - 500/$TailleSeq - 0.62*$Formamide}]}
	    "DNARNA"  {set Tm [expr {79.8 + 18.5*log($Salt) + 58.4*($GCP) + 11.8*pow($GCP,2) - 820/$TailleSeq - 0.50*$Formamide}]}
	    "RNARNA"  {set Tm [expr {79.8 + 18.5*log($Salt) + 58.4*($GCP) + 11.8*pow($GCP,2) - 820/$TailleSeq - 0.35*$Formamide}]}
	    "default" {set Tm ""}
	}
    }

    if {$Tm == ""} {
	return ""
    } else {
	return [format "%.1f" $Tm]
    }
}

proc NearestNeighborTm {Seq {DNA ""}} {
   
    Wup "Calcul du Tm avec la methode du Nearest Neighbor"
    Wup "Utilisation des parametres unifies du NN pour 1M NaCl"
    Wup "J. SantaLucia, 1998, PNAS, vol 95, pp 1460-1465"
    
    #Nearest Neighbor Method
    ########################
    #Tm= 1000(DH)/((A + DS)) + Rln([ADN]/4) -273.15 + 16.6log([Na+])
    #Correction pour le Na+
    #Tm= 1000 DH /     (DS)  + Rln([ADN]/4) -273.15) 
    #Correction comprise dans les parametres DH et DS 
    #R gaz cst = 1.987 cal.deg-1.mol-1, A correction for initiation -10.8
    #DNA = concentration des brins non auto complementaires 
    #        DNA/4 si meme concentration
    #(DNA1-DNA2)/2 si pas la meme concentraion
    #Ici DNA = 10E-6 M a voir 
    #Tm =(S)+Rln(C))-273.15-12.0log[Na+]
 
    set Seq [split $Seq ""]
    
    global Enthalpy
    global Entropy
    #global FreeNRJ
    
    set DeltaH   0.0
    set DeltaS   0.0
    #set DeltaG  0.0
    #set DNA      0.000001
    if {$DNA == ""} {set DNA [LesParametresDuDesign TmConcentration "LaValeurMerci"]}

    if {! [info exists Enthalpy] && ![info exists Entropy]} {
	#Parametres d initialisation
	set Enthalpy(A)  2.3
	set Enthalpy(T)  2.3
	set Enthalpy(G)  0.1
	set Enthalpy(C)  0.1
	#set Enthalpy(Y)  1.2
	#set Enthalpy(R)  1.2
	
	set Entropy(A)   4.1
	set Entropy(T)   4.1
	set Entropy(G)  -2.8
	set Entropy(C)  -2.8
	#set Entropy(Y)   0.6
	#set Entropy(R)   0.6
	
	#set FreeNRJ(A)   1.03
	#set FreeNRJ(T)   1.03
	#set FreeNRJ(G)   0.98
	#set FreeNRJ(C)   0.98

	#Parametres pour l entropie
	set Entropy(A,A)  -22.2
	set Entropy(A,T)  -20.4
	set Entropy(A,G)  -21.0
	set Entropy(A,C)  -22.4
	set Entropy(T,A)  -21.3
	set Entropy(T,T)  -22.2
	set Entropy(T,G)  -22.7
	set Entropy(T,C)  -22.2
	set Entropy(G,A)  -22.2
	set Entropy(G,T)  -22.4
	set Entropy(G,G)  -19.9
	set Entropy(G,C)  -24.4
	set Entropy(C,A)  -22.7
	set Entropy(C,T)  -21.0
	set Entropy(C,G)  -27.2
	set Entropy(C,C)  -19.9

	#R Purine on a pris la moyenne
	#set Entropy(A,R)  -21.6
	#set Entropy(T,R)  -22.0
	#set Entropy(G,R)  -21.0
	#set Entropy(C,R)  -24.9
	#set Entropy(R,A)  -22.2
	#set Entropy(R,T)  -21.4
	#set Entropy(R,G)  -20.4
	#set Entropy(R,C)  -23.4

	#Y Pyrimidine on a pris la moyenne
	#set Entropy(A,Y)  -21.4
	#set Entropy(T,Y)  -22.2
	#set Entropy(G,Y)  -21.4
	#set Entropy(C,Y)  -20.4
	#set Entropy(Y,A)  -22.0
	#set Entropy(Y,T)  -21.6
	#set Entropy(Y,G)  -24.9
	#set Entropy(Y,C)  -21.0

	#Parametres pour l enthalpie
	set Enthalpy(A,A)  -7.9
	set Enthalpy(A,T)  -7.2
	set Enthalpy(A,G)  -7.8
	set Enthalpy(A,C)  -8.4
	set Enthalpy(T,A)  -7.2
	set Enthalpy(T,T)  -7.9
	set Enthalpy(T,G)  -8.5
	set Enthalpy(T,C)  -8.2
	set Enthalpy(G,A)  -8.2
	set Enthalpy(G,T)  -8.4
	set Enthalpy(G,G)  -8.0
	set Enthalpy(G,C)  -9.8
	set Enthalpy(C,A)  -8.5
	set Enthalpy(C,T)  -7.8
	set Enthalpy(C,G) -10.6
	set Enthalpy(C,C)  -8.0

	#R Purine on a pris la moyenne
	#set Enthalpy(A,R)  -7.8
	#set Enthalpy(T,R)  -7.5
	#set Enthalpy(G,R)  -8.1
	#set Enthalpy(C,R)  -9.5
	#set Enthalpy(R,A)  -8.0
	#set Enthalpy(R,T)  -7.8
	#set Enthalpy(R,G)  -7.9
	#set Enthalpy(R,C)  -9.1
	
	#Y Pyrimidine on a pris la moyenne
	set Enthalpy(A,Y)  -7.8
	set Enthalpy(T,Y)  -8.0
	set Enthalpy(G,Y)  -9.1
	set Enthalpy(C,Y)  -7.9
	set Enthalpy(Y,A)  -7.8
	set Enthalpy(Y,T)  -8.0
	set Enthalpy(Y,G)  -9.5
	set Enthalpy(Y,C)  -8.1

	#Parametres pour l energie libre
	#set FreeNRJ(A,A)  -1
	#set FreeNRJ(A,T)  -0.88
	#set FreeNRJ(A,G)  -1.28
	#set FreeNRJ(A,C)  -1.44
	#set FreeNRJ(T,A)  -0.58
	#set FreeNRJ(T,T)  -1
	#set FreeNRJ(T,G)  -1.45
	#set FreeNRJ(T,C)  -1.3
	#set FreeNRJ(C,A)  -1.45
	#set FreeNRJ(C,T)  -1.28
	#set FreeNRJ(C,G)  -2.17
	#set FreeNRJ(C,C)  -1.84
	#set FreeNRJ(G,A)  -1.3
	#set FreeNRJ(G,T)  -1.44
	#set FreeNRJ(G,G)  -1.84
	#set FreeNRJ(G,C)  -2.24
    }
    set B1 0
    set B2 0
    foreach Base $Seq {
	set B2 $Base
	if {![info exists Enthalpy($Base)] | ![info exists Entropy($Base)]} {return "-1"}
	if {$B1=="0"} {
	    #Initialisation 5prime
	    set DeltaH  [expr {$DeltaH + [set Enthalpy($B2)]}]
	    set DeltaS  [expr {$DeltaS + [set Entropy($B2)]}]
	} else {
	    if {![info exists Enthalpy($B1,$B2)] | ![info exists Entropy($B1,$B2)]} {return "-1"}
	    set DeltaH  [expr {$DeltaH + [set Enthalpy($B1,$B2)]}]
	    set DeltaS  [expr {$DeltaS + [set Entropy($B1,$B2)]}]
	}
	set B1 $B2
    }
    #Initialisation 3prime
    set LastBase [string index $Seq end]
    set DeltaH   [expr {$DeltaH + [set Enthalpy($LastBase)]}]
    set DeltaS   [expr {$DeltaS + [set Entropy($LastBase)]}]

    return "[expr {( ( (1000.0*$DeltaH) / ($DeltaS+(1.9872*log($DNA/4.0)))) - 273.15)}]"
} 

proc NearestNeighborTm2 {Seq {Methode ""}} {
    
    Wup "NOT USED WARNING"
    Wup "Calcul du Tm avec la methode du Nearest Neighbor"
    Wup "Utilisation des parametres unifies du NN pour 1M NaCl"
    Wup "J. SantaLucia, 1998, PNAS, vol 95, pp 1460-1465"
    
    #Nearest Neighbor Method
    ########################
    #Tm= 1000(DH)/((A + DS)) + Rln([ADN]/4) -273.15 + 16.6log([Na+])
    #Correction pour le Na+
    #Tm= 1000 DH /     (DS)  + Rln([ADN]/4) -273.15) 
    #Correction comprise dans les parametres DH et DS 
    #R gaz cst = 1.987 cal.deg-1.mol-1, A correction for initiation -10.8
    #DNA = concentration des brins non auto complementaires 
    #        DNA/4 si meme concentration
    #(DNA1-DNA2)/2 si pas la meme concentraion
    #Ici DNA = 10E-6 M a voir 
 
    set Seq [split $Seq ""]
    
    set DeltaH   0.0
    set DeltaS   0.0
    #set DeltaG  0.0
    set DNA      0.000001

    if {$Methode =="" } {set Methode SantaLucia1997}

    ChargeLesParametresThermodynamiques Enthalpy Entropy FreeNRJ $Methode
    
    set B1 0
    set B2 0
    foreach Base $Seq {
	set B2 $Base
	if {![info exists Enthalpy($Base)] | ![info exists Entropy($Base)]} {return -1}
	if {$B1=="0"} {
	    set DeltaH  [expr $DeltaH + [set Enthalpy($B2)]]
	    set DeltaS  [expr $DeltaS + [set Entropy($B2)]]
	} else {
	    if {![info exists Enthalpy($B1,$B2)] | ![info exists Entropy($B1,$B2)]} {return -1}
	    set DeltaH  [expr $DeltaH + [set Enthalpy($B1,$B2)]]
	    set DeltaS  [expr $DeltaS + [set Entropy($B1,$B2)]]
	}
	set B1 $B2
    }
    set LastBase [string index $Seq end]
    set DeltaH   [expr $DeltaH + [set Enthalpy($LastBase)]]
    set DeltaS   [expr $DeltaS + [set Entropy($LastBase)]]
    
    return "[expr ( ( (1000.0*$DeltaH) / ($DeltaS+(1.9872*log($DNA/4.0)))) - 273.15)]"
} 

proc ChargeLesParametresThermodynamiques {aEnthalpy aEntropy aFreeNRJ {Quoi ""}} {

    Wup "NOT USED WARNING"
    Wup "Charge les energies selon le fichier choisi"

    global Enthalpy
    global Entropy
    global FreeNRJ
    global TabThermo

    if {$Quoi == ""} {
	set Quoi "SantaLucia1997"
    }

    if {[info exists Enthalpy] && [info exists TabThermo($Quoi)]} {
	upvar $aEnthalpy Enthalpy
	upvar $aEntropy  Entropy
	upvar $aFreeNRJ  FreeNRJ
	return ""
    }

    upvar $aEnthalpy Enthalpy
    upvar $aEntropy  Entropy
    upvar $aFreeNRJ  FreeNRJ

    if {[info exists Enthalpy]}  {array unset Enthalpy}
    if {[info exists Entropy]}   {array unset Entropy}
    if {[info exists FreeNRJ]}   {array unset FreeNRJ}
    if {[info exists TabThermo]} {array unset TabThermo}

    switch -exact -- $Quoi {
	"SantaLucia1997" {set Fichier "SantaLucia1997.nrj"}
	"0"              {set Fichier "SantaLucia1997.nrj"}
	"1"              {set Fichier "SantaLucia1997.nrj"}
	"default"        {set Fichier "$Quoi"}
    }

    if {! [file exists $Fichier]} {
	Espionne "No file for $Fichier"
	return ""
    }
    set F [open "$Fichier"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	set LesInfos [split $Ligne "\t"]
	set Base     [lindex $LesInfos 0]
	set Enthalpi [lindex $LesInfos 1]
	set Entropi  [lindex $LesInfos 2]
	#set FreeNRJ  [lindex $LesInfos 3]

	set Enthalpy($Base) $Enthalpi
	set Entropy($Base)  $Entropi
	#set FreeNRJ($Base)  $FreeNRJ
    }
    set TabThermo($Quoi) EstCharge

    close $F
    return ""
}

proc CreerLeFichierNRJSantaLucia {FichierNew} {

    Wup "NOT USED WARNING"

    #Parametres d initialisation
    set Enthalpy(A)  2.3
    set Enthalpy(T)  2.3
    set Enthalpy(G)  0.1
    set Enthalpy(C)  0.1
    set Enthalpy(Y)  1.2
    set Enthalpy(R)  1.2
    
    set Entropy(A)   4.1
    set Entropy(T)   4.1
    set Entropy(G)  -2.8
    set Entropy(C)  -2.8
    set Entropy(Y)   0.6
    set Entropy(R)   0.6
    
    set FreeNRJ(A)   1.03
    set FreeNRJ(T)   1.03
    set FreeNRJ(G)   0.98
    set FreeNRJ(C)   0.98
    set FreeNRJ(Y)   1.00
    set FreeNRJ(R)   1.00
    
    #Parametres pour l entropie
    set Entropy(A,A)  -22.2
    set Entropy(A,T)  -20.4
    set Entropy(A,G)  -21.0
    set Entropy(A,C)  -22.4
    set Entropy(T,A)  -21.3
    set Entropy(T,T)  -22.2
    set Entropy(T,G)  -22.7
    set Entropy(T,C)  -22.2
    set Entropy(G,A)  -22.2
    set Entropy(G,T)  -22.4
    set Entropy(G,G)  -19.9
    set Entropy(G,C)  -24.4
    set Entropy(C,A)  -22.7
    set Entropy(C,T)  -21.0
    set Entropy(C,G)  -27.2
    set Entropy(C,C)  -19.9
    
    #R Purine on a pris la moyenne
    set Entropy(A,R)  -21.6
    set Entropy(T,R)  -22.0
    set Entropy(G,R)  -21.0
    set Entropy(C,R)  -24.9
    set Entropy(R,A)  -22.2
    set Entropy(R,T)  -21.4
    set Entropy(R,G)  -20.4
    set Entropy(R,C)  -23.4
    
    #Y Pyrimidine on a pris la moyenne
    set Entropy(A,Y)  -21.4
    set Entropy(T,Y)  -22.2
    set Entropy(G,Y)  -21.4
    set Entropy(C,Y)  -20.4
    set Entropy(Y,A)  -22.0
    set Entropy(Y,T)  -21.6
    set Entropy(Y,G)  -24.9
    set Entropy(Y,C)  -21.0
    
    #Parametres pour l enthalpie
    set Enthalpy(A,A)  -7.9
    set Enthalpy(A,T)  -7.2
    set Enthalpy(A,G)  -7.8
    set Enthalpy(A,C)  -8.4
    set Enthalpy(T,A)  -7.2
    set Enthalpy(T,T)  -7.9
    set Enthalpy(T,G)  -8.5
    set Enthalpy(T,C)  -8.2
    set Enthalpy(G,A)  -8.2
    set Enthalpy(G,T)  -8.4
    set Enthalpy(G,G)  -8.0
    set Enthalpy(G,C)  -9.8
    set Enthalpy(C,A)  -8.5
    set Enthalpy(C,T)  -7.8
    set Enthalpy(C,G) -10.6
    set Enthalpy(C,C)  -8.0
    
    #R Purine on a pris la moyenne
    set Enthalpy(A,R)  -7.8
    set Enthalpy(T,R)  -7.5
    set Enthalpy(G,R)  -8.1
    set Enthalpy(C,R)  -9.5
    set Enthalpy(R,A)  -8.0
    set Enthalpy(R,T)  -7.8
    set Enthalpy(R,G)  -7.9
    set Enthalpy(R,C)  -9.1
    
    #Y Pyrimidine on a pris la moyenne
    set Enthalpy(A,Y)  -7.8
    set Enthalpy(T,Y)  -8.0
    set Enthalpy(G,Y)  -9.1
    set Enthalpy(C,Y)  -7.9
    set Enthalpy(Y,A)  -7.8
    set Enthalpy(Y,T)  -8.0
    set Enthalpy(Y,G)  -9.5
    set Enthalpy(Y,C)  -8.1
    
    #Parametres pour l energie libre
    set FreeNRJ(A,A)  -1
    set FreeNRJ(A,T)  -0.88
    set FreeNRJ(A,G)  -1.28
    set FreeNRJ(A,C)  -1.44
    set FreeNRJ(T,A)  -0.58
    set FreeNRJ(T,T)  -1
    set FreeNRJ(T,G)  -1.45
    set FreeNRJ(T,C)  -1.3
    set FreeNRJ(C,A)  -1.45
    set FreeNRJ(C,T)  -1.28
    set FreeNRJ(C,G)  -2.17
    set FreeNRJ(C,C)  -1.84
    set FreeNRJ(G,A)  -1.3
    set FreeNRJ(G,T)  -1.44
    set FreeNRJ(G,G)  -1.84
    set FreeNRJ(G,C)  -2.24
    
    #R Purine on a pris la moyenne
    set FreeNRJ(A,R)  -1.14
    set FreeNRJ(T,R)  -1.01
    set FreeNRJ(G,R)  -1.57
    set FreeNRJ(C,R)  -1.81
    set FreeNRJ(R,A)  -1.15
    set FreeNRJ(R,T)  -1.16
    set FreeNRJ(R,G)  -1.56
    set FreeNRJ(R,C)  -1.84
    
    #Y Pyrimidine on a pris la moyenne
    set FreeNRJ(A,Y)  -1.16
    set FreeNRJ(T,Y)  -1.15
    set FreeNRJ(G,Y)  -1.84
    set FreeNRJ(C,Y)  -1.56
    set FreeNRJ(Y,A)  -1.01
    set FreeNRJ(Y,T)  -1.14
    set FreeNRJ(Y,G)  -1.81
    set FreeNRJ(Y,C)  -1.57

    set LesBases [lsort -dictionary [array names Enthalpy]]
    foreach Base $LesBases {
	#Espionne "$Base\t[set Enthalpy($Base)]\t[set Entropy($Base)]\t[set FreeNRJ($Base)]"
	set Ligne "$Base\t[set Enthalpy($Base)]\t[set Entropy($Base)]\t[set FreeNRJ($Base)]"
	AppendAuFichier $FichierNew $Ligne
    }
    return ""
}

######################################
#    Meilleur Melting temperature    #
######################################
proc BestTmAnalysis {{Sequence ""}} {
    
    Wup "Permet de lancer les analyses pour determiner le meilleur Tm pour un groupe de sequence"

    if {$Sequence==""} {
	set LesQuerys [LesParametresDuDesign Query "LaValeurMerci"]
    } else {
	set LesQuerys $Sequence
    }
    if {$LesQuerys==""} {return ""}

    foreach Q $LesQuerys {
	ChargeTmCount $Q
    }

    CalculLaMoyenneTmCount

    IllustreTmCount
    #DechargeTmCount
    return ""
}

proc IsItDNA {Sequence} {

    Wup "Verifie si une sequence est nucleique ou pas"

    if {[regexp -nocase {[^ACTGURYKMSWBDHVN]} $Sequence]} {return "-1"} else {return "1"}
}

proc CalculLaMoyenneTmCount {} {

    Wup "Permet de calculer le moyenne des Tms"
    Wup "Somme de tous les Tms / Nombre de Valeurs mesurees"

    global TabDesTms

    if {! [info exists TabDesTms(TmTotal)] || ! [info exists TabDesTms(NbTm)]} {return ""}
    set TabDesTms(TmMoyenne) [format "%.1f" [expr {([set TabDesTms(TmTotal)]*1.0)/ [set TabDesTms(NbTm)]}]]
    return ""
}

proc LesResultatsTmCount {} {
    
    Wup "Permet de rendre les resultats du best Tm sous forme de liste"
    
    global TabDesTms

    if {! [info exists TabDesTms]} {return ""}

    set     LesResultats [array get TabDesTms]
    return $LesResultats
}

proc SauveTmCount {} {

    Wup "Permet de sauver les resultats du best Tm sous forme de liste"
        
    while {1} {
	set Texte "Choose a file name to save Tm calculation"	

	if {[BROWSEDDIR] != ""} {
	    set FichierSave [tk_getSaveFile -title "$Texte" -initialdir "[BROWSEDDIR]"]
	} else {
	    set FichierSave [tk_getSaveFile -title "$Texte" -initialdir "[HOMEDIR]"]
	}
	if {   $FichierSave == ""} {
	    if {! [OuiOuNon "Nothing selected\nWould you like to continue ?"]} {return ""}
	} else {
	    if {[file exists $FichierSave]} {
		file delete -force $FichierSave
	    }
	    set BrowsedDir [file dirname [lindex $FichierSave end]]
	    BROWSEDDIR $BrowsedDir
	    break
	}
    }
    
    set LesLignes ""

    set  TmMethode     [LesParametresDuDesign TmMethod    "LaValeurMerci"]
    if {$TmMethode     == ""} {set TmMethode "NearestNeighbor"}
    set  SlidingWindow [LesParametresDuDesign LengthOligo "LaValeurMerci"]
    if {$SlidingWindow == ""} {set SlidingWindow 60}
    set  Step          [LesParametresDuDesign Step        "LaValeurMerci"]
    if {$Step          == ""} {set Step 10}
    
    lappend LesLignes [join [list TmMethode     $TmMethode]     "\t"]
    lappend LesLignes [join [list SlidingWindow $SlidingWindow] "\t"]
    lappend LesLignes [join [list Step          $Step]          "\t"]

    #Traite les Resultats
    set LesResultats [LesResultatsTmCount]
    set TmpResultat  ""
    set XMax         ""
    set XMin         ""
    set TmMoyenne    ""
    set NbTmCalcules ""
    set NbSeq        ""
    foreach {a b} $LesResultats {
	switch $a {
	    "TmMax"     {set XMax         $b}
	    "TmMin"     {set XMin         $b}
	    "TmMoyenne" {set TmMoyenne    $b}
	    "TmTotal"   {}
	    "NbTm"      {set NbTmCalcules $b}
	    "NbSeq"     {set NbSeq        $b}
	    "default"   {lappend TmpResultat [list $a $b]}
	}
    }
    
    foreach {T R} [list NbSeq $NbSeq NbTm $NbTmCalcules TmMax $XMax TmMin $XMin] {
	lappend LesLignes [join [list $T $R] "\t"]
    }
    set LesResultats [lsort -increasing -real -index 0 $TmpResultat]
    foreach TiRe $LesResultats {
	set Ti [lindex $TiRe 0]
	set Re [lindex $TiRe 1]
	lappend LesLignes [join [list $Ti $Re] "\t"]
    }

    AppendAuFichier $FichierSave [join $LesLignes "\n"]

    return ""
}

proc ChargeTmCount {{PAB ""}} {
    
    Wup "Calcul le Tm pour les regions de PAB et comptabilise le nombre de fois qu il est rencontre"
    Wup "Stock les resultats dans TabDesTms"
    Wup "Utilisable pour un PAB ou une sequence"

    global TabDesTms
    
    if {$PAB==""} {return ""}
      
    #Attention il faut une query, sinon pas de taille ....
    set RepmRNA   "[file join [RepertoireDuGenome] mrnatfa_Homo_sapiens]"
    set Repnuctfa "[file join [RepertoireDuGenome] nuctfa]"

    set FichierQuery $PAB
    if {[regexp {/} $PAB]} {set PAB [file tail $PAB]}
    if {[file exists $FichierQuery]} {
	set FichierQuery $FichierQuery
    } elseif { [file exists "[file join $RepmRNA   $PAB]"]} {
	set FichierQuery    "[file join $RepmRNA   $PAB]"
    } elseif  {[file exists "[file join $Repnuctfa $PAB]"]} {
	set FichierQuery    "[file join $Repnuctfa $PAB]"
    } else {
	set  LaSeq      $PAB
	if {         $LaSeq ==""}   {AjouteAuLog "$FichierQuery no sequence available";return ""}
	if {[IsItDNA $LaSeq]=="-1"} {AjouteAuLog "$LaSeq is not a valid DNA sequence";return ""}
	set TailleQuery [string length $LaSeq]
    }
    
    if {! [info exists LaSeq]} {
	set  LaSeq      [QueLaSequenceDuFichierTFA $FichierQuery]
	if {$LaSeq==""} {AjouteAuLog "$FichierQuery no sequence available";return ""}
	set TailleQuery [string length $LaSeq]
    }
    set  LaSeq [split $LaSeq ""]

    if {! [info exists TabDesTms(TmMin)]}     {set TabDesTms(TmMin)  99999.9}
    if {! [info exists TabDesTms(TmMax)]}     {set TabDesTms(TmMax)      0.0}
    if {! [info exists TabDesTms(TmMoyenne)]} {set TabDesTms(TmMoyenne)  0.0} 
    if {! [info exists TabDesTms(TmTotal)]}   {set TabDesTms(TmTotal)    0.0} 
    if {! [info exists TabDesTms(NbTm)]}      {set TabDesTms(NbTm)         0} 
    if {! [info exists TabDesTms(NbSeq)]}     {set TabDesTms(NbSeq)        0} 
    
    incr TabDesTms(NbSeq)

    set nbTm         0
    set TmTotal    0.0
    set DebutSeq     1

    set  LengthOligo [LesParametresDuDesign LengthOligo "LaValeurMerci"]
    if {$LengthOligo==""} {set LengthOligo 60}

    set  Methode     [LesParametresDuDesign TmMethod "LaValeurMerci"]
    if {$Methode    ==""} {set Methode "NearestNeighbor"}

    set  Pas         [LesParametresDuDesign Step "LaValeurMerci"]
    if {$Pas        ==""} {set Pas 10}
 
    set Fin         $LengthOligo

    #Calcul des Tms sur une sliding window de taille LengthOligo et de pas j
    for {set Debut $DebutSeq} {$Fin <= $TailleQuery} {incr Debut $Pas} {
	set Fin [expr {$Debut + $LengthOligo - 1}] 
	if {![expr $Fin <= $TailleQuery]}          {break}

	set Seq [join [lrange $LaSeq [expr {$Debut - 1}] [expr {$Fin - 1}]] ""]
	if {[string length $Seq]<$LengthOligo}     {continue}

	#set Tm [format "%.1f" [CalculTmDeLaSeq $Seq "$Methode"]]
	set  Tm  [CalculTmDeLaSeq $Seq "$Methode"]
	if {$Tm=="-1"} {continue} elseif {$Tm==""} {return ""}	

	if {! [info exists TabDesTms($Tm)]} {
	    set  TabDesTms($Tm) 1
	} else {
	    incr TabDesTms($Tm)
	}
	incr nbTm
	set TmTotal [expr {$TmTotal + $Tm}]

	if {$Tm > [set TabDesTms(TmMax)]} {set TabDesTms(TmMax) $Tm}
	if {$Tm < [set TabDesTms(TmMin)]} {set TabDesTms(TmMin) $Tm}
    }
    
    #Pour la moyenne des Tms
    if {[set TabDesTms(TmTotal)]!="0.0"} {
	set  TabDesTms(TmTotal) [expr {[set TabDesTms(TmTotal)] + $TmTotal}]
    } else {
	set  TabDesTms(TmTotal) $TmTotal
    } 
    if {[set TabDesTms(NbTm)]!="0"} {
	set  TabDesTms(NbTm) [expr {[set TabDesTms(NbTm)] + $nbTm}]
    } else {
	set  TabDesTms(NbTm) $nbTm
    } 

    if {0} {
	#Moyenne des Tms
	Espionne "On est qd meme passe pas Tmmoyenne"
	set Moyennetmp [format "%.1f" [expr {($TmTotal*1.0)/$nbTm}]]
	if {[set TabDesTms(TmMoyenne)]!="0.0"} {
	    set  TabDesTms(TmMoyenne) [format "%.1f" [expr {(([set TabDesTms(TmMoyenne)] + $Moyennetmp)*1.0)/2.0}] ]
	} else {
	    set  TabDesTms(TmMoyenne) $Moyennetmp
	}
	
	#Espionne "$PAB -- Moyen $Moyennetmp Nb $nbTm [set TabDesTms(NbTm)] Total $TmTotal [set TabDesTms(TmTotal)]"
    }
    return ""
}

proc DechargeTmCount {} {

    Wup "Permet d effacer les resultats du best Tm"

    global TabDesTms

    if {[info exists TabDesTms]} {array unset TabDesTms}

    return ""
}

proc IllustreTmCount {} {
    
    Wup "Interface du BestTmCount"
    Wup "Montre le graphe cumule des valeurs de Tm pour les sequences ou boites choisies"
    Wup "Permet de choisir, de montrer et sauver des bornes"
    Wup "26-04-2005 Modif pour tenir compte de step, SlidingWindow et TmMethode modifiable et affichage de ces infos"
    Wup "31-01-2006 Modif pour tenir sauver les resultats"

    #Charge les Resultats
    set  LesResultats [LesResultatsTmCount]
    if {$LesResultats==""} {AjouteAuLog "Best Tm calculation: No results to display";return ""}

    set w "[NomDe fenetre]"
    if {[winfo exists $w]} {wm deiconify $w;raise $w;return ""} else {toplevel $w}
    
    wm geometry  $w +300+100
    wm title     $w "Best Tm analysis"
    #wm resizable $w false false
    wm protocol  $w WM_DELETE_WINDOW "DechargeTmCount;after 10 destroy $w"

    #Frame contenant le graphe et les scrollbar
    ###########################################
    set    FInfo "$w.frameinfo"
    frame $FInfo -borderwidth "3" -relief "groove"
    
    set Police         {-family "arial" -size 12 -weight bold}
    set LargeurFenetre [expr {[winfo screenwidth  .] * 2/3}]
    set HauteurFenetre [expr {[winfo screenheight .] * 1/2}]
    set XMilieu        [expr {$LargeurFenetre/2}]
    set BordureX       50.0
    set BordureY       50.0
    set Intervalle     10.0
    set EpaisseurBoite 15.0

    #Position du Graph content
    set XDebutGraph  [expr {0.0             + $BordureX}]
    set XFinGraph    [expr {$LargeurFenetre - $BordureX -20}]
    set YDebutGraph  [expr {0.0             + $BordureY}]
    set YFinGraph    [expr {$HauteurFenetre - $BordureY}]
    set TailleXGraph [expr {$XFinGraph - $XDebutGraph}]
    set TailleYGraph [expr {$YFinGraph - $YDebutGraph}]
    
    #Traite les Resultats
    set TmpResultat ""
    foreach {a b} $LesResultats {
	switch $a {
	    "TmMax"     {set XMax         $b}
	    "TmMin"     {set XMin         $b}
	    "TmMoyenne" {set TmMoyenne    $b}
	    "TmTotal"   {}
	    "NbTm"      {set NbTmCalcules $b}
	    "NbSeq"     {set NbSeq        $b}
	    "default"   {lappend TmpResultat [list $a $b]}
	}
    }
    set TailleEnX [expr {$XMax - $XMin}]

    set LesResultats [lsort -decreasing -integer -index 1 $TmpResultat]
    set YMax [lindex [lindex $LesResultats   0] 1]
    set YMin [lindex [lindex $LesResultats end] 1]
    set TailleEnY [expr {$YMax - $YMin}]
    
    #Nombre de pixel pour un degre ou un nombre
    set RapportEnX [expr {($TailleXGraph)/($TailleEnX)}] 
    set RapportEnY [expr {($TailleYGraph)/($TailleEnY)}]

    set K       "$FInfo.canvas"
    set ScroHor "$FInfo.xscroll"
    set ScroVer "$FInfo.yscroll"
    scrollbar $ScroVer -command "$K yview"
    scrollbar $ScroHor -command "$K xview" -orient "horizontal"

    #Canvas pour le graphe
    canvas $K -width $LargeurFenetre -height $HauteurFenetre -xscrollcommand "$ScroHor set" -yscrollcommand "$ScroVer set"
    $K configure -background   "grey" 
    #$K configure -scrollregion [list 0.0 0.0 $XFinGraph $YFinGraph]
    $K configure -scrollregion [list 0.0 0.0 $LargeurFenetre $HauteurFenetre]
    
    #Rectangle de masque pour les graphes
    $K create rectangle [expr {$XDebutGraph -2}] [expr {$YDebutGraph -2}] [expr {$XFinGraph +2}] [expr {$YFinGraph +2}] \
	    -fill    "gray97" \
	    -outline "gray97" \
	    -tags    [list "" "Masque"]

    set ColorLine "deepskyblue2"
    set n 0
    set LesResultats [lsort -increasing -real -index 0 $TmpResultat]
    foreach TmNb $LesResultats {
	scan $TmNb "%f %d" x y
	
	set X [expr {$XDebutGraph + ($x * $RapportEnX) - ($XMin * $RapportEnX)}]
	set Y [expr {$YFinGraph   - ($y * $RapportEnY) + ($YMin * $RapportEnY)}]

	incr n
	if {$n > 1} { 
	    $K create line [set Coordonnees(LastX)] [set Coordonnees(LastY)] $X $Y \
		    -tags [list "" "Tm $x Nb $y" "Affichable" "OligoGraph" "Tm" "TmLines"] \
		    -fill $ColorLine
	    set n 1
	}
	set Coordonnees(VraiY) $y
	set Coordonnees(LastX) $X
	set Coordonnees(LastY) $Y

	$K create oval [expr {$X-1}] [expr {$Y-1}] [expr {$X+1}] [expr {$Y+1}] \
		-width   "3" \
		-outline "darkviolet" \
		-fill    "darkviolet" \
		-tags    [list "" "Tm $x Nb $y" "Affichable" "OligoGraph" "Tm" "TmPlot"]
    }
    #La moyenne
    set XMoyenne [expr {$XDebutGraph + ($TmMoyenne * $RapportEnX) - ($XMin * $RapportEnX)}]
    $K create line $XMoyenne [expr {$YFinGraph +1}] $XMoyenne $YDebutGraph \
	    -tags [list "" "Mean $TmMoyenne" "Affichable" "OligoGraph" "Mean" "MeanLines"] \
	    -fill "red"
    $K create oval [expr {$XMoyenne -1}] [expr {$YDebutGraph -1}] [expr {$XMoyenne +1}] [expr {$YDebutGraph +1}] \
	    -width   "3" \
	    -outline "red" \
	    -fill    "red" \
	    -tags    [list "" "Mean" "Affichable" "OligoGraph" "Mean" "MeanPlot"]
    $K create oval [expr {$XMoyenne -1}] [expr {$YFinGraph -1}] [expr {$XMoyenne +1}] [expr {$YFinGraph +1}] \
	    -width   "3" \
	    -outline "red" \
	    -fill    "red" \
	    -tags    [list "" "Mean" "Affichable" "OligoGraph" "Mean" "MeanPlot"]
    $K create text $XMoyenne [expr {$YFinGraph +15}] \
	    -font    "$Police" \
	    -text    "[format "%.1f" $TmMoyenne]" \
	    -fill    "yellow" \
	    -anchor  "center" \
	    -tags    [list "" "Mean" "OligoGraph" "Mean"]

    #Axe des ordonnees
    $K create line $XDebutGraph $YFinGraph $XDebutGraph [expr {$YDebutGraph -20}] \
	    -arrow last \
	    -tags [list "" "Y (Nb) Axes" "Affichable" "Axes"] \
	    -width 2
    $K create text $XDebutGraph [expr {$YDebutGraph -25}] \
	    -text    "Count" \
	    -tags    [list "" "Y title" "Affichable" "Edges"] \
	    -anchor  "center"
    
    $K create text [expr {$XDebutGraph -10}] $YFinGraph \
	-text    "$YMin" \
	-tags    [list "" "Nb Min" "Affichable" "Edges"] \
	-anchor  "e" 
    $K create text [expr {$XDebutGraph -10}] $YDebutGraph \
	    -text    "$YMax" \
	    -tags    [list "" "Nb Max" "Affichable" "Edges"] \
	    -anchor  "e" 

    #Axe des abscisses
    $K create line $XDebutGraph $YFinGraph [expr {$XFinGraph +20}] $YFinGraph \
	    -arrow last \
	    -tags [list "" "X (Tm) Axes" "Affichable" "Axes"] \
	    -width 2
    $K create text [expr {$XFinGraph +25}] $YFinGraph \
	    -text    "Tm \C" \
	    -tags    [list "" "X title" "Affichable" "Edges"] \
	    -anchor  "w"

    $K create text $XDebutGraph [expr {$YFinGraph +15}] \
	    -text    "$XMin" \
	    -tags    [list "" "Tm Min" "Affichable" "Edges"] \
	    -anchor  "center" 
    $K create text $XFinGraph   [expr {$YFinGraph +15}] \
	    -text    "$XMax" \
	    -tags    [list "" "Tm Max" "Affichable" "Edges"] \
	    -anchor  "center" 

    #La fiche contenant les infos
    set  FicheInfo  ""
    set  TextFrame  "$w.textframe"
    set  Text       "$TextFrame.text"
    #set  ListBoxSBH "$TextFrame.xscroll"
    set  ListBoxSBV "$TextFrame.yscroll"

    frame  $TextFrame  -borderwidth "3"      -relief "groove"
    #scrollbar $ListBoxSBH -orient "horizontal" -command "$Text xview"
    scrollbar $ListBoxSBV -orient   "vertical" -command "$Text yview"
    #text $Text -height "4" -state "disabled" -wrap "word" -font "courier 10" -xscrollcommand "$ListBoxSBH set" -yscrollcommand "$ListBoxSBV set" 
    text $Text -height "4" -state "disabled" -wrap "word" -font "courier 10" -yscrollcommand "$ListBoxSBV set" 
	    
    set  Step          [LesParametresDuDesign Step        "LaValeurMerci"]
    if {$Step         ==""} {set Step 10}

    set  SlidingWindow [LesParametresDuDesign LengthOligo "LaValeurMerci"]
    if {$SlidingWindow==""} {set SlidingWindow 60}

    set  TmMethode     [LesParametresDuDesign TmMethod    "LaValeurMerci"]
    if {$TmMethode    ==""} {set TmMethode "NearestNeighbor"}

    set FicheInfo [list "$NbTmCalcules Tm values measured for $NbSeq sequence(s) submitted\n" "Tm Method: $TmMethode\n" "Sliding Window: $SlidingWindow nt(s)\n" "Step: $Step nt(s)"]
    foreach element $FicheInfo {
	$Text configure -state normal
	$Text insert    "end"  "$element"
	$Text configure -state disabled
    }

    #Les boutons
    set ButtonFrame    "$w.buttonframe"
    set  ActionButton  "$ButtonFrame.button1"
    set  DestroyButton "$ButtonFrame.button2"
    set  LCharpente1   "$ButtonFrame.lcharpente1" 
    set  LCharpente2   "$ButtonFrame.lcharpente2" 
    set  ZoneButton1   "$ButtonFrame.zbutton1"
    set  ZoneButton2   "$ButtonFrame.zbutton2"
    set  ZoneButton3   "$ButtonFrame.zbutton3"
    set  SaveButton    "$ButtonFrame.sbutton"

    frame  $ButtonFrame   -borderwidth "3"      -relief "groove"
    button $ActionButton  -background  "orange" -text "Graph Panel" 
      bind $ActionButton <1> "TmPanel $K"
    set Texte   [format      "%-10s" "Panel to change graph display (order,color)"]
    set_balloon $ActionButton "$Texte"

    button $DestroyButton -background "red"     -text "Close" -command "eval \[wm protocol $w WM_DELETE_WINDOW\]"
 
    button $ZoneButton1   -background "green"   -text "Area Draw / Clear / Save" 
      bind $ZoneButton1  <1> "MontreLesZonesDeTm $K $XDebutGraph $XFinGraph $YDebutGraph $YFinGraph $RapportEnX $XMin"
      bind $ZoneButton1  <2> "ClearLesZones      $K"
      bind $ZoneButton1  <3> "SauveLesZonesDeTm  $K"
    set Texte   [format      "%-10s" "Allow to show (defined), clear or save a Tm area"]
    set_balloon $ZoneButton1 "$Texte"

    button $ZoneButton2   -background "green"   -text "Range +/-5 / Reset"  
      bind $ZoneButton2  <1> "RangeTm $K $TmMoyenne 5 5 $XDebutGraph $XFinGraph $YDebutGraph $YFinGraph $RapportEnX $XMin"
      bind $ZoneButton2  <2> "ResetLesZonesDeTm  $K"
    set Texte   [format      "%-10s" "Allow to display 5C range, clear saved Tm area"]
    set_balloon $ZoneButton2 "$Texte"

    button $ZoneButton3   -background "yellow"  -text "Zoom In / Out / Reset"
      bind $ZoneButton3 <1> "zoom $K 1.25 1.25"
      bind $ZoneButton3 <2> "zoom $K 0.8 0.8"
      bind $ZoneButton3 <3> "ResetZoom $K"

    button $SaveButton    -background  "orange" -text "Save Calculation" 
      bind $SaveButton <1> "SauveTmCount"
    set Texte   [format      "%-10s" "Allow to save the Tm computation under tab file"]
    set_balloon $SaveButton "$Texte"
    
    label  $LCharpente1   -background "#808FFF" -relief "groove" 
    label  $LCharpente2   -background "#808FFF" -relief "groove" 

    pack $FInfo         -fill both              -expand true
    pack $ScroVer       -fill y    -side right 
    pack $ScroHor       -fill x    -side bottom
    pack $K             -fill both -side top    -expand true

    pack $ButtonFrame   -fill both -side top 
    pack $LCharpente1   -fill both -side left   -expand true
    pack $DestroyButton            -side left
    pack $ZoneButton3              -side left
    pack $ActionButton             -side left
    pack $ZoneButton1              -side left
    pack $ZoneButton2              -side left
    pack $SaveButton               -side left
    pack $LCharpente2   -fill both -side left   -expand true
    
    pack $TextFrame     -fill both -side top 
    pack $ListBoxSBV    -fill y    -side right  
    #pack $ListBoxSBH    -fill x    -side bottom
    pack $Text          -fill both -side top    -expand true

    Raiser $K "TmPlot"
    Lower  $K "Edges"
    Lower  $K "Axes"
    Lower  $K "Masque"
    
    $K bind "Affichable" <Enter> "AfficheInfoBulle $K %x %y Enter"
    $K bind "Affichable" <Leave> "AfficheInfoBulle $K %x %y Leave"

    bind $K <Control-1>  "PositionneLesZonesDeTm  $K %x %y $XDebutGraph $XFinGraph $YDebutGraph $YFinGraph $RapportEnX $XMin"
    bind $K <Shift-B1-Motion> "ViseurDeTm         $K %x %y $XDebutGraph $XFinGraph $YDebutGraph $YFinGraph $RapportEnX $XMin"
    bind $K <Shift-1>         "ViseurDeTm         $K %x %y $XDebutGraph $XFinGraph $YDebutGraph $YFinGraph $RapportEnX $XMin"
    #Binding pour permettre de zoomer en utilisant le 3eme bouton
    bind $K <3>               "zoomMark   $K %x %y"
    bind $K <B3-Motion>       "zoomStroke $K %x %y"
    bind $K <ButtonRelease-3> "zoomArea   $K %x %y"
    return "" 
}

proc TmPanel {LeCanvasPere} {

    Wup "Panel de choix d affichage et de couleur du graphes du meilleur Tm"

    set  LesGraphes [LesGraphesDeLOligo $LeCanvasPere]
    if {$LesGraphes==""} {return ""}

    set GrapheTopLevel "$LeCanvasPere.besttmgraphe"
    if {[winfo exists $GrapheTopLevel]} {
	wm deiconify $GrapheTopLevel
	raise $GrapheTopLevel
	return ""
    } else {toplevel $GrapheTopLevel}
    
    wm geometry  $GrapheTopLevel +300+100
    wm title     $GrapheTopLevel "Best Tm Selection"
    wm resizable $GrapheTopLevel false false
     
    set    FrameGraphe "$GrapheTopLevel.besttmframe"
    frame $FrameGraphe -borderwidth "3" -relief "groove"
    pack  $FrameGraphe -expand   "true" -side      "top" -fill "both"
    
    set NbGraph  [llength $LesGraphes]
    set NbRow    0
    set IndGraph 0
    foreach Graphe $LesGraphes {
	incr IndGraph
	incr NbRow
	set TagOrId $Graphe
	set Graphe  [string tolower $Graphe]
		
	set PetiteFrameGraphe "$FrameGraphe.f$Graphe"
	set LGraph      "$PetiteFrameGraphe.l$Graphe"
	set BGrOn       "$PetiteFrameGraphe.bgon$Graphe"
	set BGrOff      "$PetiteFrameGraphe.bgoff$Graphe"
	set BMoyOn      "$PetiteFrameGraphe.bmoyon$Graphe"
	set BMoyOff     "$PetiteFrameGraphe.bmoyoff$Graphe"
	set BColor1     "$PetiteFrameGraphe.bcolor1$Graphe"
	set BColor2     "$PetiteFrameGraphe.bcolor2$Graphe"
	
	frame $PetiteFrameGraphe                                 -borderwidth "0" -relief "groove" 
	label  $LGraph  -text "$TagOrId"               -padx "2" -borderwidth "3" -relief "groove" -background "orange"
	button $BGrOn   -text "On"    -anchor "center" -padx "3" -borderwidth "2" -relief "groove" \
		-command "Raiser $LeCanvasPere ${TagOrId}Lines"
	button $BGrOff  -text "Off"   -anchor "center" -padx "3" -borderwidth "2" -relief "groove" \
		-command "Lower  $LeCanvasPere ${TagOrId}Lines"
	button $BMoyOn  -text "On"    -anchor "center" -padx "3" -borderwidth "2" -relief "groove" \
		-command "Raiser $LeCanvasPere ${TagOrId}Plot"
	button $BMoyOff -text "Off"   -anchor "center" -padx "3" -borderwidth "2" -relief "groove" \
		-command "Lower  $LeCanvasPere ${TagOrId}Plot"
	button $BColor1 -text "Color" -anchor "center" -padx "3" -borderwidth "2" -relief "groove" -background deepskyblue2\
		-command "LePanneauDeCouleursDuGraph $BColor1 $LeCanvasPere ${TagOrId}Lines"
	button $BColor2 -text "Color" -anchor "center" -padx "3" -borderwidth "2" -relief "groove" -background deepskyblue2\
		-command "LePanneauDeCouleursDuGraph $BColor1 $LeCanvasPere ${TagOrId}Plot"
	
	grid $PetiteFrameGraphe             -sticky ewns 
	grid $LGraph  -row $NbRow -column 0 -sticky ewns -rowspan 2 
	grid $BGrOn   -row $NbRow -column 1 -sticky ewns
	grid $BGrOff  -row $NbRow -column 2 -sticky ewns 
	grid $BMoyOn  -row $NbRow -column 3 -sticky ewns
	grid $BMoyOff -row $NbRow -column 4 -sticky ewns
	incr NbRow
	grid $BColor1 -row $NbRow -column 1 -sticky ewns -columnspan 2
	grid $BColor2 -row $NbRow -column 3 -sticky ewns -columnspan 2

	if {$IndGraph==1} {
	    set DismissB   "$PetiteFrameGraphe.destroybutton"
	    set LDuGraph   "$PetiteFrameGraphe.ldugraph"
	    set LDuMoyenne "$PetiteFrameGraphe.ldumoyenne"
    
	    button $DismissB  -text "Exit"  -anchor "center" -padx "3" -background "red"     -borderwidth "2" -relief groove -activebackground "red" -overrelief "sunken" \
		    -command "destroy $GrapheTopLevel"
	    label $LDuGraph   -text "Lines" -anchor "center" -padx "2" -background "#808FFF" -borderwidth "3" -relief groove
	    label $LDuMoyenne -text "Plot"  -anchor "center" -padx "2" -background "#808FFF" -borderwidth "3" -relief groove
	    grid $DismissB    -row 0 -column 0 -sticky ewns 
	    grid $LDuGraph    -row 0 -column 1 -sticky ewns -columnspan 2
	    grid $LDuMoyenne  -row 0 -column 3 -sticky ewns -columnspan 2
	}

	if {$IndGraph==$NbGraph} {
	    set LAllDuGr  "$PetiteFrameGraphe.lalldugraph"
	    set BAllGrOn  "$PetiteFrameGraphe.ballgon"
	    set BAllGrOff "$PetiteFrameGraphe.ballgoff"
	   	    
	    label  $LAllDuGr  -text "Both" -anchor "center" -padx "2" -relief "groove" -background "green3" -borderwidth "3" 
	    button $BAllGrOn  -text "On"   -anchor "center" -padx "3" -relief "groove" -background "yellow" -borderwidth "2"\
		    -command "Raiser $LeCanvasPere OligoGraph"
	    button $BAllGrOff -text "Off"  -anchor "center" -padx "3" -relief "groove" -background "yellow" -borderwidth "2"\
		    -command "Lower  $LeCanvasPere OligoGraph"
	    incr NbRow

	    grid $LAllDuGr  -row $NbRow -column 0 -sticky ewns 
	    grid $BAllGrOn  -row $NbRow -column 1 -sticky ewns -columnspan 2
	    grid $BAllGrOff -row $NbRow -column 3 -sticky ewns -columnspan 2
	}
	grid columnconfigure $PetiteFrameGraphe 0 -weight 1
	grid columnconfigure $PetiteFrameGraphe 1 -weight 0
	grid columnconfigure $PetiteFrameGraphe 2 -weight 0
	grid columnconfigure $PetiteFrameGraphe 3 -weight 0
	grid columnconfigure $PetiteFrameGraphe 4 -weight 0
    }
    return ""
}

proc MontreLesZonesDeTm {K XDebut XFin YDebut {YFin ""} {OneNtPixel ""} {Minimum ""}} {
   
    Wup "Affiche les zones de Tm deja definies dans le Panel"
    Wup "Domaine de validite sur X (XDebut XFin) et valeur sur Y (YDebut YFin)"
    Wup "Minimum est necessaire car on ne commence pas a 0 ou 1 depend des valeurs observees"
    Wup "Attention le tableau est indexee par la melting temperature en degres"
    
    global LigneZone 

    ClearLesZones $K

    set Police   {-family "comic" -size 10 -weight bold}
    set Tm       [LesParametresDuDesign TmLimit "LaValeurMerci"]
    set Range    [LesParametresDuDesign TmRange "LaValeurMerci"]

    if {$Range==""} {set Range 0.0}
    if {$Tm   ==""} {return ""}

    set Borne1 [expr {$Tm + $Range}]
    set Borne2 [expr {$Tm - $Range}]
    set CoupleDeZone [list $Borne1 $Borne2]
    foreach Zone $CoupleDeZone {
	set x [expr {$XDebut + $Zone * $OneNtPixel - $Minimum * $OneNtPixel}]
	if {[info exists LigneZone($K,$Zone)]} {
	    catch {$K delete [set LigneZone($K,$Zone)]}
	    catch {$K delete [set LigneZone($K,$Zone,zone)]}
	    
	    unset LigneZone($K,$Zone)
	    if {[info exists LigneZone($K,$Zone,zone)]} {unset LigneZone($K,$Zone,zone)}
	}
	if {$x>$XFin} {
	    set x    $XFin 
	    set Zone [format "%.1f" [expr {(($x-$XDebut) / (1.0*$OneNtPixel)) +$Minimum}]]
	}    

	set Numero ""
	set Numero $Zone
	set LigneZone($K,$Zone,zone) [$K create text $x [expr {$YDebut-11}] \
		-font   "$Police" \
		-text   "$Numero" \
		-fill   "red" \
		-anchor "center" \
		-tags [list "" "Zonable" "Flag"]] 
	set LigneZone($K,$Zone)      [$K create line $x $YDebut $x $YFin \
		-fill "#808FFF" \
		-tags [list "" "[format "%.1f" $Zone]" "Affichable" "Zonable"]]
    }
    return ""
}

proc SauveLesZonesDeTm {K} {

    Wup "Sauvegarde les zones de Tm pour le design"

    set LesZones ""
    set LesIds   [$K find withtag {"Zonable" && "Affichable"}]
    foreach Id $LesIds {
	lappend LesZones [lindex [$K gettags $Id] 1]  
    }
    
    set LesZones [lsort -real -increasing $LesZones]
    set NbZones  [llength $LesZones]
    #Si impair
    if {$NbZones!="2"} {
	FaireLire "You need 2 Flags for a range of Tm"
    } else {
	LesParametresDuDesign TmLimit "Reset" 
	LesParametresDuDesign TmRange "Reset"

	if {[lindex $LesZones 0]> [lindex $LesZones 1]} {
	    set TmSup [lindex $LesZones 0]
	    set TmInf [lindex $LesZones 1]
	} else {
	    set TmSup [lindex $LesZones 1]
	    set TmInf [lindex $LesZones 0]
	}
	set TmToStore    [expr {($TmSup + $TmInf)/2.0}]
	set RangeToStore [expr {$TmSup - $TmToStore}]

	LesParametresDuDesign TmLimit $TmToStore    "Positionne"
	LesParametresDuDesign TmRange $RangeToStore "Positionne"
	AjouteAuLog "Tm: $TmToStore \C +/- $RangeToStore \C"
    }
    return ""
}

proc ResetLesZonesDeTm {K} {
    
    Wup "Unset les zones de Tm pour le design "

    LesParametresDuDesign TmLimit "Reset" 
    LesParametresDuDesign TmRange "Reset"

    AjouteAuLog "Own Tm limit and range reset to null."
    return ""    
}

proc RangeTm {K TmMoyenne X Y XDebut XFin YDebut {YFin ""} {OneNtPixel ""} {Minimum ""}} {

    Wup "Permet de dessiner les zones de Tm de + ou - 5 degres"
    
    global LigneZone 

    set Police   {-family "comic" -size 10 -weight bold}

    set XMoyenneMoins [expr {$TmMoyenne - 5.0}]
    set XMoyennePlus  [expr {$TmMoyenne + 5.0}]

    set Borne1 $XMoyenneMoins
    set Borne2 $XMoyennePlus
    set CoupleDeZone [list $Borne1 $Borne2]
    foreach Zone $CoupleDeZone {
	set x [expr {$XDebut + $Zone * $OneNtPixel - $Minimum * $OneNtPixel}]
	if {[info exists LigneZone($K,$Zone)]} {
	    catch {$K delete [set LigneZone($K,$Zone)]}
	    catch {$K delete [set LigneZone($K,$Zone,zone)]}
	    
	    unset LigneZone($K,$Zone)
	    if {[info exists LigneZone($K,$Zone,zone)]} {unset LigneZone($K,$Zone,zone)}
	}
	if {$x>$XFin} {
	    set x    $XFin 
	    set Zone [format "%.1f" [expr {(($x-$XDebut) / (1.0*$OneNtPixel)) +$Minimum}]]
	}    

	set Numero ""
	set Numero $Zone
	set LigneZone($K,$Zone,zone) [$K create text $x [expr {$YDebut-11}] \
		-font   "$Police" \
		-text   "$Numero" \
		-fill   "yellow" \
		-anchor "center" \
		-tags [list "" "Zonable" "Flag"]] 
	set LigneZone($K,$Zone)      [$K create line $x $YDebut $x $YFin \
		-fill   "green3" \
		-tags   [list "" "$Zone" "Affichable" "Zonable"]]
    }
    return ""
}

proc PositionneLesZonesDeTm {K X Y XDebut XFin YDebut {YFin ""} {OneNtPixel ""} {Minimum ""}} {

    Wup "Positionne une zone de validite de Tm pour le design"
    Wup "Domaine de validite sur X (XDebut XFin) et valeur sur Y (YDebut YFin)"
    Wup "Minimum est necessaire car on ne commence pas a 0 ou 1 depend des valeurs observees"
    Wup "Attention le tableau est indexee par la melting temperature en degres"

    global LigneZone 

    if {$YFin   ==""} {set YFin    [$YDebut + 1000]}
    if {$Minimum==""} {set Minimum 0}
    set Police {-family "Comic" -size 10 -weight bold}

    #Les positions sur le canvas
    set x [$K canvasx $X]
    set y [$K canvasy $Y]
    if {$x>$XFin || $x<$XDebut} {return ""}
   
    set X [format "%.1f" [expr {(($x-$XDebut) / (1.0*$OneNtPixel)) +$Minimum}]]

    set Numero ""
    if {[info exists LigneZone($K,$X)]} {
	catch {$K delete [set LigneZone($K,$X)]}
	catch {$K delete [set LigneZone($K,$X,zone)]}
	AfficheInfoBulle $K %x %y Leave
	unset LigneZone($K,$X)
	if {[info exists LigneZone($K,$X,zone)]} {unset LigneZone($K,$X,zone)}
	return ""
    }

    set Numero $X
    set LigneZone($K,$X,zone) [$K create text $x [expr {$YDebut-11}] \
	    -font   "$Police" \
	    -text   "$Numero" \
	    -fill   "yellow" \
	    -anchor "center" \
	    -tags   [list "" "Zonable" "Flag"]] 
    set LigneZone($K,$X)      [$K create line $x $YDebut $x $YFin \
	    -fill   "green3" \
	    -tags   [list "" "$X" "Affichable" "Zonable"]]

    return ""
}

proc ViseurDeTm {K X Y XDebut XFin YDebut {YFin ""} {OneNtPixel ""} {Minimum ""}} {
 
    Wup "Un viseur avec la melting temperature en degre sur le graph"
    Wup "Domaine de validite sur X (XDebut XFin) et valeur sur Y (YDebut YFin)"
    Wup "Minimum est necessaire car on ne commence pas a 0 ou 1 depend des valeurs observees"
    
    global LigneX OldX

    if {$YFin   ==""} {set YFin    [$YDebut + 1000]}
    if {$Minimum==""} {set Minimum 0}
    
    set Police {-family "Comic" -size 10 -weight bold}

    if {[info exists LigneX($K)]} {
	catch {$K delete [set LigneX($K)]}
	catch {$K delete [set LigneX($K,zone)]}
	unset LigneX($K)
	if {[info exists LigneX($K,$X,zone)]} {unset LigneX($K,$X,zone)}
	if {$X == [set OldX($K)] } {return ""}
    }
    set OldX($K) $X 

    set x [$K canvasx $X]
    if {$x>$XFin || $x<$XDebut} {return ""}

    set Tm [expr {(($x-$XDebut) / (1.0*$OneNtPixel)) + $Minimum}]

    set LigneX($K)      [$K create line $x $YDebut $x $YFin] 
    set LigneX($K,zone) [$K create text $x [expr {$YDebut-11}] \
	    -font   "$Police" \
	    -text   "[format "%.1f" $Tm]" \
	    -fill   "yellow" \
	    -anchor "center"] 
    return ""
}


#############################################
#    Hybridation et calcul du GC Content    #
#############################################
proc NombreDeBaseDe {Seq Quoi} {
    
    Wup "Rend le nombre de fois que Quoi apparait dans Seq"

    set Results [regexp -nocase -all "$Quoi" $Seq]
    return $Results
}

####################################
#    Meilleur pourcentage en GC    #
####################################
proc BestGCAnalysis {{Sequence ""}} {
    
    Wup "Permet de lancer les analyses pour determiner le meilleur GC pour un groupe de sequence"

    if {$Sequence==""} {
	set LesQuerys [LesParametresDuDesign Query "LaValeurMerci"]
    } else {
	set LesQuerys $Sequence
    }
    if {$LesQuerys==""} {return ""}

    foreach Q $LesQuerys {
	ChargeGCCount $Q
    }

    CalculLaMoyenneGCCount
    IllustreGCCount 
    #DechargeGCCount
    return ""
}

proc CalculLaMoyenneGCCount {} {

    Wup "Permet de calculer le moyenne des GCs"
    Wup "Somme de tous les GCs / Nombre de Valeurs mesurees"

    global TabDesGCs

    if {! [info exists TabDesGCs(GCTotal)]|| ! [info exists TabDesGCs(NbGC)]} {return ""}
    set TabDesGCs(GCMoyenne) [format "%.1f" [expr {([set TabDesGCs(GCTotal)]*1.0)/ [set TabDesGCs(NbGC)]}]]
    return ""
}

proc LesResultatsGCCount {} {

    Wup "Permet de rendre les resultats du best GC sous forme de liste"

    global TabDesGCs
    
    if {! [info exists TabDesGCs]} {return ""}

    set     LesResultats [array get TabDesGCs]
    return $LesResultats
}

proc SauveGCCount {} {

    Wup "Permet de sauver les resultats du best GC sous forme de liste"
        
    while {1} {
	set Texte "Choose a file name to save GC calculation"	

	if {[BROWSEDDIR] != ""} {
	    set FichierSave [tk_getSaveFile -title "$Texte" -initialdir "[BROWSEDDIR]"]
	} else {
	    set FichierSave [tk_getSaveFile -title "$Texte" -initialdir "[HOMEDIR]"]
	}
	if {   $FichierSave == ""} {
	    if {! [OuiOuNon "Nothing selected\nWould you like to continue ?"]} {return ""}
	} else {
	    if {[file exists $FichierSave]} {
		file delete -force $FichierSave
	    }
	    set BrowsedDir [file dirname [lindex $FichierSave end]]
	    BROWSEDDIR $BrowsedDir
	    break
	}
    }
    
    set LesLignes ""

    set  SlidingWindow [LesParametresDuDesign LengthOligo "LaValeurMerci"]
    if {$SlidingWindow == ""} {set SlidingWindow 60}
    set  Step          [LesParametresDuDesign Step        "LaValeurMerci"]
    if {$Step          == ""} {set Step 10}
    
    lappend LesLignes [join [list SlidingWindow $SlidingWindow] "\t"]
    lappend LesLignes [join [list Step          $Step]          "\t"]

    #Traite les Resultats
    set LesResultats [LesResultatsGCCount]
    set TmpResultat  ""
    set XMax         ""
    set XMin         ""
    set GCMoyenne    ""
    set NbGCCalcules ""
    set NbSeq        ""
    foreach {a b} $LesResultats {
	switch $a {
	    "GCMax"     {set XMax         $b}
	    "GCMin"     {set XMin         $b}
	    "GCMoyenne" {set GCMoyenne    $b}
	    "GCTotal"   {}
	    "NbGC"      {set NbGCCalcules $b}
	    "NbSeq"     {set NbSeq        $b}
	    "default"   {lappend TmpResultat [list $a $b]}
	}
    }
    
    foreach {T R} [list NbSeq $NbSeq NbGC $NbGCCalcules GCMax $XMax GCMin $XMin] {
	lappend LesLignes [join [list $T $R] "\t"]
    }
    set LesResultats [lsort -increasing -real -index 0 $TmpResultat]
    foreach TiRe $LesResultats {
	set Ti [lindex $TiRe 0]
	set Re [lindex $TiRe 1]
	lappend LesLignes [join [list $Ti $Re] "\t"]
    }

    AppendAuFichier $FichierSave [join $LesLignes "\n"]

    return ""
}

proc ChargeGCCount {{PAB ""}} {
    
    Wup "Calcul le GC pour les regions de la query"
    
    global TabDesGCs
    
    if {$PAB==""} {return ""}
      
    #Attention il faut une query, sinon pas de taille ....
    set RepmRNA   "[file join [RepertoireDuGenome] mrnatfa_Homo_sapiens]"
    set Repnuctfa "[file join [RepertoireDuGenome] nuctfa]"

    set FichierQuery $PAB
    if {[regexp {/} $PAB]} {set PAB [file tail $PAB]}
    if {[file exists $FichierQuery]} {
	set FichierQuery $FichierQuery
    } elseif { [file exists "[file join $RepmRNA $PAB]"]} {
	set FichierQuery    "[file join $RepmRNA $PAB]"
    } elseif  {[file exists "[file join $Repnuctfa $PAB]"]} {
	set FichierQuery    "[file join $Repnuctfa $PAB]"
    } else {
	set  LaSeq      $PAB
	if {         $LaSeq ==""}   {AjouteAuLog "$FichierQuery no sequence available";return ""}
	if {[IsItDNA $LaSeq]=="-1"} {AjouteAuLog "$LaSeq is not a valid DNA sequence";return ""}
	set TailleQuery [string length $LaSeq]
    }
    
    if {! [info exists LaSeq]} {
	set  LaSeq      [QueLaSequenceDuFichierTFA $FichierQuery]
	if {$LaSeq==""} {AjouteAuLog "$FichierQuery no sequence available";return ""}
	set TailleQuery [string length $LaSeq]
    }
    set  LaSeq [split $LaSeq ""]

    if {! [info exists TabDesGCs(GCMin)]}     {set TabDesGCs(GCMin)  99999.9}
    if {! [info exists TabDesGCs(GCMax)]}     {set TabDesGCs(GCMax)      0.0}
    if {! [info exists TabDesGCs(GCMoyenne)]} {set TabDesGCs(GCMoyenne)  0.0} 
    if {! [info exists TabDesGCs(GCTotal)]}   {set TabDesGCs(GCTotal)    0.0} 
    if {! [info exists TabDesGCs(NbGC)]}      {set TabDesGCs(NbGC)         0} 
    if {! [info exists TabDesGCs(NbSeq)]}     {set TabDesGCs(NbSeq)        0} 
    
    incr TabDesGCs(NbSeq)

    set nbGC         0
    set GCTotal    0.0
    set DebutSeq     1

    set  LengthOligo [LesParametresDuDesign LengthOligo "LaValeurMerci"]
    if {$LengthOligo==""} {set LengthOligo 60}

    set  Pas         [LesParametresDuDesign Step "LaValeurMerci"]
    if {$Pas        ==""} {set Pas 10}

    set Fin         $LengthOligo
    #Calcul de la moyenne sur une sliding window de taille Facteur et de pas j
    for {set Debut $DebutSeq} {$Fin <= $TailleQuery} {incr Debut $Pas} {
	set Fin [expr {$Debut + $LengthOligo - 1}] 
	if {![expr $Fin <= $TailleQuery]}          {break}
		
	set Seq [join [lrange $LaSeq [expr {$Debut - 1}] [expr {$Fin - 1}]] ""]
	set  TailleSeq [string length $Seq]
	if {$TailleSeq<$LengthOligo}     {continue}

	set GC [format "%.1f" [expr {([NombreDeBaseDe $Seq "G|C"]*100.0)/$TailleSeq}]]
	if {$GC=="-1"} {continue} elseif {$GC==""} {return ""}	

	if {! [info exists TabDesGCs($GC)]} {
	    set  TabDesGCs($GC) 1
	} else {
	    incr TabDesGCs($GC)
	}
	incr nbGC
	set GCTotal [expr {$GCTotal + $GC}]
	
	if {$GC > [set TabDesGCs(GCMax)]} {set TabDesGCs(GCMax) $GC}
	if {$GC < [set TabDesGCs(GCMin)]} {set TabDesGCs(GCMin) $GC}
    }

    #Pour la Moyenne des GCs
    if {[set TabDesGCs(GCTotal)]!="0.0"} {
	set  TabDesGCs(GCTotal) [expr {[set TabDesGCs(GCTotal)] + $GCTotal}]
    } else {
	set  TabDesGCs(GCTotal) $GCTotal
    } 
    if {[set TabDesGCs(NbGC)]!="0"} {
	set  TabDesGCs(NbGC) [expr {[set TabDesGCs(NbGC)] + $nbGC}]
    } else {
	set  TabDesGCs(NbGC) $nbGC
    } 

    if {0} {
	#Moyenne des GCs
	set Moyennetmp [format "%.1f" [expr {($GCTotal*1.0)/$nbGC}]]
	if {[set TabDesGCs(GCMoyenne)]!="0.0"} {
	    set TabDesGCs(GCMoyenne) [format "%.1f" [expr {(([set TabDesGCs(GCMoyenne)] + $Moyennetmp)*1.0)/2.0}] ]
	} else {
	    set TabDesGCs(GCMoyenne) $Moyennetmp
	}
    }

    return ""
}

proc DechargeGCCount {} {

    Wup "Permet d effacer les resultats du best GC"

    global TabDesGCs

    if {[info exists TabDesGCs]} {array unset TabDesGCs}

    return ""
}

proc IllustreGCCount {} {
    
    Wup "Interface du BestGCCount"
    Wup "Montre le graphe cumule des valeurs de GC pour les sequences ou boites choisies"
    Wup "Permet de choisir, de montrer et sauver des bornes"
    Wup "26-04-2005 Modif pour tenir compte de step et SlidingWindow modifiable et affichage de ces infos"
    Wup "31-01-2006 Modif pour tenir sauver les resultats"

    #Charge les Resultats
    set  LesResultats [LesResultatsGCCount]
    if {$LesResultats==""} {AjouteAuLog "Best GC calculation: No results to display";return ""}

    set w "[NomDe fenetre]"
    if {[winfo exists $w]} {wm deiconify $w;raise $w;return ""} else {toplevel $w}
    
    wm geometry  $w +300+100
    wm title     $w "Best GC analysis"
    #wm resizable $w false false
    wm protocol  $w WM_DELETE_WINDOW "DechargeGCCount;after 10 destroy $w"

    #Frame contenant le graphe et les scrollbar
    ###########################################
    set    FInfo "$w.frameinfo"
    frame $FInfo -borderwidth "3" -relief "groove"
    
    set Police         {-family "arial" -size 12 -weight bold}
    set LargeurFenetre [expr {[winfo screenwidth  .] * 2/3}]
    set HauteurFenetre [expr {[winfo screenheight .] * 1/2}]
    set XMilieu        [expr {$LargeurFenetre/2}]
    set BordureX       50.0
    set BordureY       50.0
    set Intervalle     10.0
    set EpaisseurBoite 15.0

    #Position du Graph content
    set XDebutGraph  [expr {0.0             + $BordureX}]
    set XFinGraph    [expr {$LargeurFenetre - $BordureX -20}]
    set YDebutGraph  [expr {0.0             + $BordureY}]
    set YFinGraph    [expr {$HauteurFenetre - $BordureY}]
    set TailleXGraph [expr {$XFinGraph - $XDebutGraph}]
    set TailleYGraph [expr {$YFinGraph - $YDebutGraph}]
    
    #Traite les Resultats
    set TmpResultat ""
    foreach {a b} $LesResultats {
	switch $a {
	    "GCMax"     {set XMax         $b}
	    "GCMin"     {set XMin         $b}
	    "GCMoyenne" {set GCMoyenne    $b}
	    "GCTotal"   {}
	    "NbGC"      {set NbGCCalcules $b}
	    "NbSeq"     {set NbSeq        $b}
	    "default"   {lappend TmpResultat [list $a $b]}
	}
    }
    set TailleEnX [expr {$XMax - $XMin}]

    set LesResultats [lsort -decreasing -integer -index 1 $TmpResultat]
    set YMax [lindex [lindex $LesResultats   0] 1]
    set YMin [lindex [lindex $LesResultats end] 1]
    set TailleEnY [expr {$YMax - $YMin}]
    
    #Nombre de pixel pour un pourcent de GC
    set RapportEnX [expr {($TailleXGraph)/($TailleEnX)}] 
    set RapportEnY [expr {($TailleYGraph)/($TailleEnY)}]
    
    set K       "$FInfo.canvas"
    set ScroHor "$FInfo.xscroll"
    set ScroVer "$FInfo.yscroll"
    scrollbar $ScroVer -command "$K yview"
    scrollbar $ScroHor -command "$K xview" -orient "horizontal"

    #Canvas pour le graphe
    canvas $K -width $LargeurFenetre -height $HauteurFenetre -xscrollcommand "$ScroHor set" -yscrollcommand "$ScroVer set"
    $K configure -background   "grey" 
    $K configure -scrollregion [list 0.0 0.0 $XFinGraph $YFinGraph]
    
    #Rectangle de masque pour les graphes
    $K create rectangle [expr {$XDebutGraph -2}] [expr {$YDebutGraph -2}] [expr {$XFinGraph +2}] [expr {$YFinGraph +2}] \
	-fill    "gray97" \
	-outline "gray97" \
	-tags    [list "" "Masque"]
    
    set ColorLine "deepskyblue2"
    set n 0
    set LesResultats [lsort -increasing -real -index 0 $TmpResultat]
    foreach GCNb $LesResultats {
	scan $GCNb "%f %d" x y
	
	set X [expr {$XDebutGraph + ($x * $RapportEnX) - ($XMin * $RapportEnX)}]
	set Y [expr {$YFinGraph   - ($y * $RapportEnY) + ($YMin * $RapportEnY)}]

	incr n
	if {$n > 1} { 
	    $K create line [set Coordonnees(LastX)] [set Coordonnees(LastY)] $X $Y \
		    -tags [list "" "GC $x Nb $y" "Affichable" "OligoGraph" "GC" "GCLines"] \
		    -fill $ColorLine
	    set n 1
	}
	set Coordonnees(VraiY) $y
	set Coordonnees(LastX) $X
	set Coordonnees(LastY) $Y

	$K create oval [expr {$X-1}] [expr {$Y-1}] [expr {$X+1}] [expr {$Y+1}] \
	    -width   "3" \
		-outline "darkviolet" \
		-fill    "darkviolet" \
		-tags    [list "" "GC $x Nb $y" "Affichable" "OligoGraph" "GC" "GCPlot"]
    }
    #La moyenne
    set XMoyenne [expr {$XDebutGraph + ($GCMoyenne * $RapportEnX) - ($XMin * $RapportEnX)}]
    $K create line $XMoyenne [expr {$YFinGraph +1}] $XMoyenne $YDebutGraph \
	-tags    [list "" "Mean $GCMoyenne" "Affichable" "OligoGraph" "Mean" "MeanLines"] \
	-fill    "red"
    $K create oval [expr {$XMoyenne -1}] [expr {$YDebutGraph -1}] [expr {$XMoyenne +1}] [expr {$YDebutGraph +1}] \
	-width   "3" \
	-outline "red" \
	-fill    "red" \
	-tags    [list "" "Mean" "Affichable" "OligoGraph" "Mean" "MeanPlot"]
    $K create oval [expr {$XMoyenne -1}] [expr {$YFinGraph -1}] [expr {$XMoyenne +1}] [expr {$YFinGraph +1}] \
	    -width   "3" \
	    -outline "red" \
	    -fill    "red" \
	    -tags    [list "" "Mean" "Affichable" "OligoGraph" "Mean" "MeanPlot"]
    $K create text $XMoyenne [expr {$YFinGraph +15}] \
	    -font    "$Police" \
	    -text    "[format "%.1f" $GCMoyenne]" \
	    -fill    "yellow" \
	    -anchor  "center" \
	    -tags [list "" "Mean" "OligoGraph" "Mean"]

    #Axe des ordonnees
    $K create line $XDebutGraph $YFinGraph $XDebutGraph [expr {$YDebutGraph -20}] \
	    -arrow last \
	    -tags [list "" "Y (Nb) Axes" "Affichable" "Axes"] \
	    -width 2
    $K create text $XDebutGraph [expr {$YDebutGraph -25}] \
	    -text    "Count" \
	    -tags    [list "" "Y title" "Affichable" "Edges"] \
	    -anchor  "center"
 
    $K create text [expr {$XDebutGraph -10}] $YFinGraph \
	    -text    "$YMin" \
	    -tags    [list "" "Nb Min" "Affichable" "Edges"] \
	    -anchor  "e" 
    $K create text [expr {$XDebutGraph -10}] $YDebutGraph \
	    -text    "$YMax" \
	    -tags    [list "" "Nb Max" "Affichable" "Edges"] \
	    -anchor  "e" 

    #Axe des abscisses
    $K create line $XDebutGraph $YFinGraph [expr {$XFinGraph +20}] $YFinGraph \
	    -arrow last \
	    -tags [list "" "X (GC) Axes" "Affichable" "Axes"] \
	    -width 2
    $K create text [expr {$XFinGraph +25}] $YFinGraph \
	    -text    "GC %" \
	    -tags    [list "" "X title" "Affichable" "Edges"] \
	    -anchor  "w"

    $K create text $XDebutGraph [expr {$YFinGraph +15}] \
	    -text    "$XMin" \
	    -tags    [list "" "GC Min" "Affichable" "Edges"] \
	    -anchor  "center" 
    $K create text $XFinGraph   [expr {$YFinGraph +15}] \
	    -text    "$XMax" \
	    -tags    [list "" "GC Max" "Affichable" "Edges"] \
	    -anchor  "center" 

    #La fiche contenant les infos
    set  FicheInfo  ""
    set  TextFrame  "$w.textframe"
    set  Text       "$TextFrame.text"
    #set  ListBoxSBH "$TextFrame.xscroll"
    set  ListBoxSBV "$TextFrame.yscroll"

    frame  $TextFrame  -borderwidth "3"      -relief "groove"
    #scrollbar $ListBoxSBH -orient "horizontal" -command "$Text xview"
    scrollbar $ListBoxSBV -orient   "vertical" -command "$Text yview"
    #text $Text -height "3" -state "disabled" -wrap "word" -font "courier 10" -xscrollcommand "$ListBoxSBH set" -yscrollcommand "$ListBoxSBV set" 
    text $Text -height "3" -state "disabled" -wrap "word" -font "courier 10" -yscrollcommand "$ListBoxSBV set" 
	    
    set  Step          [LesParametresDuDesign Step        "LaValeurMerci"]
    if {$Step         ==""} {set Step 10}

    set  SlidingWindow [LesParametresDuDesign LengthOligo "LaValeurMerci"]
    if {$SlidingWindow==""} {set SlidingWindow 60}

    set FicheInfo [list "$NbGCCalcules GC values measured for $NbSeq sequence(s) submitted\n" "Sliding Window: $SlidingWindow nt(s)\n" "Step: $Step nt(s)"]
    foreach element $FicheInfo {
	$Text configure -state normal
	$Text insert    "end"  "$element"
	$Text configure -state disabled
    }

    #Les boutons
    set ButtonFrame    "$w.buttonframe"
    set  ActionButton  "$ButtonFrame.button1"
    set  DestroyButton "$ButtonFrame.button2"
    set  LCharpente1   "$ButtonFrame.lcharpente1" 
    set  LCharpente2   "$ButtonFrame.lcharpente2" 
    set  ZoneButton1   "$ButtonFrame.zbutton1"
    set  ZoneButton2   "$ButtonFrame.zbutton2"
    set  ZoneButton3   "$ButtonFrame.zbutton3"
    set  SaveButton    "$ButtonFrame.sbutton"

    frame  $ButtonFrame   -borderwidth "3"      -relief "groove"
    button $ActionButton  -background  "orange" -text "Graph Panel" 
      bind $ActionButton <1> "GCPanel $K"
    set Texte   [format      "%-10s" "Panel to change graph display (order,color)"]
    set_balloon $ActionButton "$Texte"
    
    button $DestroyButton -background "red"     -text "Close" -command "eval \[wm protocol $w WM_DELETE_WINDOW\]"

    button $ZoneButton1   -background "green"   -text "Area Draw / Clear / Save" 
      bind $ZoneButton1  <1> "MontreLesZonesDeGC $K $XDebutGraph $XFinGraph $YDebutGraph $YFinGraph $RapportEnX $XMin"
      bind $ZoneButton1  <2> "ClearLesZones      $K"
      bind $ZoneButton1  <3> "SauveLesZonesDeGC  $K"
    set Texte   [format      "%-10s" "Allow to show (defined), clear or save a GC area"]
    set_balloon $ZoneButton1 "$Texte"

    button $ZoneButton2   -background "green"   -text "Reset" 
      bind $ZoneButton2  <1> "ResetLesZonesDeGC  $K"
    set Texte   [format      "%-10s" "Allow to clear saved GC area"]
    set_balloon $ZoneButton2 "$Texte"

    button $ZoneButton3   -background "yellow"  -text "Zoom In / Out / Reset"
      bind $ZoneButton3 <1> "zoom $K 1.25 1.25"
      bind $ZoneButton3 <2> "zoom $K 0.8 0.8"
      bind $ZoneButton3 <3> "ResetZoom $K"

    button $SaveButton    -background  "orange" -text "Save Calculation" 
      bind $SaveButton <1> "SauveGCCount"
    set Texte   [format      "%-10s" "Allow to save the GC computation under tab file"]
    set_balloon $SaveButton "$Texte"

    label  $LCharpente1   -background "#808FFF" -relief "groove" 
    label  $LCharpente2   -background "#808FFF" -relief "groove" 

    pack $FInfo         -fill both              -expand true
    pack $ScroVer       -fill y    -side right 
    pack $ScroHor       -fill x    -side bottom
    pack $K             -fill both -side top    -expand true

    pack $ButtonFrame   -fill both -side top 
    pack $LCharpente1   -fill both -side left   -expand true
    pack $DestroyButton            -side left
    pack $ZoneButton3              -side left
    pack $ActionButton             -side left
    pack $ZoneButton1              -side left
    pack $ZoneButton2              -side left
    pack $SaveButton               -side left
    pack $LCharpente2   -fill both -side left   -expand true
    
    pack $TextFrame     -fill both -side top 
    pack $ListBoxSBV    -fill y    -side right  
    #pack $ListBoxSBH    -fill x    -side bottom
    pack $Text          -fill both -side top    -expand true

    Raiser $K "GCPlot"
    Lower  $K "Edges"
    Lower  $K "Axes"
    Lower  $K "Masque"
    
    $K bind "Affichable" <Enter> "AfficheInfoBulle $K %x %y Enter"
    $K bind "Affichable" <Leave> "AfficheInfoBulle $K %x %y Leave"

    bind $K <Control-1>  "PositionneLesZonesDeGC  $K %x %y $XDebutGraph $XFinGraph $YDebutGraph $YFinGraph $RapportEnX $XMin"
    bind $K <Shift-B1-Motion> "ViseurDeGC         $K %x %y $XDebutGraph $XFinGraph $YDebutGraph $YFinGraph $RapportEnX $XMin"
    bind $K <Shift-1>         "ViseurDeGC         $K %x %y $XDebutGraph $XFinGraph $YDebutGraph $YFinGraph $RapportEnX $XMin"
    #Binding pour permettre de zoomer en utilisant le 3eme bouton
    bind $K <3>               "zoomMark   $K %x %y"
    bind $K <B3-Motion>       "zoomStroke $K %x %y"
    bind $K <ButtonRelease-3> "zoomArea   $K %x %y"
    return "" 
}

proc GCPanel {LeCanvasPere} {

    Wup "Panel de choix d affichage et de couleur du graphes du meilleur GC"

    set  LesTmpGraphes [LesGraphesDeLOligo $LeCanvasPere]
    if {$LesTmpGraphes==""} {return ""}

    set GrapheTopLevel "$LeCanvasPere.bestgcgraphe"
    if {[winfo exists $GrapheTopLevel]} {
	wm deiconify  $GrapheTopLevel
	raise $GrapheTopLevel
	return ""
    } else {toplevel $GrapheTopLevel}
    
    wm geometry  $GrapheTopLevel +300+100
    wm title     $GrapheTopLevel "Best GC Selection"
    wm resizable $GrapheTopLevel false false
     
    set    FrameGraphe "$GrapheTopLevel.bestgcframe"
    frame $FrameGraphe -borderwidth "3" -relief "groove"
    pack  $FrameGraphe -expand   "true" -side      "top" -fill "both"
    
    set NbGraph    [llength $LesTmpGraphes]
    set NbRow      0
    set IndGraph   0
    set LesGraphes ""
    foreach G $LesTmpGraphes {
	if {$G=="GC"} {
	    set LesGraphes [linsert $LesGraphes 0 $G]
	    continue
	}
	lappend LesGraphes $G
    }

    foreach Graphe $LesGraphes {
	incr IndGraph
	incr NbRow
	set TagOrId $Graphe
	set Graphe  [string tolower $Graphe]
		
	set PetiteFrameGraphe "$FrameGraphe.f$Graphe"
	set LGraph      "$PetiteFrameGraphe.l$Graphe"
	set BGrOn       "$PetiteFrameGraphe.bgon$Graphe"
	set BGrOff      "$PetiteFrameGraphe.bgoff$Graphe"
	set BMoyOn      "$PetiteFrameGraphe.bmoyon$Graphe"
	set BMoyOff     "$PetiteFrameGraphe.bmoyoff$Graphe"
	set BColor1     "$PetiteFrameGraphe.bcolor1$Graphe"
	set BColor2     "$PetiteFrameGraphe.bcolor2$Graphe"
	
	frame $PetiteFrameGraphe                                 -borderwidth "0" -relief "groove" 
	label  $LGraph  -text "$TagOrId"               -padx "2" -borderwidth "3" -relief "groove" -background "orange"
	button $BGrOn   -text "On"    -anchor "center" -padx "3" -borderwidth "2" -relief "groove" \
		-command "Raiser $LeCanvasPere ${TagOrId}Lines"
	button $BGrOff  -text "Off"   -anchor "center" -padx "3" -borderwidth "2" -relief "groove" \
		-command "Lower  $LeCanvasPere ${TagOrId}Lines"
	button $BMoyOn  -text "On"    -anchor "center" -padx "3" -borderwidth "2" -relief "groove" \
		-command "Raiser $LeCanvasPere ${TagOrId}Plot"
	button $BMoyOff -text "Off"   -anchor "center" -padx "3" -borderwidth "2" -relief "groove" \
		-command "Lower  $LeCanvasPere ${TagOrId}Plot"
	button $BColor1 -text "Color" -anchor "center" -padx "3" -borderwidth "2" -relief "groove" -background deepskyblue2\
		-command "LePanneauDeCouleursDuGraph $BColor1 $LeCanvasPere ${TagOrId}Lines"
	button $BColor2 -text "Color" -anchor "center" -padx "3" -borderwidth "2" -relief "groove" -background deepskyblue2\
		-command "LePanneauDeCouleursDuGraph $BColor1 $LeCanvasPere ${TagOrId}Plot"
	
	grid $PetiteFrameGraphe             -sticky ewns 
	grid $LGraph  -row $NbRow -column 0 -sticky ewns -rowspan 2 
	grid $BGrOn   -row $NbRow -column 1 -sticky ewns
	grid $BGrOff  -row $NbRow -column 2 -sticky ewns 
	grid $BMoyOn  -row $NbRow -column 3 -sticky ewns
	grid $BMoyOff -row $NbRow -column 4 -sticky ewns
	incr NbRow
	grid $BColor1 -row $NbRow -column 1 -sticky ewns -columnspan 2
	grid $BColor2 -row $NbRow -column 3 -sticky ewns -columnspan 2

	if {$IndGraph==1} {
	    set DismissB   "$PetiteFrameGraphe.destroybutton"
	    set LDuGraph   "$PetiteFrameGraphe.ldugraph"
	    set LDuMoyenne "$PetiteFrameGraphe.ldumoyenne"
    
	    button $DismissB  -text "Exit"  -anchor "center" -padx "3" -background "red"     -borderwidth "2" -relief groove -activebackground "red" -overrelief "sunken" \
		    -command "destroy $GrapheTopLevel"
	    label $LDuGraph   -text "Lines" -anchor "center" -padx "2" -background "#808FFF" -borderwidth "3" -relief groove
	    label $LDuMoyenne -text "Plot"  -anchor "center" -padx "2" -background "#808FFF" -borderwidth "3" -relief groove
	    grid $DismissB    -row 0 -column 0 -sticky ewns 
	    grid $LDuGraph    -row 0 -column 1 -sticky ewns -columnspan 2
	    grid $LDuMoyenne  -row 0 -column 3 -sticky ewns -columnspan 2
	}

	if {$IndGraph==$NbGraph} {
	    set LAllDuGr  "$PetiteFrameGraphe.lalldugraph"
	    set BAllGrOn  "$PetiteFrameGraphe.ballgon"
	    set BAllGrOff "$PetiteFrameGraphe.ballgoff"
	   	    
	    label  $LAllDuGr  -text "Both" -anchor "center" -padx "2" -relief "groove" -background "green3" -borderwidth "3" 
	    button $BAllGrOn  -text "On"   -anchor "center" -padx "3" -relief "groove" -background "yellow" -borderwidth "2"\
		    -command "Raiser $LeCanvasPere OligoGraph"
	    button $BAllGrOff -text "Off"  -anchor "center" -padx "3" -relief "groove" -background "yellow" -borderwidth "2"\
		    -command "Lower  $LeCanvasPere OligoGraph"
	    
	    incr NbRow

	    grid $LAllDuGr -row $NbRow -column 0 -sticky ewns 
	    grid $BAllGrOn    -row $NbRow -column 1 -sticky ewns -columnspan 2
	    grid $BAllGrOff   -row $NbRow -column 3 -sticky ewns -columnspan 2
	}
	grid columnconfigure $PetiteFrameGraphe 0 -weight 1
	grid columnconfigure $PetiteFrameGraphe 1 -weight 0
	grid columnconfigure $PetiteFrameGraphe 2 -weight 0
	grid columnconfigure $PetiteFrameGraphe 3 -weight 0
	grid columnconfigure $PetiteFrameGraphe 4 -weight 0
    }
    return ""
}

proc MontreLesZonesDeGC {K XDebut XFin YDebut {YFin ""} {OneNtPixel ""} {Minimum ""}} {
   
    Wup "Affiche les zones de GC deja definies dans le Panel"
    Wup "Domaine de validite sur X (XDebut XFin) et valeur sur Y (YDebut YFin)"
    Wup "Minimum est necessaire car on ne commence pas a 0 ou 1 depend des valeurs observees"
    Wup "Attention le tableau est indexee par la melting temperature en degres"
    
    global LigneZone 

    set Police   {-family "comic" -size 10 -weight bold}
    set GC1 [LesParametresDuDesign GCContentMinus "LaValeurMerci"]
    set GC2 [LesParametresDuDesign GCContentPlus  "LaValeurMerci"]

    if {$GC1=="" || $GC2==""} {return ""}

    set CoupleDeZone [list $GC1 $GC2]
    foreach Zone $CoupleDeZone {
	set x [expr {$XDebut + $Zone * $OneNtPixel - $Minimum * $OneNtPixel}]
	if {[info exists LigneZone($K,$Zone)]} {
	    catch {$K delete [set LigneZone($K,$Zone)]}
	    catch {$K delete [set LigneZone($K,$Zone,zone)]}
	    
	    unset LigneZone($K,$Zone)
	    if {[info exists LigneZone($K,$Zone,zone)]} {unset LigneZone($K,$Zone,zone)}
	}
	if {$x>$XFin} {
	    set x    $XFin 
	    set Zone [format "%.1f" [expr {(($x-$XDebut) / (1.0*$OneNtPixel)) +$Minimum}]]
	}    

	set Numero ""
	set Numero $Zone
	set LigneZone($K,$Zone,zone) [$K create text $x [expr {$YDebut-11}] \
		-font   "$Police" \
		-text   "$Numero" \
		-fill   "red" \
		-anchor "center" \
		-tags [list "" "Zonable" "Flag"]] 
	set LigneZone($K,$Zone)      [$K create line $x $YDebut $x $YFin \
		-fill "#808FFF" \
		-tags [list "" "[format "%.1f" $Zone]" "Affichable" "Zonable"]]
    }
    return ""
}

proc SauveLesZonesDeGC {K} {

    Wup "Sauvegarde les zones de GC pour le design"

    set LesZones ""
    set LesIds   [$K find withtag {"Zonable" && "Affichable"}]
    foreach Id $LesIds {
	lappend LesZones [lindex [$K gettags $Id] 1]  
    }
    
    set LesZones [lsort -real -increasing $LesZones]
    set NbZones  [llength $LesZones]
    #Si impair
    if {$NbZones!="2"} {
	FaireLire "You need 2 Flags for a range of GC"
    } else {
	LesParametresDuDesign GCContentMinus "Reset" 
	LesParametresDuDesign GCContentPlus  "Reset"

	if {[lindex $LesZones 0]> [lindex $LesZones 1]} {
	    set GCSup [lindex $LesZones 0]
	    set GCInf [lindex $LesZones 1]
	} else {
	    set GCSup [lindex $LesZones 1]
	    set GCInf [lindex $LesZones 0]
	}
	LesParametresDuDesign GCContentMinus "$GCInf" "Positionne"
	LesParametresDuDesign GCContentPlus  "$GCSup" "Positionne"

	AjouteAuLog "% GC lower: $GCInf %"
	AjouteAuLog "% GC upper: $GCSup %" 
    }
    return ""
}

proc ResetLesZonesDeGC {K} {
    
    Wup "Unset les zones de GC pour le design "

    LesParametresDuDesign GCContentMinus "Reset" 
    LesParametresDuDesign GCContentPlus  "Reset"

    AjouteAuLog "Own GC limit reset to null."
    return ""    
}

proc PositionneLesZonesDeGC {K X Y XDebut XFin YDebut {YFin ""} {OneNtPixel ""} {Minimum ""}} {

    Wup "Positionne une zone de validite de GC pour le design"
    Wup "Domaine de validite sur X (XDebut XFin) et valeur sur Y (YDebut YFin)"
    Wup "Minimum est necessaire car on ne commence pas a 0 ou 1 depend des valeurs observees"
    Wup "Attention le tableau est indexee par la melting temperature en degres"

    global LigneZone 

    if {$YFin   ==""} {set YFin    [$YDebut + 1000]}
    if {$Minimum==""} {set Minimum 0}
    set Police {-family "Comic" -size 10 -weight bold}

    #Les positions sur le canvas
    set x [$K canvasx $X]
    set y [$K canvasy $Y]
    if {$x>$XFin || $x<$XDebut} {return ""}

    set X [format "%.1f" [expr {(($x-$XDebut) / (1.0*$OneNtPixel)) +$Minimum}]]

    set Numero ""
    if {[info exists LigneZone($K,$X)]} {
	catch {$K delete [set LigneZone($K,$X)]}
	catch {$K delete [set LigneZone($K,$X,zone)]}
	AfficheInfoBulle $K %x %y Leave
	unset LigneZone($K,$X)
	if {[info exists LigneZone($K,$X,zone)]} {unset LigneZone($K,$X,zone)}
	return ""
    }

    set Numero $X
    set LigneZone($K,$X,zone) [$K create text $x [expr {$YDebut-11}] \
	    -font "$Police" \
	    -text "$Numero" \
	    -fill "yellow" \
	    -anchor center \
	    -tags [list "" "Zonable" "Flag"]] 
    set LigneZone($K,$X)      [$K create line $x $YDebut $x $YFin \
	    -fill "green3" \
	    -tags [list "" "$X" "Affichable" "Zonable"]]

    return ""
}

proc ViseurDeGC {K X Y XDebut XFin YDebut {YFin ""} {OneNtPixel ""} {Minimum ""}} {
 
    Wup "Un viseur avec la melting temperature en degre sur le graph"
    Wup "Domaine de validite sur X (XDebut XFin) et valeur sur Y (YDebut YFin)"
    Wup "Minimum est necessaire car on ne commence pas a 0 ou 1 depend des valeurs observees"
    
    global LigneX OldX

    if {$YFin   ==""} {set YFin    [$YDebut + 1000]}
    if {$Minimum==""} {set Minimum 0}
    
    set Police {-family "Comic" -size 10 -weight bold}

    if {[info exists LigneX($K)]} {
	catch {$K delete [set LigneX($K)]}
	catch {$K delete [set LigneX($K,zone)]}
	unset LigneX($K)
	if {[info exists LigneX($K,$X,zone)]} {unset LigneX($K,$X,zone)}
	if {$X == [set OldX($K)] } {return ""}
    }
    set OldX($K) $X 

    set x [$K canvasx $X]
    if {$x>$XFin || $x<$XDebut} {return ""}

    set GC [expr {(($x-$XDebut) / (1.0*$OneNtPixel)) + $Minimum}]

    set LigneX($K)      [$K create line $x $YDebut $x $YFin] 
    set LigneX($K,zone) [$K create text $x [expr {$YDebut-11}] \
	    -font   "$Police" \
	    -text   "[format "%.1f" $GC]" \
	    -fill   "yellow" \
	    -anchor "center"] 
    return ""
}
